
#include "applet.h"
#include <libgnomenu/monitor.h>
#include <libgnomenu/menubarbox.h>
#include <libgnomenu/globalmenu.h>
#include <libnotify/notify.h>
#include <glib/gi18n-lib.h>
#include <stdlib.h>
#include <string.h>
#include <libbonoboui.h>
#include <gtk/gtk.h>
#include <configmake.h>
#include <licenses.h>
#include <libwnck/libwnck.h>
#include <libgnomenu/menubar.h>
#include <libgnomenu/helper.h>
#include <gconf/gconf-client.h>
#include "switcher.h"
#include "gtkextra-gconfdialog.h"
#include "template.h"




static glong string_get_length (const char* self);
struct _GlobalMenuPanelAppletPrivate {
	GnomenuMonitor* monitor;
	GnomenuMenuBarBox* menubars;
	gboolean disposed;
	GnomenuGlobalMenu* main_menubar;
	Switcher* switcher;
	NotifyNotification* notify_no_plugin;
};

#define APPLET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_APPLET, GlobalMenuPanelAppletPrivate))
enum  {
	APPLET_DUMMY_PROPERTY
};
#define APPLET_APPLET_NAME _ ("globalmenu-panel-applet")
#define APPLET_APPLET_ICON "globalmenu"
#define APPLET_GCONF_SCHEMA_DIR "/schemas/apps/globalmenu-panel-applet/prefs"
static void _applet_on_about_clicked_bonobo_ui_verb_fn (BonoboUIComponent* component, void* user_data, const char* cname);
static void _applet_on_help_clicked_bonobo_ui_verb_fn (BonoboUIComponent* component, void* user_data, const char* cname);
static void _applet_on_preferences_clicked_bonobo_ui_verb_fn (BonoboUIComponent* component, void* user_data, const char* cname);
static void applet_real_dispose (GObject* base);
static void _applet_check_module_g_object_notify (GtkSettings* _sender, GParamSpec* pspec, gpointer self);
static void applet_real_screen_changed (GtkWidget* base, GdkScreen* previous_screen);
static void applet_on_window_changed (GlobalMenuPanelApplet* self, GnomenuMonitor* monitor, gulong prev_xid);
static void applet_real_change_background (PanelApplet* base, PanelAppletBackgroundType type, const GdkColor* color, GdkPixmap* pixmap);
static void applet_real_change_orient (PanelApplet* base, PanelAppletOrient orient);
static void __lambda4 (GConfClient* key, const char* value, GlobalMenuPanelApplet* self);
static void ___lambda4_gconf_client_value_changed (GConfClient* _sender, const char* key, void* value, gpointer self);
static void applet_check_module (GlobalMenuPanelApplet* self);
static void applet_real_realize (GtkWidget* base);
static void applet_get_prefs (GlobalMenuPanelApplet* self);
static void applet_on_about_clicked (BonoboUIComponent* component, void* user_data, const char* cname);
static void applet_on_help_clicked (BonoboUIComponent* component, void* user_data, const char* cname);
static void applet_on_preferences_clicked (BonoboUIComponent* component, void* user_data, const char* cname);
static gboolean applet_real_button_press_event (GtkWidget* base, const GdkEventButton* event);
static gboolean __lambda5 (GtkWidget* widget, const GdkEventButton* event, GlobalMenuPanelApplet* self);
static gboolean ___lambda5_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self);
static void applet_setup_popup_menu (GlobalMenuPanelApplet* self, GtkWidget* widget);
static void _applet_on_window_changed_gnomenu_monitor_window_changed (GnomenuMonitor* _sender, gulong old_xid, gpointer self);
static GObject * applet_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer applet_parent_class = NULL;
static void applet_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const char* APPLET_APPLET_AUTHORS[] = {"Coding:", "Yu Feng <rainwoodman@gmail.com>", "Mingxi Wu <fengshenx@gmail.com>", "bis0n.lives <bis0n.lives@gmail.com>", "Luca Viggiani <lviggiani@gmail.com>", "", "Packaging:", "sstasyuk <sstasyuk@gmail.com>", "David Watson <dwatson031@gmail.com>", "Valiant Wing <Valiant.Wing@gmail.com>", NULL};
static const char* APPLET_APPLET_ADOCUMENTERS[] = {"Pierre Slamich <pierre.slamich@gmail.com>", NULL};
static const BonoboUIVerb APPLET_verbs[] = {{"About", _applet_on_about_clicked_bonobo_ui_verb_fn, NULL}, {"Help", _applet_on_help_clicked_bonobo_ui_verb_fn, NULL}, {"Preferences", _applet_on_preferences_clicked_bonobo_ui_verb_fn, NULL}, {NULL, NULL, NULL}};


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static void _applet_on_about_clicked_bonobo_ui_verb_fn (BonoboUIComponent* component, void* user_data, const char* cname) {
	applet_on_about_clicked (component, user_data, cname);
}


static void _applet_on_help_clicked_bonobo_ui_verb_fn (BonoboUIComponent* component, void* user_data, const char* cname) {
	applet_on_help_clicked (component, user_data, cname);
}


static void _applet_on_preferences_clicked_bonobo_ui_verb_fn (BonoboUIComponent* component, void* user_data, const char* cname) {
	applet_on_preferences_clicked (component, user_data, cname);
}


GlobalMenuPanelApplet* applet_construct (GType object_type) {
	GlobalMenuPanelApplet * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


GlobalMenuPanelApplet* applet_new (void) {
	return applet_construct (TYPE_APPLET);
}


static void applet_real_dispose (GObject* base) {
	GlobalMenuPanelApplet * self;
	self = (GlobalMenuPanelApplet*) base;
	if (!self->priv->disposed) {
		self->priv->disposed = TRUE;
		panel_applet_set_background_widget ((PanelApplet*) self, NULL);
	}
	G_OBJECT_CLASS (applet_parent_class)->dispose ((GObject*) PANEL_APPLET (self));
}


static void _applet_check_module_g_object_notify (GtkSettings* _sender, GParamSpec* pspec, gpointer self) {
	applet_check_module (self);
}


static void applet_real_screen_changed (GtkWidget* base, GdkScreen* previous_screen) {
	GlobalMenuPanelApplet * self;
	GdkScreen* _tmp0;
	GdkScreen* screen;
	self = (GlobalMenuPanelApplet*) base;
	_tmp0 = NULL;
	screen = (_tmp0 = gtk_widget_get_screen ((GtkWidget*) self), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (previous_screen != NULL) {
		GtkSettings* _tmp1;
		GtkSettings* old_settings;
		guint _tmp2;
		_tmp1 = NULL;
		old_settings = (_tmp1 = gtk_settings_get_for_screen (previous_screen), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
		/* Work around an old vala bug on disconnecting signals
		 * perhaps already fixed but ...*/
		g_signal_handlers_disconnect_matched ((GObject*) old_settings, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("notify", G_TYPE_OBJECT, &_tmp2, NULL, FALSE), _tmp2), 0, NULL, (GCallback) _applet_check_module_g_object_notify, self);
		(old_settings == NULL) ? NULL : (old_settings = (g_object_unref (old_settings), NULL));
	}
	if (screen != NULL) {
		applet_check_module (self);
		gnomenu_monitor_set_screen (self->priv->monitor, screen);
		g_signal_connect_object ((GObject*) gtk_widget_get_settings ((GtkWidget*) self), "notify::gtk-modules", (GCallback) _applet_check_module_g_object_notify, self, 0);
	}
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


static void applet_on_window_changed (GlobalMenuPanelApplet* self, GnomenuMonitor* monitor, gulong prev_xid) {
	gulong xid;
	WnckWindow* _tmp0;
	WnckWindow* window;
	g_return_if_fail (self != NULL);
	g_return_if_fail (monitor != NULL);
	xid = gnomenu_monitor_get_current_xid (monitor);
	_tmp0 = NULL;
	window = (_tmp0 = wnck_window_get (xid), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (WNCK_IS_WINDOW (window)) {
		switcher_set_current_window (self->priv->switcher, window);
	}
	gnomenu_global_menu_switch_to (self->priv->main_menubar, xid);
	(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
}


static void applet_real_change_background (PanelApplet* base, PanelAppletBackgroundType type, const GdkColor* color, GdkPixmap* pixmap) {
	GlobalMenuPanelApplet * self;
	GnomenuBackground* bg;
	self = (GlobalMenuPanelApplet*) base;
	bg = gnomenu_background_new ();
	switch (type) {
		case PANEL_NO_BACKGROUND:
		{
			break;
		}
		case PANEL_COLOR_BACKGROUND:
		{
			bg->type = GNOMENU_BACKGROUND_TYPE_COLOR;
			bg->color = *color;
			break;
		}
		case PANEL_PIXMAP_BACKGROUND:
		{
			GdkPixmap* _tmp1;
			GdkPixmap* _tmp0;
			bg->type = GNOMENU_BACKGROUND_TYPE_PIXMAP;
			_tmp1 = NULL;
			_tmp0 = NULL;
			bg->pixmap = (_tmp1 = (_tmp0 = pixmap, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (bg->pixmap == NULL) ? NULL : (bg->pixmap = (g_object_unref (bg->pixmap), NULL)), _tmp1);
			break;
		}
	}
	gnomenu_menu_bar_box_set_background (self->priv->menubars, bg);
	(bg == NULL) ? NULL : (bg = (gnomenu_background_unref (bg), NULL));
}


static void applet_real_change_orient (PanelApplet* base, PanelAppletOrient orient) {
	GlobalMenuPanelApplet * self;
	self = (GlobalMenuPanelApplet*) base;
	switch (orient) {
		case PANEL_APPLET_ORIENT_UP:
		{
			gnomenu_menu_bar_box_set_gravity (self->priv->menubars, GNOMENU_GRAVITY_DOWN);
			gnomenu_menu_bar_box_set_pack_direction (self->priv->menubars, GTK_PACK_DIRECTION_LTR);
			gnomenu_menu_bar_box_set_child_pack_direction (self->priv->menubars, GTK_PACK_DIRECTION_LTR);
			break;
		}
		case PANEL_APPLET_ORIENT_DOWN:
		{
			gnomenu_menu_bar_box_set_gravity (self->priv->menubars, GNOMENU_GRAVITY_DOWN);
			gnomenu_menu_bar_box_set_pack_direction (self->priv->menubars, GTK_PACK_DIRECTION_LTR);
			gnomenu_menu_bar_box_set_child_pack_direction (self->priv->menubars, GTK_PACK_DIRECTION_LTR);
			break;
		}
		case PANEL_APPLET_ORIENT_LEFT:
		{
			gnomenu_menu_bar_box_set_gravity (self->priv->menubars, GNOMENU_GRAVITY_LEFT);
			gnomenu_menu_bar_box_set_pack_direction (self->priv->menubars, GTK_PACK_DIRECTION_TTB);
			gnomenu_menu_bar_box_set_child_pack_direction (self->priv->menubars, GTK_PACK_DIRECTION_TTB);
			break;
		}
		case PANEL_APPLET_ORIENT_RIGHT:
		{
			gnomenu_menu_bar_box_set_gravity (self->priv->menubars, GNOMENU_GRAVITY_RIGHT);
			gnomenu_menu_bar_box_set_pack_direction (self->priv->menubars, GTK_PACK_DIRECTION_TTB);
			gnomenu_menu_bar_box_set_child_pack_direction (self->priv->menubars, GTK_PACK_DIRECTION_TTB);
			break;
		}
	}
}


static void __lambda4 (GConfClient* key, const char* value, GlobalMenuPanelApplet* self) {
	g_return_if_fail (key != NULL);
	g_return_if_fail (value != NULL);
	applet_get_prefs (self);
}


static void ___lambda4_gconf_client_value_changed (GConfClient* _sender, const char* key, void* value, gpointer self) {
	__lambda4 (_sender, key, self);
}


void applet_init (GlobalMenuPanelApplet* self) {
	GError * inner_error;
	GConfClient* _tmp0;
	GConfClient* client;
	char* applet_menu_xml_template;
	char** _tmp5;
	gint subs_size;
	gint subs_length1;
	char** _tmp4;
	const char* _tmp3;
	const char* _tmp2;
	const char* _tmp1;
	char** subs;
	char* applet_menu_xml;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	{
		panel_applet_add_preferences ((PanelApplet*) self, APPLET_GCONF_SCHEMA_DIR, &inner_error);
		if (inner_error != NULL) {
			goto __catch0_g_error;
			goto __finally0;
		}
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_warning ("applet.vala:169: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally0:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	_tmp0 = NULL;
	client = (_tmp0 = gconf_client_get_default (), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (client != NULL) {
		g_signal_connect (client, "value-changed", (GCallback) ___lambda4_gconf_client_value_changed, self);
	}
	applet_menu_xml_template = g_strdup ("\n<popup name=\"button3\">\n\t<menuitem name=\"Preferences\" \n\t\tverb=\"Preferences\" \n\t\t_label=\"@Preferences@\"\n\t\tpixtype=\"stock\" \n\t\tpixname=\"gtk-preferences\"/>\n\t<menuitem name=\"Help\" \n\t\tverb=\"Help\" \n\t\t_label=\"@Help@\"\n\t\tpixtype=\"stock\" \n\t\tpixname=\"gtk-help\"/>\n\t<menuitem name=\"About\" \n\t\tverb=\"About\" \n\t\t_label=\"@About@\"\n\t\tpixtype=\"stock\" \n\t\tpixname=\"gtk-about\"/>\n</popup>\n\t\t");
	_tmp5 = NULL;
	_tmp4 = NULL;
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	subs = (_tmp5 = (_tmp4 = g_new0 (char*, 6 + 1), _tmp4[0] = g_strdup ("@Preferences@"), _tmp4[1] = (_tmp1 = _ ("_Preferences"), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), _tmp4[2] = g_strdup ("@Help@"), _tmp4[3] = (_tmp2 = _ ("_Help"), (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), _tmp4[4] = g_strdup ("@About@"), _tmp4[5] = (_tmp3 = _ ("_About"), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), _tmp4), subs_length1 = 6, subs_size = subs_length1, _tmp5);
	applet_menu_xml = template_replace (applet_menu_xml_template, subs, subs_length1);
	panel_applet_setup_menu ((PanelApplet*) self, applet_menu_xml, APPLET_verbs, self);
	applet_get_prefs (self);
	(client == NULL) ? NULL : (client = (g_object_unref (client), NULL));
	applet_menu_xml_template = (g_free (applet_menu_xml_template), NULL);
	subs = (_vala_array_free (subs, subs_length1, (GDestroyNotify) g_free), NULL);
	applet_menu_xml = (g_free (applet_menu_xml), NULL);
}


static void applet_check_module (GlobalMenuPanelApplet* self) {
	GError * inner_error;
	GtkSettings* _tmp0;
	GtkSettings* settings;
	char* _tmp1;
	char* modules;
	char* _tmp2;
	gboolean _tmp3;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	_tmp0 = NULL;
	settings = (_tmp0 = gtk_widget_get_settings ((GtkWidget*) self), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (settings == NULL) {
		(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
		return;
	}
	_tmp1 = NULL;
	modules = (g_object_get (settings, "gtk-modules", &_tmp1, NULL), _tmp1);
	_tmp2 = NULL;
	modules = (_tmp2 = g_strconcat (modules, g_getenv ("GTK_MODULES"), NULL), modules = (g_free (modules), NULL), _tmp2);
	_tmp3 = FALSE;
	if (modules != NULL) {
		_tmp3 = strstr (modules, "globalmenu") != NULL;
	} else {
		_tmp3 = FALSE;
	}
	if (_tmp3) {
		(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
		modules = (g_free (modules), NULL);
		return;
	}
	if (self->priv->notify_no_plugin == NULL) {
		NotifyNotification* _tmp6;
		char* _tmp5;
		char* _tmp4;
		if (!notify_is_initted ()) {
			if (!notify_init (APPLET_APPLET_NAME)) {
				g_critical ("applet.vala:225: libnotify is not initialized");
				(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
				modules = (g_free (modules), NULL);
				return;
			}
		}
		_tmp6 = NULL;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->notify_no_plugin = (_tmp6 = notify_notification_new (_ ("No Global Menu?"), _tmp5 = g_strconcat (_tmp4 = g_strconcat (_ ("The Global Menu Plugin is not enabled on this desktop."), _ ("Enable the plugin by accessing the preferences dialog via a right-click,"), NULL), _ ("or by exporting GTK_MODULES=globalmenu-gnome in your profile."), NULL), "globalmenu", NULL), (self->priv->notify_no_plugin == NULL) ? NULL : (self->priv->notify_no_plugin = (g_object_unref (self->priv->notify_no_plugin), NULL)), _tmp6);
		_tmp5 = (g_free (_tmp5), NULL);
		_tmp4 = (g_free (_tmp4), NULL);
	}
	{
		notify_notification_show (self->priv->notify_no_plugin, &inner_error);
		if (inner_error != NULL) {
			goto __catch1_g_error;
			goto __finally1;
		}
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally1:
	if (inner_error != NULL) {
		(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
		modules = (g_free (modules), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
	modules = (g_free (modules), NULL);
}


/*ignore the error*/
static void applet_real_realize (GtkWidget* base) {
	GlobalMenuPanelApplet * self;
	self = (GlobalMenuPanelApplet*) base;
	GTK_WIDGET_CLASS (applet_parent_class)->realize ((GtkWidget*) PANEL_APPLET (self));
}


static void applet_get_prefs (GlobalMenuPanelApplet* self) {
	GError * inner_error;
	gint _tmp0;
	gboolean _tmp1;
	gboolean _tmp2;
	gboolean _tmp3;
	gboolean _tmp4;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	_tmp0 = panel_applet_gconf_get_int ((PanelApplet*) self, "title_max_width", &inner_error);
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	switcher_set_max_size (self->priv->switcher, _tmp0);
	_tmp1 = panel_applet_gconf_get_bool ((PanelApplet*) self, "show_icon", &inner_error);
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	switcher_set_show_icon (self->priv->switcher, _tmp1);
	_tmp2 = panel_applet_gconf_get_bool ((PanelApplet*) self, "show_name", &inner_error);
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	switcher_set_show_label (self->priv->switcher, _tmp2);
	_tmp3 = panel_applet_gconf_get_bool ((PanelApplet*) self, "show_window_actions", &inner_error);
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	switcher_set_show_window_actions (self->priv->switcher, _tmp3);
	_tmp4 = panel_applet_gconf_get_bool ((PanelApplet*) self, "show_window_list", &inner_error);
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	switcher_set_show_window_list (self->priv->switcher, _tmp4);
}


static void applet_on_about_clicked (BonoboUIComponent* component, void* user_data, const char* cname) {
	GtkAboutDialog* dialog;
	char* ver;
	g_return_if_fail (component != NULL);
	g_return_if_fail (cname != NULL);
	/*Applet _this = (Applet) user_data;*/
	dialog = g_object_ref_sink ((GtkAboutDialog*) gtk_about_dialog_new ());
	gtk_about_dialog_set_program_name (dialog, APPLET_APPLET_NAME);
	ver = g_strdup (VERSION);
	if (string_get_length (SVNVERSION) > 0) {
		char* _tmp0;
		_tmp0 = NULL;
		ver = (_tmp0 = g_strconcat (ver, ".svn" SVNVERSION, NULL), ver = (g_free (ver), NULL), _tmp0);
	}
	gtk_about_dialog_set_version (dialog, ver);
	gtk_about_dialog_set_website (dialog, "http://code.google.com/p/gnome2-globalmenu");
	gtk_about_dialog_set_website_label (dialog, _ ("Project Home"));
	gtk_about_dialog_set_wrap_license (dialog, FALSE);
	gtk_about_dialog_set_license (dialog, LICENSES_GPLv2);
	gtk_about_dialog_set_logo_icon_name (dialog, APPLET_APPLET_ICON);
	gtk_about_dialog_set_authors (dialog, APPLET_APPLET_AUTHORS);
	gtk_about_dialog_set_documenters (dialog, APPLET_APPLET_ADOCUMENTERS);
	gtk_about_dialog_set_translator_credits (dialog, _ ("translator-credits"));
	gtk_window_set_icon_name ((GtkWindow*) dialog, "gtk-about");
	gtk_dialog_run ((GtkDialog*) dialog);
	gtk_object_destroy ((GtkObject*) dialog);
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	ver = (g_free (ver), NULL);
}


static void applet_on_help_clicked (BonoboUIComponent* component, void* user_data, const char* cname) {
	GtkAboutDialog* dialog;
	g_return_if_fail (component != NULL);
	g_return_if_fail (cname != NULL);
	/*Applet _this = (Applet) user_data;*/
	dialog = g_object_ref_sink ((GtkAboutDialog*) gtk_about_dialog_new ());
	gtk_about_dialog_set_program_name (dialog, APPLET_APPLET_NAME);
	gtk_about_dialog_set_version (dialog, VERSION);
	gtk_about_dialog_set_website (dialog, "http://code.google.com/p/gnome2-globalmenu/wiki/HelpCentral");
	gtk_about_dialog_set_website_label (dialog, _ ("On-line help"));
	gtk_about_dialog_set_logo_icon_name (dialog, "gtk-help");
	gtk_window_set_icon_name ((GtkWindow*) dialog, "gtk-help");
	gtk_dialog_run ((GtkDialog*) dialog);
	gtk_object_destroy ((GtkObject*) dialog);
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
}


static void applet_on_preferences_clicked (BonoboUIComponent* component, void* user_data, const char* cname) {
	GlobalMenuPanelApplet* _tmp0;
	GlobalMenuPanelApplet* _this;
	GConfDialog* gcd;
	gint _tmp3_length1;
	char** _tmp3;
	char** _tmp2;
	char* _tmp1;
	g_return_if_fail (component != NULL);
	g_return_if_fail (cname != NULL);
	_tmp0 = NULL;
	_this = (_tmp0 = APPLET (user_data), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	gcd = g_object_ref_sink (gconf_dialog_new (_ ("Global Menu Applet Preferences")));
	gconf_dialog_add_key (gcd, "/apps/gnome_settings_daemon/gtk-modules/globalmenu-gnome");
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	gconf_dialog_add_subkeys (gcd, _tmp1 = panel_applet_get_preferences_key ((PanelApplet*) _this), (_tmp3 = (_tmp2 = g_new0 (char*, 5 + 1), _tmp2[0] = g_strdup ("show_icon"), _tmp2[1] = g_strdup ("show_name"), _tmp2[2] = g_strdup ("title_max_width"), _tmp2[3] = g_strdup ("show_window_actions"), _tmp2[4] = g_strdup ("show_window_list"), _tmp2), _tmp3_length1 = 5, _tmp3), 5);
	_tmp3 = (_vala_array_free (_tmp3, _tmp3_length1, (GDestroyNotify) g_free), NULL);
	_tmp1 = (g_free (_tmp1), NULL);
	gtk_dialog_run ((GtkDialog*) gcd);
	gtk_object_destroy ((GtkObject*) gcd);
	(_this == NULL) ? NULL : (_this = (g_object_unref (_this), NULL));
	(gcd == NULL) ? NULL : (gcd = (g_object_unref (gcd), NULL));
}


static gboolean applet_real_button_press_event (GtkWidget* base, const GdkEventButton* event) {
	GlobalMenuPanelApplet * self;
	self = (GlobalMenuPanelApplet*) base;
	if ((*event).button == 3) {
		return bonobo_control_do_popup (panel_applet_get_control ((PanelApplet*) self), (*event).button, (*event).time);
	}
	return FALSE;
}


static gboolean __lambda5 (GtkWidget* widget, const GdkEventButton* event, GlobalMenuPanelApplet* self) {
	g_return_val_if_fail (widget != NULL, FALSE);
	if ((*event).button == 3) {
		gboolean _tmp1;
		GlobalMenuPanelApplet* _tmp0;
		_tmp0 = NULL;
		g_signal_emit_by_name ((_tmp0 = self, GTK_IS_WIDGET (_tmp0) ? ((GtkWidget*) _tmp0) : NULL), "button-press-event", &(*event), &_tmp1);
		_tmp1;
		return TRUE;
	} else {
		return FALSE;
	}
}


static gboolean ___lambda5_gtk_widget_button_press_event (GtkWidget* _sender, const GdkEventButton* event, gpointer self) {
	return __lambda5 (_sender, event, self);
}


static void applet_setup_popup_menu (GlobalMenuPanelApplet* self, GtkWidget* widget) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	g_signal_connect (widget, "button-press-event", (GCallback) ___lambda5_gtk_widget_button_press_event, self);
}


static void _applet_on_window_changed_gnomenu_monitor_window_changed (GnomenuMonitor* _sender, gulong old_xid, gpointer self) {
	applet_on_window_changed (self, _sender, old_xid);
}


static GObject * applet_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GlobalMenuPanelAppletClass * klass;
	GObjectClass * parent_class;
	GlobalMenuPanelApplet * self;
	klass = APPLET_CLASS (g_type_class_peek (TYPE_APPLET));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = APPLET (obj);
	{
		GdkColor color = {0};
		GdkPixmap* pixmap;
		PanelAppletBackgroundType bgtype;
		GdkPixmap* _tmp2;
		PanelAppletBackgroundType _tmp1;
		GdkPixmap* _tmp0;
		GlobalMenuPanelApplet* _tmp3;
		self->priv->disposed = FALSE;
		gtk_widget_add_events ((GtkWidget*) self, (gint) GDK_KEY_PRESS_MASK);
		g_object_set ((GtkWidget*) self->priv->menubars, "visible", TRUE, NULL);
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->menubars);
		g_object_set ((GtkWidget*) self->priv->switcher, "visible", TRUE, NULL);
		gtk_container_add ((GtkContainer*) self->priv->menubars, (GtkWidget*) self->priv->switcher);
		applet_setup_popup_menu (self, (GtkWidget*) self->priv->switcher);
		gnomenu_menu_bar_set_min_length ((GnomenuMenuBar*) self->priv->main_menubar, 12);
		/*Then it will have a overflown item*/
		applet_setup_popup_menu (self, (GtkWidget*) self->priv->main_menubar);
		gtk_container_add ((GtkContainer*) self->priv->menubars, (GtkWidget*) self->priv->main_menubar);
		gtk_container_child_set ((GtkContainer*) self->priv->menubars, (GtkWidget*) self->priv->main_menubar, "expand", TRUE, NULL, NULL);
		/*Let the main_menubar use the remaining space in the applet.
		init panel */
		panel_applet_set_flags ((PanelApplet*) self, PANEL_APPLET_EXPAND_MINOR | PANEL_APPLET_EXPAND_MAJOR);
		panel_applet_set_background_widget ((PanelApplet*) self, (GtkWidget*) self);
		pixmap = NULL;
		bgtype = 0;
		_tmp2 = NULL;
		_tmp0 = NULL;
		bgtype = (_tmp1 = panel_applet_get_background ((PanelApplet*) self, &color, &_tmp0), pixmap = (_tmp2 = _tmp0, (pixmap == NULL) ? NULL : (pixmap = (g_object_unref (pixmap), NULL)), _tmp2), _tmp1);
		_tmp3 = NULL;
		g_signal_emit_by_name ((_tmp3 = self, PANEL_IS_APPLET (_tmp3) ? ((PanelApplet*) _tmp3) : NULL), "change-background", bgtype, &color, pixmap);
		g_signal_connect_object (self->priv->monitor, "window-changed", (GCallback) _applet_on_window_changed_gnomenu_monitor_window_changed, self, 0);
		applet_on_window_changed (self, self->priv->monitor, (gulong) 0);
		(pixmap == NULL) ? NULL : (pixmap = (g_object_unref (pixmap), NULL));
	}
	return obj;
}


static void applet_class_init (GlobalMenuPanelAppletClass * klass) {
	applet_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GlobalMenuPanelAppletPrivate));
	G_OBJECT_CLASS (klass)->constructor = applet_constructor;
	G_OBJECT_CLASS (klass)->finalize = applet_finalize;
	G_OBJECT_CLASS (klass)->dispose = applet_real_dispose;
	GTK_WIDGET_CLASS (klass)->screen_changed = applet_real_screen_changed;
	PANEL_APPLET_CLASS (klass)->change_background = applet_real_change_background;
	PANEL_APPLET_CLASS (klass)->change_orient = applet_real_change_orient;
	GTK_WIDGET_CLASS (klass)->realize = applet_real_realize;
	GTK_WIDGET_CLASS (klass)->button_press_event = applet_real_button_press_event;
}


static void applet_instance_init (GlobalMenuPanelApplet * self) {
	self->priv = APPLET_GET_PRIVATE (self);
	self->priv->monitor = gnomenu_monitor_new ();
	self->priv->menubars = g_object_ref_sink (gnomenu_menu_bar_box_new ());
	self->priv->main_menubar = g_object_ref_sink (gnomenu_global_menu_new ());
	self->priv->switcher = g_object_ref_sink (switcher_new ());
}


static void applet_finalize (GObject* obj) {
	GlobalMenuPanelApplet * self;
	self = APPLET (obj);
	{
	}
	(self->priv->monitor == NULL) ? NULL : (self->priv->monitor = (g_object_unref (self->priv->monitor), NULL));
	(self->priv->menubars == NULL) ? NULL : (self->priv->menubars = (g_object_unref (self->priv->menubars), NULL));
	(self->priv->main_menubar == NULL) ? NULL : (self->priv->main_menubar = (g_object_unref (self->priv->main_menubar), NULL));
	(self->priv->switcher == NULL) ? NULL : (self->priv->switcher = (g_object_unref (self->priv->switcher), NULL));
	(self->priv->notify_no_plugin == NULL) ? NULL : (self->priv->notify_no_plugin = (g_object_unref (self->priv->notify_no_plugin), NULL));
	G_OBJECT_CLASS (applet_parent_class)->finalize (obj);
}


GType applet_get_type (void) {
	static GType applet_type_id = 0;
	if (applet_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GlobalMenuPanelAppletClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) applet_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GlobalMenuPanelApplet), 0, (GInstanceInitFunc) applet_instance_init, NULL };
		applet_type_id = g_type_register_static (PANEL_TYPE_APPLET, "GlobalMenuPanelApplet", &g_define_type_info, 0);
	}
	return applet_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}




