/*******************************
 * GConfDialog
 * Author: Luca Viggiani
 * Version: 0.1
 * Date: 2009-12-07
 * License: GPL
 * Description: creates a gconf settings back end dialog 
 * 				automatically from a settings key root.
 * Usage: new GConfDialog (string key, string dialog_title)
 * Params:
 * 	    string key: the gconf key to connect to (i.e /apps/panel/applets/applet_46/prefs)
 * 		string dislog_title: text to be shown on window title bar
 *		string[] subkeys: used to choose the subkeys to hadle (comprising display order).
 * 			  if null all subkeys will be retieved.
 * 
 * TODO: Only BOOL and INT data types are implemented. STRING is implemented only partially (read)
 *******************************/

#ifndef __GTKEXTRA_GCONFDIALOG_H__
#define __GTKEXTRA_GCONFDIALOG_H__

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>

G_BEGIN_DECLS


#define TYPE_GCONF_DIALOG (gconf_dialog_get_type ())
#define GCONF_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GCONF_DIALOG, GConfDialog))
#define GCONF_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GCONF_DIALOG, GConfDialogClass))
#define IS_GCONF_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GCONF_DIALOG))
#define IS_GCONF_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GCONF_DIALOG))
#define GCONF_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GCONF_DIALOG, GConfDialogClass))

typedef struct _GConfDialog GConfDialog;
typedef struct _GConfDialogClass GConfDialogClass;
typedef struct _GConfDialogPrivate GConfDialogPrivate;

struct _GConfDialog {
	GtkDialog parent_instance;
	GConfDialogPrivate * priv;
};

struct _GConfDialogClass {
	GtkDialogClass parent_class;
};


GConfDialog* gconf_dialog_construct (GType object_type, const char* dialog_title);
GConfDialog* gconf_dialog_new (const char* dialog_title);
GConfDialog* gconf_dialog_construct_with_subkeys (GType object_type, const char* key, const char* dialog_title, char** subkeys, int subkeys_length1);
GConfDialog* gconf_dialog_new_with_subkeys (const char* key, const char* dialog_title, char** subkeys, int subkeys_length1);
void gconf_dialog_add_key (GConfDialog* self, const char* key);
void gconf_dialog_add_subkeys (GConfDialog* self, const char* key, char** subkeys, int subkeys_length1);
GType gconf_dialog_get_type (void);


G_END_DECLS

#endif
