
#include "switcher.h"
#include <libgnomenu/interface-shell.h>
#include <libgnomenu/interface-item.h>
#include <gdk/gdk.h>
#include <libgnomenu/menu.h>
#include <libgnomenu/application.h>
#include <glib/gi18n-lib.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <float.h>
#include <math.h>
#include <libgnomenu/parser.h>
#include "template.h"
#include "workspaceselector.h"




static char* string_substring (const char* self, glong offset, glong len);
static char* bool_to_string (gboolean self);
static glong string_get_length (const char* self);
struct _SwitcherPrivate {
	char* _label;
	gint _max_size;
	gboolean _show_icon;
	gboolean _show_label;
	gboolean _show_window_list;
	gboolean _show_window_actions;
	GnomenuMenuItem* _label_item;
	WnckWindow* _current_window;
	char* NOWIN_TEMPLATE;
	char* NOACT_TEMPLATE;
	char* WINACT_TEMPLATE;
	SearchBoxMenuItem* search_box;
	gboolean disposed;
};

#define SWITCHER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SWITCHER, SwitcherPrivate))
enum  {
	SWITCHER_DUMMY_PROPERTY,
	SWITCHER_CURRENT_WINDOW,
	SWITCHER_MAX_SIZE,
	SWITCHER_SHOW_ICON,
	SWITCHER_SHOW_LABEL,
	SWITCHER_SHOW_WINDOW_LIST,
	SWITCHER_SHOW_WINDOW_ACTIONS
};
#define SWITCHER_MENU_TEMPLATE "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<menu>\n\t<item id=\"switcher\" underline=\"false\" label=\"%label%\" font=\"bold\" type=\"%type%\" icon=\"%icon%\">\n\t\t\t%submenu%\n\t</item>\n</menu>"
#define SWITCHER_ITEM_TEMPLATE "<item type=\"image\" id=\"XID%id%\" label=\"%label%\" underline=\"false\" font=\"%font%\" icon=\"pixbuf:%pixdata%\" sensitive=\"true\">%submenu%</item>"
static void switcher_real_dispose (GObject* base);
static char* switcher_replace (Switcher* self, const char* source, const char* find, const char* replacement);
static void switcher_on_deactivate (Switcher* self);
static void switcher_on_activate (Switcher* self, GnomenuMenuBar* _this, GnomenuMenuItem* item);
static void switcher_perhaps_minimize_window (Switcher* self, WnckWindow* window);
static char* switcher_remove_path (Switcher* self, const char* txt, const char* separator);
static void switcher_set_iconify_destination (Switcher* self, WnckWindow* window);
static gboolean switcher_is_in_sight (Switcher* self, WnckWindow* window);
static char* switcher_pixbuf_encode_b64 (Switcher* self, GdkPixbuf* pixbuf);
static char* switcher_do_xml_menu (Switcher* self);
static char* switcher_do_action_menu (Switcher* self, WnckWindow* window);
static WnckWindow* switcher_find_desktop (Switcher* self);
static GdkPixbuf* switcher_guess_icon (Switcher* self, WnckWindow* window, gint width, gint height);
static GdkPixbuf* switcher_guess_icon_array (Switcher* self, GdkPixbuf** icons, int icons_length1, gint width, gint height);
static void switcher_update (Switcher* self, gboolean include_menu);
static gboolean switcher_guess_dock_is_around (Switcher* self);
static GnomenuMenuItem* switcher_window_to_item (Switcher* self, WnckWindow* window);
static WnckWindow* switcher_item_to_window (Switcher* self, GnomenuMenuItem* item);
static void switcher_real_size_request (GtkWidget* base, GtkRequisition* req);
static void _switcher_on_activate_gnomenu_menu_bar_activate (Switcher* _sender, GnomenuMenuItem* item, gpointer self);
static void _switcher_on_deactivate_gtk_menu_shell_deactivate (Switcher* _sender, gpointer self);
static GObject * switcher_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer switcher_parent_class = NULL;
static void switcher_finalize (GObject* obj);
struct _SearchBoxMenuItemPrivate {
	GtkEntry* textbox;
};

#define SEARCH_BOX_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_SEARCH_BOX_MENU_ITEM, SearchBoxMenuItemPrivate))
enum  {
	SEARCH_BOX_MENU_ITEM_DUMMY_PROPERTY,
	SEARCH_BOX_MENU_ITEM_TEXT
};
static gboolean _search_box_menu_item_steal_key_press_event_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self);
static void search_box_menu_item_real_parent_set (GtkWidget* base, GtkWidget* previous_parent);
static gboolean search_box_menu_item_steal_key_press_event (SearchBoxMenuItem* self, GtkWidget* widget, const GdkEventKey* event);
static void search_box_menu_item_real_activate_item (GtkMenuItem* base);
static void search_box_menu_item_real_activate (GtkMenuItem* base);
static void __lambda0 (GtkEntry* box, GtkStateType previous_state, SearchBoxMenuItem* self);
static void ___lambda0_gtk_widget_state_changed (GtkEntry* _sender, GtkStateType previous_state, gpointer self);
static void __lambda1 (GtkEntry* box, SearchBoxMenuItem* self);
static void ___lambda1_gtk_editable_changed (GtkEntry* _sender, gpointer self);
static void __lambda2 (GtkEntry* box, SearchBoxMenuItem* self);
static void ___lambda2_gtk_entry_activate (GtkEntry* _sender, gpointer self);
static void __lambda3 (SearchBoxMenuItem* _this, GtkStateType previous_state, SearchBoxMenuItem* self);
static void ___lambda3_gtk_widget_state_changed (SearchBoxMenuItem* _sender, GtkStateType previous_state, gpointer self);
static GObject * search_box_menu_item_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer search_box_menu_item_parent_class = NULL;
static void search_box_menu_item_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static char* bool_to_string (gboolean self) {
	if (self) {
		return g_strdup ("true");
	} else {
		return g_strdup ("false");
	}
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


Switcher* switcher_construct (GType object_type) {
	Switcher * self;
	self = (Switcher*) gnomenu_menu_bar_construct (object_type);
	return self;
}


Switcher* switcher_new (void) {
	return switcher_construct (TYPE_SWITCHER);
}


static void switcher_real_dispose (GObject* base) {
	Switcher * self;
	self = (Switcher*) base;
	if (!self->priv->disposed) {
		self->priv->disposed = TRUE;
		switcher_set_current_window (self, NULL);
	}
}


static char* switcher_replace (Switcher* self, const char* source, const char* find, const char* replacement) {
	char** _tmp1;
	gint buf_size;
	gint buf_length1;
	char** _tmp0;
	char** buf;
	GString* sb;
	const char* _tmp2;
	char* _tmp3;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (source != NULL, NULL);
	g_return_val_if_fail (find != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	/* replaces the string.replace method which depends on GLib.RegEx >= 2.12 */
	_tmp1 = NULL;
	_tmp0 = NULL;
	buf = (_tmp1 = _tmp0 = g_strsplit (source, find, 0), buf_length1 = _vala_array_length (_tmp0), buf_size = buf_length1, _tmp1);
	sb = g_string_new ("");
	{
		gint co;
		co = 0;
		for (; co < buf_length1; co++) {
			g_string_append (sb, buf[co]);
			if (co != (buf_length1 - 1)) {
				g_string_append (sb, replacement);
			}
		}
	}
	_tmp2 = NULL;
	_tmp3 = NULL;
	return (_tmp3 = (_tmp2 = sb->str, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), buf = (_vala_array_free (buf, buf_length1, (GDestroyNotify) g_free), NULL), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), _tmp3);
}


static void switcher_on_deactivate (Switcher* self) {
	g_return_if_fail (self != NULL);
	switcher_update (self, FALSE);
}


static void switcher_on_activate (Switcher* self, GnomenuMenuBar* _this, GnomenuMenuItem* item) {
	WnckWindow* window;
	GQuark _tmp8;
	char* _tmp7;
	g_return_if_fail (self != NULL);
	g_return_if_fail (_this != NULL);
	g_return_if_fail (item != NULL);
	if (item == self->priv->_label_item) {
		switcher_update (self, TRUE);
		return;
	}
	/** Then handle the window actions */
	window = switcher_item_to_window (self, item);
	/* if it is an Action Menu item, try to get reference to parent */
	if (window == NULL) {
		char** _tmp3;
		gint paths_size;
		gint paths_length1;
		char** _tmp1;
		char* _tmp0;
		char** _tmp2;
		char** paths;
		char* parent_path;
		WnckWindow* _tmp6;
		GnomenuMenuItem* _tmp5;
		_tmp3 = NULL;
		_tmp1 = NULL;
		_tmp0 = NULL;
		_tmp2 = NULL;
		paths = (_tmp3 = (_tmp2 = _tmp1 = g_strsplit (_tmp0 = gnomenu_item_get_item_path ((GnomenuItem*) item), "/", 0), _tmp0 = (g_free (_tmp0), NULL), _tmp2), paths_length1 = _vala_array_length (_tmp1), paths_size = paths_length1, _tmp3);
		parent_path = g_strdup ("");
		{
			gint co;
			co = 0;
			for (; co < (paths_length1 - 1); co++) {
				if (_vala_strcmp0 (paths[co], "") != 0) {
					char* _tmp4;
					_tmp4 = NULL;
					parent_path = (_tmp4 = g_strconcat (parent_path, "/", paths[co], NULL, NULL), parent_path = (g_free (parent_path), NULL), _tmp4);
				}
			}
		}
		_tmp6 = NULL;
		_tmp5 = NULL;
		window = (_tmp6 = switcher_item_to_window (self, _tmp5 = gnomenu_menu_bar_get ((GnomenuMenuBar*) self, parent_path)), (window == NULL) ? NULL : (window = (g_object_unref (window), NULL)), _tmp6);
		(_tmp5 == NULL) ? NULL : (_tmp5 = (g_object_unref (_tmp5), NULL));
		paths = (_vala_array_free (paths, paths_length1, (GDestroyNotify) g_free), NULL);
		parent_path = (g_free (parent_path), NULL);
	}
	_tmp7 = NULL;
	static GQuark _tmp8_label0 = 0;
	static GQuark _tmp8_label1 = 0;
	static GQuark _tmp8_label2 = 0;
	static GQuark _tmp8_label3 = 0;
	static GQuark _tmp8_label4 = 0;
	static GQuark _tmp8_label5 = 0;
	static GQuark _tmp8_label6 = 0;
	static GQuark _tmp8_label7 = 0;
	static GQuark _tmp8_label8 = 0;
	static GQuark _tmp8_label9 = 0;
	static GQuark _tmp8_label10 = 0;
	static GQuark _tmp8_label11 = 0;
	_tmp7 = gnomenu_item_get_item_id ((GnomenuItem*) item);
	_tmp8 = (NULL == _tmp7) ? 0 : g_quark_from_string (_tmp7);
	if (_tmp8 == ((0 != _tmp8_label0) ? _tmp8_label0 : (_tmp8_label0 = g_quark_from_static_string ("minimize"))))
	do {
		if (window != NULL) {
			wnck_window_minimize (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label1) ? _tmp8_label1 : (_tmp8_label1 = g_quark_from_static_string ("maximize"))))
	do {
		if (window != NULL) {
			wnck_window_maximize (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label2) ? _tmp8_label2 : (_tmp8_label2 = g_quark_from_static_string ("unmaximize"))))
	do {
		if (window != NULL) {
			wnck_window_unmaximize (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label3) ? _tmp8_label3 : (_tmp8_label3 = g_quark_from_static_string ("move"))))
	do {
		if (window != NULL) {
			wnck_window_keyboard_move (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label4) ? _tmp8_label4 : (_tmp8_label4 = g_quark_from_static_string ("resize"))))
	do {
		if (window != NULL) {
			wnck_window_keyboard_size (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label5) ? _tmp8_label5 : (_tmp8_label5 = g_quark_from_static_string ("topmost"))))
	do {
		if (window != NULL) {
			wnck_window_make_above (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label6) ? _tmp8_label6 : (_tmp8_label6 = g_quark_from_static_string ("untopmost"))))
	do {
		if (window != NULL) {
			wnck_window_unmake_above (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label7) ? _tmp8_label7 : (_tmp8_label7 = g_quark_from_static_string ("stick"))))
	do {
		if (window != NULL) {
			wnck_window_stick (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label8) ? _tmp8_label8 : (_tmp8_label8 = g_quark_from_static_string ("unstick"))))
	do {
		if (window != NULL) {
			wnck_window_unstick (window);
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label9) ? _tmp8_label9 : (_tmp8_label9 = g_quark_from_static_string ("close"))))
	do {
		if (window != NULL) {
			wnck_window_close (window, gtk_get_current_event_time ());
		}
		break;
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label10) ? _tmp8_label10 : (_tmp8_label10 = g_quark_from_static_string ("show_desktop"))))
	do {
		GList* windows;
		windows = wnck_screen_get_windows (wnck_screen_get_default ());
		{
			GList* w_collection;
			GList* w_it;
			w_collection = windows;
			for (w_it = w_collection; w_it != NULL; w_it = w_it->next) {
				WnckWindow* w;
				w = (WnckWindow*) w_it->data;
				{
					if (switcher_is_in_sight (self, w)) {
						wnck_window_minimize (w);
					}
				}
			}
		}
		break;
	} while (0); else if (_tmp8 == ((0 != _tmp8_label11) ? _tmp8_label11 : (_tmp8_label11 = g_quark_from_static_string ("workspaces"))))
	do {
		WorkspaceSelector* ws;
		ws = g_object_ref_sink (workspace_selector_new (window));
		gtk_widget_show ((GtkWidget*) ws);
		(ws == NULL) ? NULL : (ws = (g_object_unref (ws), NULL));
		break;
	} while (0); else
	do {
		if (gtk_menu_item_get_submenu ((GtkMenuItem*) item) != NULL) {
			(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
			return;
		}
		if (window != NULL) {
			switcher_perhaps_minimize_window (self, window);
		}
		break;
	} while (0);
	(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
}


/** 
 * Minimize a window if it is not minimized.
 * restore it if minimized.
 *
 */
static void switcher_perhaps_minimize_window (Switcher* self, WnckWindow* window) {
	WnckWorkspace* _tmp0;
	WnckWorkspace* workspace;
	gboolean _tmp1;
	WnckScreen* _tmp2;
	WnckScreen* screen;
	gint current_workspace_x;
	gint current_workspace_y;
	gint x;
	gint y;
	gint w;
	gint h;
	g_return_if_fail (self != NULL);
	g_return_if_fail (window != NULL);
	if (!switcher_guess_dock_is_around (self)) {
		switcher_set_iconify_destination (self, window);
	}
	_tmp0 = NULL;
	workspace = (_tmp0 = wnck_window_get_workspace (window), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	_tmp1 = FALSE;
	if (wnck_window_is_active (window)) {
		_tmp1 = wnck_window_is_visible_on_workspace (window, workspace);
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		wnck_window_minimize (window);
		(workspace == NULL) ? NULL : (workspace = (g_object_unref (workspace), NULL));
		return;
	}
	/* Ensure viewport visibility*/
	_tmp2 = NULL;
	screen = (_tmp2 = wnck_window_get_screen (window), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
	current_workspace_x = wnck_workspace_get_viewport_x (workspace);
	current_workspace_y = wnck_workspace_get_viewport_y (workspace);
	x = 0;
	y = 0;
	w = 0;
	h = 0;
	wnck_window_get_geometry (window, &x, &y, &w, &h);
	wnck_screen_move_viewport (screen, current_workspace_x + x, current_workspace_y + y);
	wnck_window_activate (window, gtk_get_current_event_time ());
	wnck_window_unminimize (window, gtk_get_current_event_time ());
	/* ensure is on top*/
	wnck_window_make_above (window);
	wnck_window_unmake_above (window);
	(workspace == NULL) ? NULL : (workspace = (g_object_unref (workspace), NULL));
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


static char* switcher_remove_path (Switcher* self, const char* txt, const char* separator) {
	glong co;
	char* ret;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (txt != NULL, NULL);
	g_return_val_if_fail (separator != NULL, NULL);
	co = string_get_length (txt) - 1;
	while (TRUE) {
		gboolean _tmp0;
		_tmp0 = FALSE;
		if (co >= 0) {
			char* _tmp1;
			_tmp1 = NULL;
			_tmp0 = _vala_strcmp0 (_tmp1 = string_substring (txt, co, (glong) 1), separator) != 0;
			_tmp1 = (g_free (_tmp1), NULL);
		} else {
			_tmp0 = FALSE;
		}
		if (!_tmp0) {
			break;
		}
		co--;
	}
	ret = string_substring (txt, co + 1, (string_get_length (txt) - co) - 1);
	return ret;
}


static void switcher_set_iconify_destination (Switcher* self, WnckWindow* window) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (window != NULL);
	if (!GTK_WIDGET_REALIZED ((GtkWidget*) self)) {
		return;
	}
	if (wnck_window_get_window_type (window) != WNCK_WINDOW_DESKTOP) {
		gint ax;
		gint ay;
		ax = 0;
		ay = 0;
		gdk_window_get_origin (((GtkWidget*) self)->window, &ax, &ay);
		wnck_window_set_icon_geometry (window, ax, ay, 8, 8);
	}
}


static gboolean switcher_is_in_sight (Switcher* self, WnckWindow* window) {
	gboolean _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (window != NULL, FALSE);
	_tmp0 = FALSE;
	_tmp1 = FALSE;
	if (wnck_window_is_in_viewport (window, wnck_window_get_workspace (self->priv->_current_window))) {
		_tmp1 = !wnck_window_is_minimized (window);
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		_tmp0 = !wnck_window_is_skip_pager (window);
	} else {
		_tmp0 = FALSE;
	}
	return _tmp0;
}


static char* switcher_pixbuf_encode_b64 (Switcher* self, GdkPixbuf* pixbuf) {
	GdkPixdata pixdata = {(guint32) 0};
	guchar* _tmp1;
	gint _tmp0;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (pixbuf != NULL, NULL);
	gdk_pixdata_from_pixbuf (&pixdata, pixbuf, TRUE);
	_tmp1 = NULL;
	return (_tmp1 = gdk_pixdata_serialize (&pixdata, &_tmp0), g_base64_encode (_tmp1, _tmp0));
}


static char* switcher_do_xml_menu (Switcher* self) {
	char* items;
	GList* windows;
	char* _tmp22;
	char* _tmp23;
	char* _tmp24;
	g_return_val_if_fail (self != NULL, NULL);
	if (!self->priv->_show_window_list) {
		return g_strdup ("");
	}
	items = g_strdup ("");
	windows = wnck_screen_get_windows (wnck_screen_get_default ());
	{
		GList* window_collection;
		GList* window_it;
		window_collection = windows;
		for (window_it = window_collection; window_it != NULL; window_it = window_it->next) {
			WnckWindow* window;
			window = (WnckWindow*) window_it->data;
			{
				if (!wnck_window_is_skip_pager (window)) {
					char* font;
					char* submenu_xml;
					char** _tmp9;
					gint substs_size;
					gint substs_length1;
					char** _tmp7;
					GdkPixbuf* _tmp6;
					const char* _tmp5;
					const char* _tmp4;
					char** _tmp8;
					char** substs;
					char* item;
					char* _tmp10;
					font = g_strdup ("");
					submenu_xml = g_strdup ("");
					if (wnck_window_is_active (window)) {
						char* _tmp1;
						char* _tmp2;
						_tmp1 = NULL;
						font = (_tmp1 = g_strdup ("bold"), font = (g_free (font), NULL), _tmp1);
						_tmp2 = NULL;
						submenu_xml = (_tmp2 = switcher_do_action_menu (self, window), submenu_xml = (g_free (submenu_xml), NULL), _tmp2);
					} else {
						if (wnck_window_is_minimized (window)) {
							char* _tmp3;
							_tmp3 = NULL;
							font = (_tmp3 = g_strdup ("italic"), font = (g_free (font), NULL), _tmp3);
						}
					}
					_tmp9 = NULL;
					_tmp7 = NULL;
					_tmp6 = NULL;
					_tmp5 = NULL;
					_tmp4 = NULL;
					_tmp8 = NULL;
					substs = (_tmp9 = (_tmp8 = (_tmp7 = g_new0 (char*, 10 + 1), _tmp7[0] = g_strdup ("%label%"), _tmp7[1] = g_markup_escape_text (wnck_window_get_name (window), -1), _tmp7[2] = g_strdup ("%font%"), _tmp7[3] = (_tmp4 = font, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), _tmp7[4] = g_strdup ("%submenu%"), _tmp7[5] = (_tmp5 = submenu_xml, (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), _tmp7[6] = g_strdup ("%pixdata%"), _tmp7[7] = switcher_pixbuf_encode_b64 (self, _tmp6 = switcher_guess_icon (self, window, -1, -1)), _tmp7[8] = g_strdup ("%id%"), _tmp7[9] = g_strdup_printf ("%lu", wnck_window_get_xid (window)), _tmp7), (_tmp6 == NULL) ? NULL : (_tmp6 = (g_object_unref (_tmp6), NULL)), _tmp8), substs_length1 = 10, substs_size = substs_length1, _tmp9);
					item = template_replace (SWITCHER_ITEM_TEMPLATE, substs, substs_length1);
					_tmp10 = NULL;
					items = (_tmp10 = g_strconcat (items, item, NULL), items = (g_free (items), NULL), _tmp10);
					font = (g_free (font), NULL);
					submenu_xml = (g_free (submenu_xml), NULL);
					substs = (_vala_array_free (substs, substs_length1, (GDestroyNotify) g_free), NULL);
					item = (g_free (item), NULL);
				}
			}
		}
	}
	if (_vala_strcmp0 (items, "") == 0) {
		char* _tmp12;
		const char* _tmp11;
		_tmp12 = NULL;
		_tmp11 = NULL;
		items = (_tmp12 = (_tmp11 = self->priv->NOWIN_TEMPLATE, (_tmp11 == NULL) ? NULL : g_strdup (_tmp11)), items = (g_free (items), NULL), _tmp12);
	} else {
		if (wnck_window_get_window_type (self->priv->_current_window) != WNCK_WINDOW_DESKTOP) {
			char* _tmp13;
			char* _tmp21;
			char* _tmp20;
			char* _tmp19;
			char* _tmp18;
			char* _tmp17;
			char* _tmp16;
			GdkPixbuf* _tmp15;
			WnckWindow* _tmp14;
			_tmp13 = NULL;
			items = (_tmp13 = g_strconcat (items, "<item id=\"separator3\" type=\"separator\"/>", NULL), items = (g_free (items), NULL), _tmp13);
			_tmp21 = NULL;
			_tmp20 = NULL;
			_tmp19 = NULL;
			_tmp18 = NULL;
			_tmp17 = NULL;
			_tmp16 = NULL;
			_tmp15 = NULL;
			_tmp14 = NULL;
			items = (_tmp21 = g_strconcat (items, _tmp20 = g_strconcat (_tmp19 = g_strconcat (_tmp18 = g_strconcat (_tmp17 = g_strconcat ("<item id=\"show_desktop\" type=\"image\" icon=\"pixbuf:", _tmp16 = switcher_pixbuf_encode_b64 (self, _tmp15 = switcher_guess_icon (self, _tmp14 = switcher_find_desktop (self), -1, -1)), NULL), "\" label=\"", NULL), _ ("Show _Desktop"), NULL), "\"/>", NULL), NULL), items = (g_free (items), NULL), _tmp21);
			_tmp20 = (g_free (_tmp20), NULL);
			_tmp19 = (g_free (_tmp19), NULL);
			_tmp18 = (g_free (_tmp18), NULL);
			_tmp17 = (g_free (_tmp17), NULL);
			_tmp16 = (g_free (_tmp16), NULL);
			(_tmp15 == NULL) ? NULL : (_tmp15 = (g_object_unref (_tmp15), NULL));
			(_tmp14 == NULL) ? NULL : (_tmp14 = (g_object_unref (_tmp14), NULL));
		}
	}
	_tmp22 = NULL;
	_tmp23 = NULL;
	_tmp24 = NULL;
	return (_tmp24 = (_tmp23 = g_strconcat (_tmp22 = g_strconcat ("<menu>", items, NULL), "</menu>", NULL), _tmp22 = (g_free (_tmp22), NULL), _tmp23), items = (g_free (items), NULL), _tmp24);
}


static char* switcher_do_action_menu (Switcher* self, WnckWindow* window) {
	gboolean _tmp3;
	gboolean _tmp4;
	gboolean _tmp5;
	gboolean _tmp6;
	char** _tmp8;
	gint substs_size;
	gint substs_length1;
	char** _tmp7;
	char** substs;
	char* _tmp9;
	g_return_val_if_fail (self != NULL, NULL);
	if (wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP) {
		char* _tmp0;
		char* _tmp1;
		_tmp0 = NULL;
		_tmp1 = NULL;
		return (_tmp1 = g_strconcat (_tmp0 = g_strconcat ("<menu>", self->priv->NOACT_TEMPLATE, NULL), "</menu>", NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
	}
	_tmp3 = FALSE;
	if ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_MAXIMIZE) != 0) {
		_tmp3 = !wnck_window_is_maximized (window);
	} else {
		_tmp3 = FALSE;
	}
	_tmp4 = FALSE;
	if ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_MAXIMIZE) != 0) {
		_tmp4 = wnck_window_is_maximized (window);
	} else {
		_tmp4 = FALSE;
	}
	_tmp5 = FALSE;
	if ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_ABOVE) != 0) {
		_tmp5 = wnck_window_is_above (window);
	} else {
		_tmp5 = FALSE;
	}
	_tmp6 = FALSE;
	if ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_ABOVE) != 0) {
		_tmp6 = !wnck_window_is_above (window);
	} else {
		_tmp6 = FALSE;
	}
	_tmp8 = NULL;
	_tmp7 = NULL;
	substs = (_tmp8 = (_tmp7 = g_new0 (char*, 24 + 1), _tmp7[0] = g_strdup ("%visibleminimize%"), _tmp7[1] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_MINIMIZE) != 0), _tmp7[2] = g_strdup ("%visiblemaximize%"), _tmp7[3] = bool_to_string (_tmp3), _tmp7[4] = g_strdup ("%visibleunmaximize%"), _tmp7[5] = bool_to_string (_tmp4), _tmp7[6] = g_strdup ("%visiblemove%"), _tmp7[7] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_MOVE) != 0), _tmp7[8] = g_strdup ("%visibleresize"), _tmp7[9] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_RESIZE) != 0), _tmp7[10] = g_strdup ("%visibleseparator1%"), _tmp7[11] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_ABOVE) != 0), _tmp7[12] = g_strdup ("%visibleuntopmost%"), _tmp7[13] = bool_to_string (_tmp5), _tmp7[14] = g_strdup ("%visibletopmost%"), _tmp7[15] = bool_to_string (_tmp6), _tmp7[16] = g_strdup ("%visibleunstick%"), _tmp7[17] = bool_to_string (wnck_window_is_sticky (window)), _tmp7[18] = g_strdup ("%visiblestick%"), _tmp7[19] = bool_to_string (!wnck_window_is_sticky (window)), _tmp7[20] = g_strdup ("%visibleseparator2%"), _tmp7[21] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_CLOSE) != 0), _tmp7[22] = g_strdup ("%visibleclose%"), _tmp7[23] = bool_to_string ((wnck_window_get_actions (window) & WNCK_WINDOW_ACTION_CLOSE) != 0), _tmp7), substs_length1 = 24, substs_size = substs_length1, _tmp8);
	_tmp9 = NULL;
	return (_tmp9 = template_replace (self->priv->WINACT_TEMPLATE, substs, substs_length1), substs = (_vala_array_free (substs, substs_length1, (GDestroyNotify) g_free), NULL), _tmp9);
}


static WnckWindow* switcher_find_desktop (Switcher* self) {
	GList* windows;
	g_return_val_if_fail (self != NULL, NULL);
	windows = wnck_screen_get_windows (wnck_screen_get_default ());
	{
		GList* window_collection;
		GList* window_it;
		window_collection = windows;
		for (window_it = window_collection; window_it != NULL; window_it = window_it->next) {
			WnckWindow* window;
			window = (WnckWindow*) window_it->data;
			{
				if (wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP) {
					WnckWindow* _tmp0;
					_tmp0 = NULL;
					return (_tmp0 = window, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
				}
			}
		}
	}
	return NULL;
}


static GdkPixbuf* switcher_guess_icon (Switcher* self, WnckWindow* window, gint width, gint height) {
	GdkPixbuf** _tmp3;
	gint icons_size;
	gint icons_length1;
	GdkPixbuf** _tmp2;
	GdkPixbuf* _tmp1;
	GdkPixbuf* _tmp0;
	GdkPixbuf** icons;
	GdkPixbuf* _tmp4;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (window != NULL, NULL);
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	icons = (_tmp3 = (_tmp2 = g_new0 (GdkPixbuf*, 2 + 1), _tmp2[0] = (_tmp0 = wnck_window_get_mini_icon (window), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), _tmp2[1] = (_tmp1 = wnck_window_get_icon (window), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), _tmp2), icons_length1 = 2, icons_size = icons_length1, _tmp3);
	_tmp4 = NULL;
	return (_tmp4 = switcher_guess_icon_array (self, icons, icons_length1, width, height), icons = (_vala_array_free (icons, icons_length1, (GDestroyNotify) g_object_unref), NULL), _tmp4);
}


static GdkPixbuf* switcher_guess_icon_array (Switcher* self, GdkPixbuf** icons, int icons_length1, gint width, gint height) {
	GdkPixbuf* icon;
	gint min_dist;
	gint best_size;
	gboolean _tmp0;
	gint scaled_size;
	double ratio;
	gint scaled_width;
	gint scaled_height;
	GdkPixbuf* scaled_icon;
	GdkPixbuf* _tmp6;
	g_return_val_if_fail (self != NULL, NULL);
	icon = NULL;
	min_dist = 99999;
	best_size = 16;
	_tmp0 = FALSE;
	if (height == (-1)) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = width == (-1);
	}
	if (_tmp0) {
		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, &height);
	}
	scaled_size = height;
	if (width < scaled_size) {
		scaled_size = width;
	}
	{
		GdkPixbuf** i_collection;
		int i_collection_length1;
		int i_it;
		i_collection = icons;
		i_collection_length1 = icons_length1;
		for (i_it = 0; i_it < icons_length1; i_it = i_it + 1) {
			GdkPixbuf* _tmp3;
			GdkPixbuf* i;
			_tmp3 = NULL;
			i = (_tmp3 = i_collection[i_it], (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
			{
				gint size;
				gint dist;
				size = gdk_pixbuf_get_height (i);
				if (gdk_pixbuf_get_width (i) > size) {
					size = gdk_pixbuf_get_width (i);
				}
				dist = size - scaled_size;
				if (dist < 0) {
					dist = -dist;
				}
				if (dist < min_dist) {
					GdkPixbuf* _tmp2;
					GdkPixbuf* _tmp1;
					min_dist = dist;
					_tmp2 = NULL;
					_tmp1 = NULL;
					icon = (_tmp2 = (_tmp1 = i, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp2);
					best_size = size;
				}
				(i == NULL) ? NULL : (i = (g_object_unref (i), NULL));
			}
		}
	}
	/* This should never happen. In case it happens
	 * return something to make sure it doesn't crash*/
	if (icon == NULL) {
		GdkPixbuf* _tmp4;
		GdkPixbuf* _tmp5;
		_tmp4 = NULL;
		_tmp5 = NULL;
		return (_tmp5 = (_tmp4 = icons[0], (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4)), (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp5);
	}
	ratio = ((double) scaled_size) / ((double) best_size);
	if (ratio < 0) {
		ratio = 1.0;
	}
	scaled_width = (gint) (gdk_pixbuf_get_width (icon) * ratio);
	scaled_height = (gint) (gdk_pixbuf_get_height (icon) * ratio);
	scaled_icon = gdk_pixbuf_new (GDK_COLORSPACE_RGB, gdk_pixbuf_get_has_alpha (icon), 8, scaled_width, scaled_height);
	gdk_pixbuf_scale (icon, scaled_icon, 0, 0, gdk_pixbuf_get_width (scaled_icon), gdk_pixbuf_get_height (scaled_icon), (double) 0, (double) 0, ratio, ratio, GDK_INTERP_BILINEAR);
	/* can't use scale_simple because the vala binding is wrong*/
	_tmp6 = NULL;
	return (_tmp6 = scaled_icon, (icon == NULL) ? NULL : (icon = (g_object_unref (icon), NULL)), _tmp6);
}


static void switcher_update (Switcher* self, gboolean include_menu) {
	GError * inner_error;
	gboolean _tmp0;
	GnomenuApplication* _tmp1;
	GnomenuApplication* app;
	WnckWindowType wt;
	char* s;
	char* _tmp8;
	char* _tmp7;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	search_box_menu_item_set_text (self->priv->search_box, "");
	g_object_set ((GtkWidget*) self, "visible", self->priv->_show_icon | self->priv->_show_label, NULL);
	if (!(g_object_get ((GtkWidget*) self, "visible", &_tmp0, NULL), _tmp0)) {
		return;
	}
	if (switcher_get_current_window (self) == NULL) {
		return;
	}
	_tmp1 = NULL;
	app = (_tmp1 = gnomenu_application_lookup_from_wnck (wnck_window_get_application (switcher_get_current_window (self))), (_tmp1 == NULL) ? NULL : gnomenu_application_ref (_tmp1));
	wt = wnck_window_get_window_type (switcher_get_current_window (self));
	if (wt == WNCK_WINDOW_DOCK) {
		char* _tmp2;
		_tmp2 = NULL;
		self->priv->_label = (_tmp2 = g_strdup (""), self->priv->_label = (g_free (self->priv->_label), NULL), _tmp2);
	} else {
		if (app != NULL) {
			const char* _tmp3;
			char* _tmp5;
			const char* _tmp4;
			_tmp3 = NULL;
			if (wnck_window_get_window_type (switcher_get_current_window (self)) == WNCK_WINDOW_DESKTOP) {
				_tmp3 = _ ("Desktop");
			} else {
				_tmp3 = gnomenu_application_get_readable_name (app);
			}
			_tmp5 = NULL;
			_tmp4 = NULL;
			self->priv->_label = (_tmp5 = (_tmp4 = _tmp3, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), self->priv->_label = (g_free (self->priv->_label), NULL), _tmp5);
		} else {
			char* _tmp6;
			_tmp6 = NULL;
			self->priv->_label = (_tmp6 = g_strdup ("app unknown shouldn't see this"), self->priv->_label = (g_free (self->priv->_label), NULL), _tmp6);
		}
	}
	s = g_strdup (SWITCHER_MENU_TEMPLATE);
	_tmp8 = NULL;
	_tmp7 = NULL;
	s = (_tmp8 = switcher_replace (self, s, "%label%", _tmp7 = g_markup_escape_text (self->priv->_label, -1)), s = (g_free (s), NULL), _tmp8);
	_tmp7 = (g_free (_tmp7), NULL);
	if (self->priv->_show_icon) {
		gint width;
		gint height;
		char* _tmp11;
		if (self->priv->_show_label) {
			char* _tmp9;
			_tmp9 = NULL;
			s = (_tmp9 = switcher_replace (self, s, "%type%", "image"), s = (g_free (s), NULL), _tmp9);
		} else {
			char* _tmp10;
			_tmp10 = NULL;
			s = (_tmp10 = switcher_replace (self, s, "%type%", "icon"), s = (g_free (s), NULL), _tmp10);
		}
		width = ((GtkWidget*) self)->allocation.width - 2;
		height = ((GtkWidget*) self)->allocation.height - 2;
		if (width <= 0) {
			width = -1;
		}
		if (height <= 0) {
			height = -1;
		}
		_tmp11 = NULL;
		g_debug ("switcher.vala:390: iconname = %s no_in_menu = %s", gnomenu_application_get_icon_name (app), _tmp11 = bool_to_string (gnomenu_application_get_not_in_menu (app)));
		_tmp11 = (g_free (_tmp11), NULL);
		if (gnomenu_application_get_icon_pixbuf (app) != NULL) {
			char* _tmp14;
			char* _tmp13;
			char* _tmp12;
			_tmp14 = NULL;
			_tmp13 = NULL;
			_tmp12 = NULL;
			s = (_tmp14 = switcher_replace (self, s, "%icon%", _tmp13 = g_strconcat ("pixbuf:", _tmp12 = switcher_pixbuf_encode_b64 (self, gnomenu_application_get_icon_pixbuf (app)), NULL)), s = (g_free (s), NULL), _tmp14);
			_tmp13 = (g_free (_tmp13), NULL);
			_tmp12 = (g_free (_tmp12), NULL);
		} else {
			char* _tmp16;
			char* _tmp15;
			_tmp16 = NULL;
			_tmp15 = NULL;
			s = (_tmp16 = switcher_replace (self, s, "%icon%", _tmp15 = g_strconcat ("theme:", gnomenu_application_get_icon_name (app), NULL)), s = (g_free (s), NULL), _tmp16);
			_tmp15 = (g_free (_tmp15), NULL);
		}
	} else {
		char* _tmp17;
		char* _tmp18;
		_tmp17 = NULL;
		s = (_tmp17 = switcher_replace (self, s, "%type%", "normal"), s = (g_free (s), NULL), _tmp17);
		_tmp18 = NULL;
		s = (_tmp18 = switcher_replace (self, s, "icon=\"%icon%\"", ""), s = (g_free (s), NULL), _tmp18);
	}
	if (include_menu) {
		if (self->priv->_show_window_list) {
			char* _tmp20;
			char* _tmp19;
			GtkMenu* _tmp21;
			GtkMenu* menu;
			_tmp20 = NULL;
			_tmp19 = NULL;
			s = (_tmp20 = switcher_replace (self, s, "%submenu%", _tmp19 = switcher_do_xml_menu (self)), s = (g_free (s), NULL), _tmp20);
			_tmp19 = (g_free (_tmp19), NULL);
			gnomenu_parser_parse ((GnomenuShell*) self, s, &inner_error);
			if (inner_error != NULL) {
				(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
				s = (g_free (s), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
				g_clear_error (&inner_error);
				return;
			}
			_tmp21 = NULL;
			menu = (_tmp21 = gtk_menu_item_get_submenu ((GtkMenuItem*) self->priv->_label_item), (_tmp21 == NULL) ? NULL : g_object_ref (_tmp21));
			/*Gnomenu.MenuItem misd = this.get("/switcher/show_desktop");
			if (misd!=null) {
			override_item_window(misd, find_desktop());
			}*/
			{
				GList* widget_collection;
				GList* widget_it;
				widget_collection = gtk_container_get_children ((GtkContainer*) menu);
				for (widget_it = widget_collection; widget_it != NULL; widget_it = widget_it->next) {
					GtkWidget* _tmp24;
					GtkWidget* widget;
					_tmp24 = NULL;
					widget = (_tmp24 = (GtkWidget*) widget_it->data, (_tmp24 == NULL) ? NULL : g_object_ref (_tmp24));
					{
						GnomenuMenuItem* _tmp23;
						GtkWidget* _tmp22;
						GnomenuMenuItem* item;
						WnckWindow* window;
						_tmp23 = NULL;
						_tmp22 = NULL;
						item = (_tmp23 = (_tmp22 = widget, GNOMENU_IS_MENU_ITEM (_tmp22) ? ((GnomenuMenuItem*) _tmp22) : NULL), (_tmp23 == NULL) ? NULL : g_object_ref (_tmp23));
						if (item == NULL) {
							(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
							(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
							continue;
						}
						gnomenu_menu_item_set_max_width_chars (item, 30);
						/* Some sane value?
						libgnomenu's default
						is 30*/
						window = switcher_item_to_window (self, item);
						if (window == NULL) {
							(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
							(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
							(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
							continue;
						}
						(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
						(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
						(window == NULL) ? NULL : (window = (g_object_unref (window), NULL));
					}
				}
				(widget_collection == NULL) ? NULL : (widget_collection = (g_list_free (widget_collection), NULL));
			}
			(menu == NULL) ? NULL : (menu = (g_object_unref (menu), NULL));
		} else {
			/*if (window.is_active()) {
			setup_window_actions_menu(
			"/switcher/" + item.item_id + "/", 
			window);
			}*/
			if (self->priv->_show_window_actions) {
				char* _tmp26;
				char* _tmp25;
				_tmp26 = NULL;
				_tmp25 = NULL;
				s = (_tmp26 = switcher_replace (self, s, "%submenu%", _tmp25 = switcher_do_action_menu (self, self->priv->_current_window)), s = (g_free (s), NULL), _tmp26);
				_tmp25 = (g_free (_tmp25), NULL);
				gnomenu_parser_parse ((GnomenuShell*) self, s, &inner_error);
				if (inner_error != NULL) {
					(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
					s = (g_free (s), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
					g_clear_error (&inner_error);
					return;
				}
			}
		}
	} else {
		char* _tmp27;
		/*setup_window_actions_menu("/switcher/", _current_window);*/
		_tmp27 = NULL;
		s = (_tmp27 = switcher_replace (self, s, "%submenu%", "<menu/>"), s = (g_free (s), NULL), _tmp27);
		gnomenu_parser_parse ((GnomenuShell*) self, s, &inner_error);
		if (inner_error != NULL) {
			(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
			s = (g_free (s), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
			return;
		}
	}
	(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
	s = (g_free (s), NULL);
}


/*private void setup_window_actions_menu(string prefix, Wnck.Window window) {
string[] actions = {"minimize",
"maximize",
"unmaximize",
"move",
"resize",
"topmost",
"untopmost",
"stick",
"unstick",
"workspaces",
"close"};
for (int co=0; co<actions.length; co++) {
Gnomenu.MenuItem si = this.get(prefix +actions[co]);
if(si != null) {
override_item_window(si, window);
}
}
}*/
static gboolean switcher_guess_dock_is_around (Switcher* self) {
	GList* windows;
	g_return_val_if_fail (self != NULL, FALSE);
	windows = wnck_screen_get_windows (wnck_screen_get_default ());
	{
		GList* window_collection;
		GList* window_it;
		window_collection = windows;
		for (window_it = window_collection; window_it != NULL; window_it = window_it->next) {
			WnckWindow* window;
			window = (WnckWindow*) window_it->data;
			{
				if (wnck_window_get_window_type (window) == WNCK_WINDOW_DOCK) {
					GQuark _tmp2;
					char* _tmp1;
					_tmp1 = NULL;
					static GQuark _tmp2_label0 = 0;
					static GQuark _tmp2_label1 = 0;
					static GQuark _tmp2_label2 = 0;
					_tmp1 = wnck_application_get_name (wnck_window_get_application (window));
					_tmp2 = (NULL == _tmp1) ? 0 : g_quark_from_string (_tmp1);
					if (((_tmp2 == ((0 != _tmp2_label0) ? _tmp2_label0 : (_tmp2_label0 = g_quark_from_static_string ("cairo-dock")))) || (_tmp2 == ((0 != _tmp2_label1) ? _tmp2_label1 : (_tmp2_label1 = g_quark_from_static_string ("avant-window-navigator"))))) || (_tmp2 == ((0 != _tmp2_label2) ? _tmp2_label2 : (_tmp2_label2 = g_quark_from_static_string ("Do")))))
					do {
						return TRUE;
						break;
					} while (0);
				}
			}
		}
	}
	return FALSE;
}


/**
 * The following 3 functions map item to window and verse vesa
 *
 */
static GnomenuMenuItem* switcher_window_to_item (Switcher* self, WnckWindow* window) {
	char* _tmp1;
	char* _tmp0;
	GnomenuMenuItem* _tmp2;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (window != NULL, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	_tmp2 = NULL;
	return (_tmp2 = gnomenu_menu_bar_get ((GnomenuMenuBar*) self, _tmp1 = g_strconcat ("/switcher/XID", _tmp0 = g_strdup_printf ("%lu", wnck_window_get_xid (window)), NULL)), _tmp1 = (g_free (_tmp1), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp2);
}


/*private void override_item_window(Gnomenu.MenuItem item, Wnck.Window? window) {
item.user_data = window;
}*/
static WnckWindow* switcher_item_to_window (Switcher* self, GnomenuMenuItem* item) {
	const char* _tmp0;
	char* id;
	WnckWindow* _tmp3;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (item != NULL, NULL);
	_tmp0 = NULL;
	id = (_tmp0 = gnomenu_item_get_item_id ((GnomenuItem*) item), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	/*if(item.user_data != null) return item.user_data as Wnck.Window;*/
	if (g_str_has_prefix (id, "XID")) {
		WnckWindow* _tmp1;
		WnckWindow* _tmp2;
		_tmp1 = NULL;
		_tmp2 = NULL;
		return (_tmp2 = (_tmp1 = wnck_window_get (strtoul (g_utf8_offset_to_pointer (id, (glong) 3), NULL, 0)), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), id = (g_free (id), NULL), _tmp2);
	}
	_tmp3 = NULL;
	return (_tmp3 = NULL, id = (g_free (id), NULL), _tmp3);
}


static void switcher_real_size_request (GtkWidget* base, GtkRequisition* req) {
	Switcher * self;
	gboolean _tmp0;
	self = (Switcher*) base;
	GTK_WIDGET_CLASS (switcher_parent_class)->size_request ((GtkWidget*) GNOMENU_MENU_BAR (self), &(*req));
	_tmp0 = FALSE;
	if (switcher_get_max_size (self) > 0) {
		_tmp0 = (*req).width > switcher_get_max_size (self);
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		/* if max_size == -1, we don't truncate it*/
		(*req).width = switcher_get_max_size (self);
	}
}


WnckWindow* switcher_get_current_window (Switcher* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_current_window;
}


void switcher_set_current_window (Switcher* self, WnckWindow* value) {
	WnckWindow* _tmp2;
	WnckWindow* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_current_window = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL)), _tmp2);
	if (!switcher_guess_dock_is_around (self)) {
		switcher_set_iconify_destination (self, self->priv->_current_window);
	}
	/* always refresh !*/
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "current-window");
}


gint switcher_get_max_size (Switcher* self) {
	g_return_val_if_fail (self != NULL, 0);
	return self->priv->_max_size;
}


void switcher_set_max_size (Switcher* self, gint value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_max_size == value) {
		return;
	}
	self->priv->_max_size = value;
	if (self->priv->_max_size < 40) {
		self->priv->_max_size = -1;
	}
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "max-size");
}


gboolean switcher_get_show_icon (Switcher* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_show_icon;
}


void switcher_set_show_icon (Switcher* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_show_icon == value) {
		return;
	}
	self->priv->_show_icon = value;
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "show-icon");
}


gboolean switcher_get_show_label (Switcher* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_show_label;
}


void switcher_set_show_label (Switcher* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_show_label == value) {
		return;
	}
	self->priv->_show_label = value;
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "show-label");
}


gboolean switcher_get_show_window_list (Switcher* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_show_window_list;
}


void switcher_set_show_window_list (Switcher* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_show_window_list == value) {
		return;
	}
	self->priv->_show_window_list = value;
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "show-window-list");
}


gboolean switcher_get_show_window_actions (Switcher* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_show_window_actions;
}


void switcher_set_show_window_actions (Switcher* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_show_window_actions == value) {
		return;
	}
	self->priv->_show_window_actions = value;
	switcher_update (self, TRUE);
	g_object_notify ((GObject *) self, "show-window-actions");
}


static void _switcher_on_activate_gnomenu_menu_bar_activate (Switcher* _sender, GnomenuMenuItem* item, gpointer self) {
	switcher_on_activate (self, _sender, item);
}


static void _switcher_on_deactivate_gtk_menu_shell_deactivate (Switcher* _sender, gpointer self) {
	switcher_on_deactivate (self);
}


static GObject * switcher_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	SwitcherClass * klass;
	GObjectClass * parent_class;
	Switcher * self;
	GError * inner_error;
	klass = SWITCHER_CLASS (g_type_class_peek (TYPE_SWITCHER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = SWITCHER (obj);
	inner_error = NULL;
	{
		char* _tmp3;
		char* _tmp4;
		char** _tmp17;
		gint substs_size;
		gint substs_length1;
		char** _tmp16;
		const char* _tmp15;
		const char* _tmp14;
		const char* _tmp13;
		const char* _tmp12;
		const char* _tmp11;
		const char* _tmp10;
		const char* _tmp9;
		const char* _tmp8;
		const char* _tmp7;
		const char* _tmp6;
		const char* _tmp5;
		char** substs;
		char* _tmp18;
		{
			char* s;
			char* _tmp0;
			GnomenuMenuItem* _tmp2;
			GnomenuItem* _tmp1;
			s = switcher_replace (self, SWITCHER_MENU_TEMPLATE, "%label%", "Global Menu Bar");
			_tmp0 = NULL;
			s = (_tmp0 = switcher_replace (self, s, "%sub-menu%", ""), s = (g_free (s), NULL), _tmp0);
			gnomenu_parser_parse ((GnomenuShell*) self, s, &inner_error);
			if (inner_error != NULL) {
				s = (g_free (s), NULL);
				goto __catch7_g_error;
				goto __finally7;
			}
			_tmp2 = NULL;
			_tmp1 = NULL;
			self->priv->_label_item = (_tmp2 = (_tmp1 = gnomenu_shell_get_item ((GnomenuShell*) self, 0), GNOMENU_IS_MENU_ITEM (_tmp1) ? ((GnomenuMenuItem*) _tmp1) : NULL), (self->priv->_label_item == NULL) ? NULL : (self->priv->_label_item = (g_object_unref (self->priv->_label_item), NULL)), _tmp2);
			gtk_menu_shell_append ((GtkMenuShell*) self->priv->_label_item->_submenu_cache, (GtkWidget*) ((GtkMenuItem*) self->priv->search_box));
			g_signal_connect_object ((GnomenuMenuBar*) self, "activate", (GCallback) _switcher_on_activate_gnomenu_menu_bar_activate, self, 0);
			s = (g_free (s), NULL);
		}
		goto __finally7;
		__catch7_g_error:
		{
			GError * e;
			e = inner_error;
			inner_error = NULL;
			{
				g_warning ("switcher.vala:65: %s", e->message);
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally7:
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		_tmp3 = NULL;
		self->priv->NOWIN_TEMPLATE = (_tmp3 = template_replace_simple (self->priv->NOWIN_TEMPLATE, "%no_windows%", _ ("no windows")), self->priv->NOWIN_TEMPLATE = (g_free (self->priv->NOWIN_TEMPLATE), NULL), _tmp3);
		_tmp4 = NULL;
		self->priv->NOACT_TEMPLATE = (_tmp4 = template_replace_simple (self->priv->NOACT_TEMPLATE, "%no_actions%", _ ("no actions")), self->priv->NOACT_TEMPLATE = (g_free (self->priv->NOACT_TEMPLATE), NULL), _tmp4);
		_tmp17 = NULL;
		_tmp16 = NULL;
		_tmp15 = NULL;
		_tmp14 = NULL;
		_tmp13 = NULL;
		_tmp12 = NULL;
		_tmp11 = NULL;
		_tmp10 = NULL;
		_tmp9 = NULL;
		_tmp8 = NULL;
		_tmp7 = NULL;
		_tmp6 = NULL;
		_tmp5 = NULL;
		substs = (_tmp17 = (_tmp16 = g_new0 (char*, 46 + 1), _tmp16[0] = g_strdup ("%labelminimize%"), _tmp16[1] = (_tmp5 = _ ("Mi_nimize"), (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), _tmp16[2] = g_strdup ("%visiblemaximize%"), _tmp16[3] = g_strdup ("%visiblemaximize%"), _tmp16[4] = g_strdup ("%labelmaximize%"), _tmp16[5] = (_tmp6 = _ ("Ma_ximize"), (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), _tmp16[6] = g_strdup ("%visibleminimize%"), _tmp16[7] = g_strdup ("%visibleminimize%"), _tmp16[8] = g_strdup ("%labelunmaximize%"), _tmp16[9] = (_tmp7 = _ ("Unma_ximize"), (_tmp7 == NULL) ? NULL : g_strdup (_tmp7)), _tmp16[10] = g_strdup ("%visibleunmaximize%"), _tmp16[11] = g_strdup ("%visibleunmaximize%"), _tmp16[12] = g_strdup ("%labelmove%"), _tmp16[13] = (_tmp8 = _ ("_Move"), (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), _tmp16[14] = g_strdup ("%visiblemove%"), _tmp16[15] = g_strdup ("%visiblemove%"), _tmp16[16] = g_strdup ("%labelresize%"), _tmp16[17] = (_tmp9 = _ ("_Resize"), (_tmp9 == NULL) ? NULL : g_strdup (_tmp9)), _tmp16[18] = g_strdup ("%visibleresize"), _tmp16[19] = g_strdup ("%visibleresize%"), _tmp16[20] = g_strdup ("%visibleseparator1%"), _tmp16[21] = g_strdup ("%visibleseparator1%"), _tmp16[22] = g_strdup ("%labeluntopmost%"), _tmp16[23] = (_tmp10 = _ ("Always on _Top"), (_tmp10 == NULL) ? NULL : g_strdup (_tmp10)), _tmp16[24] = g_strdup ("%visibleuntopmost%"), _tmp16[25] = g_strdup ("%visibleuntopmost%"), _tmp16[26] = g_strdup ("%labeltopmost%"), _tmp16[27] = (_tmp11 = _ ("Always on _Top"), (_tmp11 == NULL) ? NULL : g_strdup (_tmp11)), _tmp16[28] = g_strdup ("%visibletopmost%"), _tmp16[29] = g_strdup ("%visibletopmost%"), _tmp16[30] = g_strdup ("%labelunstick%"), _tmp16[31] = (_tmp12 = _ ("_Always on visible Workspace"), (_tmp12 == NULL) ? NULL : g_strdup (_tmp12)), _tmp16[32] = g_strdup ("%visibleunstick%"), _tmp16[33] = g_strdup ("%visibleunstick%"), _tmp16[34] = g_strdup ("%labelstick%"), _tmp16[35] = (_tmp13 = _ ("_Always on visible Workspace"), (_tmp13 == NULL) ? NULL : g_strdup (_tmp13)), _tmp16[36] = g_strdup ("%visiblestick%"), _tmp16[37] = g_strdup ("%visiblestick%"), _tmp16[38] = g_strdup ("%labelworkspaces%"), _tmp16[39] = (_tmp14 = _ ("_Workspace..."), (_tmp14 == NULL) ? NULL : g_strdup (_tmp14)), _tmp16[40] = g_strdup ("%visibleseparator2%"), _tmp16[41] = g_strdup ("%visibleseparator2%"), _tmp16[42] = g_strdup ("%labelclose%"), _tmp16[43] = (_tmp15 = _ ("_Close"), (_tmp15 == NULL) ? NULL : g_strdup (_tmp15)), _tmp16[44] = g_strdup ("%visibleclose%"), _tmp16[45] = g_strdup ("%visibleclose%"), _tmp16), substs_length1 = 46, substs_size = substs_length1, _tmp17);
		_tmp18 = NULL;
		self->priv->WINACT_TEMPLATE = (_tmp18 = template_replace (self->priv->WINACT_TEMPLATE, substs, substs_length1), self->priv->WINACT_TEMPLATE = (g_free (self->priv->WINACT_TEMPLATE), NULL), _tmp18);
		/* fixes a rendering issue with Compiz */
		g_signal_connect_object ((GtkMenuShell*) self, "deactivate", (GCallback) _switcher_on_deactivate_gtk_menu_shell_deactivate, self, 0);
		substs = (_vala_array_free (substs, substs_length1, (GDestroyNotify) g_free), NULL);
	}
	return obj;
}


static void switcher_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Switcher * self;
	gpointer boxed;
	self = SWITCHER (object);
	switch (property_id) {
		case SWITCHER_CURRENT_WINDOW:
		g_value_set_object (value, switcher_get_current_window (self));
		break;
		case SWITCHER_MAX_SIZE:
		g_value_set_int (value, switcher_get_max_size (self));
		break;
		case SWITCHER_SHOW_ICON:
		g_value_set_boolean (value, switcher_get_show_icon (self));
		break;
		case SWITCHER_SHOW_LABEL:
		g_value_set_boolean (value, switcher_get_show_label (self));
		break;
		case SWITCHER_SHOW_WINDOW_LIST:
		g_value_set_boolean (value, switcher_get_show_window_list (self));
		break;
		case SWITCHER_SHOW_WINDOW_ACTIONS:
		g_value_set_boolean (value, switcher_get_show_window_actions (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void switcher_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	Switcher * self;
	self = SWITCHER (object);
	switch (property_id) {
		case SWITCHER_CURRENT_WINDOW:
		switcher_set_current_window (self, g_value_get_object (value));
		break;
		case SWITCHER_MAX_SIZE:
		switcher_set_max_size (self, g_value_get_int (value));
		break;
		case SWITCHER_SHOW_ICON:
		switcher_set_show_icon (self, g_value_get_boolean (value));
		break;
		case SWITCHER_SHOW_LABEL:
		switcher_set_show_label (self, g_value_get_boolean (value));
		break;
		case SWITCHER_SHOW_WINDOW_LIST:
		switcher_set_show_window_list (self, g_value_get_boolean (value));
		break;
		case SWITCHER_SHOW_WINDOW_ACTIONS:
		switcher_set_show_window_actions (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void switcher_class_init (SwitcherClass * klass) {
	switcher_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SwitcherPrivate));
	G_OBJECT_CLASS (klass)->get_property = switcher_get_property;
	G_OBJECT_CLASS (klass)->set_property = switcher_set_property;
	G_OBJECT_CLASS (klass)->constructor = switcher_constructor;
	G_OBJECT_CLASS (klass)->finalize = switcher_finalize;
	G_OBJECT_CLASS (klass)->dispose = switcher_real_dispose;
	GTK_WIDGET_CLASS (klass)->size_request = switcher_real_size_request;
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_CURRENT_WINDOW, g_param_spec_object ("current-window", "current-window", "current-window", WNCK_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_MAX_SIZE, g_param_spec_int ("max-size", "max-size", "max-size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_SHOW_ICON, g_param_spec_boolean ("show-icon", "show-icon", "show-icon", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_SHOW_LABEL, g_param_spec_boolean ("show-label", "show-label", "show-label", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_SHOW_WINDOW_LIST, g_param_spec_boolean ("show-window-list", "show-window-list", "show-window-list", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), SWITCHER_SHOW_WINDOW_ACTIONS, g_param_spec_boolean ("show-window-actions", "show-window-actions", "show-window-actions", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	{
		GType type;
		GTypeClass* _tmp19;
		/* To load wnck-stock-delete
		 * wnck-stock-maximize
		 * wnck-stock-minimize
		 * */
		type = WNCK_TYPE_ACTION_MENU;
		_tmp19 = NULL;
		_tmp19 = g_type_class_ref (type);
		(_tmp19 == NULL) ? NULL : (_tmp19 = (g_type_class_unref (_tmp19), NULL));
	}
}


static void switcher_instance_init (Switcher * self) {
	self->priv = SWITCHER_GET_PRIVATE (self);
	self->priv->_max_size = -1;
	self->priv->_show_icon = FALSE;
	self->priv->_show_label = TRUE;
	self->priv->_show_window_list = TRUE;
	self->priv->_show_window_actions = TRUE;
	self->priv->NOWIN_TEMPLATE = g_strdup ("<item label=\"%no_windows%\" type=\"image\" icon=\"theme:gtk-about\" sensitive=\"false\" font=\"italic\"/>");
	self->priv->NOACT_TEMPLATE = g_strdup ("<item label=\"%no_actions%\" type=\"image\" icon=\"theme:gtk-about\" sensitive=\"false\" font=\"italic\"/>");
	self->priv->WINACT_TEMPLATE = g_strdup ("<menu>\n\t<item id=\"maximize\" type=\"image\" icon=\"wnck-stock-maximize\" label=\"%labelmaximize%\" visible=\"%visiblemaximize%\" />\n\t<item id=\"minimize\" type=\"image\" icon=\"wnck-stock-minimize\" label=\"%labelminimize%\" visible=\"%visibleminimize%\" />\n\t<item id=\"unmaximize\" type=\"normal\" label=\"%labelunmaximize%\" visible=\"%visibleunmaximize%\" />\n\t<item id=\"move\" type=\"normal\" label=\"%labelmove%\" visible=\"%visiblemove%\" />\n\t<item id=\"resize\" type=\"normal\" label=\"%labelresize%\" visible=\"%visibleresize%\" />\n\t<item id=\"separator1\" type=\"separator\" visible=\"%visibleseparator1%\" />\n\t<item id=\"untopmost\" type=\"check\" state=\"toggled\" label=\"%labeluntopmost%\" visible=\"%visibleuntopmost%\" />\n\t<item id=\"topmost\" type=\"check\" state=\"untoggled\" label=\"%labeltopmost%\" visible=\"%visibletopmost%\" />\n\t<item id=\"unstick\" type=\"check\" state=\"toggled\" label=\"%labelunstick%\" visible=\"%visibleunstick%\" />\n\t<item id=\"stick\" type=\"check\" state=\"untoggled\" label=\"%labelstick%\" visible=\"%visiblestick%\" />\n\t<item id=\"workspaces\" type=\"normal\" label=\"%labelworkspaces%\" />\n\t<item id=\"separator2\" type=\"separator\" visible=\"%visibleseparator2%\" />\n\t<item id=\"close\" type=\"image\" icon=\"wnck-stock-delete\" label=\"%labelclose%\" visible=\"%visibleclose%\" />\n</menu>");
	self->priv->search_box = g_object_ref_sink (search_box_menu_item_new ());
	self->priv->disposed = FALSE;
}


static void switcher_finalize (GObject* obj) {
	Switcher * self;
	self = SWITCHER (obj);
	self->priv->_label = (g_free (self->priv->_label), NULL);
	(self->priv->_label_item == NULL) ? NULL : (self->priv->_label_item = (g_object_unref (self->priv->_label_item), NULL));
	(self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL));
	self->priv->NOWIN_TEMPLATE = (g_free (self->priv->NOWIN_TEMPLATE), NULL);
	self->priv->NOACT_TEMPLATE = (g_free (self->priv->NOACT_TEMPLATE), NULL);
	self->priv->WINACT_TEMPLATE = (g_free (self->priv->WINACT_TEMPLATE), NULL);
	(self->priv->search_box == NULL) ? NULL : (self->priv->search_box = (g_object_unref (self->priv->search_box), NULL));
	G_OBJECT_CLASS (switcher_parent_class)->finalize (obj);
}


GType switcher_get_type (void) {
	static GType switcher_type_id = 0;
	if (switcher_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SwitcherClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) switcher_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Switcher), 0, (GInstanceInitFunc) switcher_instance_init, NULL };
		switcher_type_id = g_type_register_static (GNOMENU_TYPE_MENU_BAR, "Switcher", &g_define_type_info, 0);
	}
	return switcher_type_id;
}


SearchBoxMenuItem* search_box_menu_item_construct (GType object_type) {
	SearchBoxMenuItem * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


SearchBoxMenuItem* search_box_menu_item_new (void) {
	return search_box_menu_item_construct (TYPE_SEARCH_BOX_MENU_ITEM);
}


GnomenuMenuItem* search_box_menu_item_find_item (SearchBoxMenuItem* self) {
	GnomenuShell* _tmp1;
	GtkContainer* _tmp0;
	GnomenuShell* shell;
	GnomenuMenuItem* _tmp9;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	shell = (_tmp1 = (_tmp0 = gtk_widget_get_parent ((GtkWidget*) self), GNOMENU_IS_SHELL (_tmp0) ? ((GnomenuShell*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	{
		gint i;
		i = 0;
		for (; i < gnomenu_shell_get_length (shell); i++) {
			GnomenuItem* _tmp2;
			GnomenuMenuItem* item;
			gboolean _tmp3;
			_tmp2 = NULL;
			item = (_tmp2 = gnomenu_shell_get_item (shell, i), GNOMENU_IS_MENU_ITEM (_tmp2) ? ((GnomenuMenuItem*) _tmp2) : NULL);
			_tmp3 = FALSE;
			if (gnomenu_item_get_item_label ((GnomenuItem*) item) != NULL) {
				char* _tmp5;
				char* _tmp4;
				_tmp5 = NULL;
				_tmp4 = NULL;
				_tmp3 = g_str_has_prefix (_tmp4 = g_utf8_casefold (gnomenu_item_get_item_label ((GnomenuItem*) item), -1), _tmp5 = g_utf8_casefold (gtk_entry_get_text (self->priv->textbox), -1));
				_tmp5 = (g_free (_tmp5), NULL);
				_tmp4 = (g_free (_tmp4), NULL);
			} else {
				_tmp3 = FALSE;
			}
			if (_tmp3) {
				GnomenuMenuItem* _tmp7;
				GnomenuMenuItem* _tmp6;
				GnomenuMenuItem* _tmp8;
				_tmp7 = NULL;
				_tmp6 = NULL;
				_tmp8 = NULL;
				return (_tmp8 = (_tmp7 = (_tmp6 = item, GNOMENU_IS_MENU_ITEM (_tmp6) ? ((GnomenuMenuItem*) _tmp6) : NULL), (_tmp7 == NULL) ? NULL : g_object_ref (_tmp7)), (item == NULL) ? NULL : (item = (g_object_unref (item), NULL)), (shell == NULL) ? NULL : (shell = (g_object_unref (shell), NULL)), _tmp8);
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
	}
	_tmp9 = NULL;
	return (_tmp9 = NULL, (shell == NULL) ? NULL : (shell = (g_object_unref (shell), NULL)), _tmp9);
}


static gboolean _search_box_menu_item_steal_key_press_event_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self) {
	return search_box_menu_item_steal_key_press_event (self, _sender, event);
}


static void search_box_menu_item_real_parent_set (GtkWidget* base, GtkWidget* previous_parent) {
	SearchBoxMenuItem * self;
	self = (SearchBoxMenuItem*) base;
	if (previous_parent != NULL) {
		guint _tmp0;
		g_signal_handlers_disconnect_matched (previous_parent, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("key-press-event", GTK_TYPE_WIDGET, &_tmp0, NULL, FALSE), _tmp0), 0, NULL, (GCallback) _search_box_menu_item_steal_key_press_event_gtk_widget_key_press_event, self);
	}
	GTK_WIDGET_CLASS (search_box_menu_item_parent_class)->parent_set ((GtkWidget*) GTK_IMAGE_MENU_ITEM (self), previous_parent);
	if (gtk_widget_get_parent ((GtkWidget*) self) != NULL) {
		g_signal_connect_object ((GtkWidget*) gtk_widget_get_parent ((GtkWidget*) self), "key-press-event", (GCallback) _search_box_menu_item_steal_key_press_event_gtk_widget_key_press_event, self, 0);
	}
}


static gboolean search_box_menu_item_steal_key_press_event (SearchBoxMenuItem* self, GtkWidget* widget, const GdkEventKey* event) {
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (widget != NULL, FALSE);
	return (g_signal_emit_by_name ((GtkWidget*) self->priv->textbox, "key-press-event", &(*event), &_tmp0), _tmp0);
}


static void search_box_menu_item_real_activate_item (GtkMenuItem* base) {
	SearchBoxMenuItem * self;
	self = (SearchBoxMenuItem*) base;
}


static void search_box_menu_item_real_activate (GtkMenuItem* base) {
	SearchBoxMenuItem * self;
	self = (SearchBoxMenuItem*) base;
}


const char* search_box_menu_item_get_text (SearchBoxMenuItem* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return gtk_entry_get_text (self->priv->textbox);
}


void search_box_menu_item_set_text (SearchBoxMenuItem* self, const char* value) {
	g_return_if_fail (self != NULL);
	gtk_entry_set_text (self->priv->textbox, value);
	g_object_notify ((GObject *) self, "text");
}


static void __lambda0 (GtkEntry* box, GtkStateType previous_state, SearchBoxMenuItem* self) {
	g_return_if_fail (box != NULL);
	((GtkWidget*) self->priv->textbox)->state = (guchar) GTK_STATE_NORMAL;
}


static void ___lambda0_gtk_widget_state_changed (GtkEntry* _sender, GtkStateType previous_state, gpointer self) {
	__lambda0 (_sender, previous_state, self);
}


static void __lambda1 (GtkEntry* box, SearchBoxMenuItem* self) {
	GnomenuMenu* _tmp1;
	GtkContainer* _tmp0;
	GnomenuMenu* shell;
	GnomenuMenuItem* item;
	g_return_if_fail (box != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	shell = (_tmp1 = (_tmp0 = gtk_widget_get_parent ((GtkWidget*) self), GNOMENU_IS_MENU (_tmp0) ? ((GnomenuMenu*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	item = search_box_menu_item_find_item (self);
	if (item != NULL) {
		gtk_menu_shell_select_item ((GtkMenuShell*) shell, (GtkWidget*) item);
	}
	(shell == NULL) ? NULL : (shell = (g_object_unref (shell), NULL));
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
}


static void ___lambda1_gtk_editable_changed (GtkEntry* _sender, gpointer self) {
	__lambda1 (_sender, self);
}


static void __lambda2 (GtkEntry* box, SearchBoxMenuItem* self) {
	GnomenuMenu* _tmp1;
	GtkContainer* _tmp0;
	GnomenuMenu* shell;
	GnomenuMenuItem* item;
	g_return_if_fail (box != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	shell = (_tmp1 = (_tmp0 = gtk_widget_get_parent ((GtkWidget*) self), GNOMENU_IS_MENU (_tmp0) ? ((GnomenuMenu*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	item = search_box_menu_item_find_item (self);
	if (item != NULL) {
		gtk_menu_shell_activate_item ((GtkMenuShell*) shell, (GtkWidget*) item, TRUE);
	}
	(shell == NULL) ? NULL : (shell = (g_object_unref (shell), NULL));
	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
}


static void ___lambda2_gtk_entry_activate (GtkEntry* _sender, gpointer self) {
	__lambda2 (_sender, self);
}


static void __lambda3 (SearchBoxMenuItem* _this, GtkStateType previous_state, SearchBoxMenuItem* self) {
	g_return_if_fail (_this != NULL);
	((GtkWidget*) self)->state = (guchar) GTK_STATE_NORMAL;
}


static void ___lambda3_gtk_widget_state_changed (SearchBoxMenuItem* _sender, GtkStateType previous_state, gpointer self) {
	__lambda3 (_sender, previous_state, self);
}


static GObject * search_box_menu_item_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	SearchBoxMenuItemClass * klass;
	GObjectClass * parent_class;
	SearchBoxMenuItem * self;
	klass = SEARCH_BOX_MENU_ITEM_CLASS (g_type_class_peek (TYPE_SEARCH_BOX_MENU_ITEM));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = SEARCH_BOX_MENU_ITEM (obj);
	{
		GtkImage* _tmp20;
		_tmp20 = NULL;
		gtk_image_menu_item_set_image ((GtkImageMenuItem*) self, (GtkWidget*) (_tmp20 = g_object_ref_sink ((GtkImage*) gtk_image_new_from_icon_name ("search", GTK_ICON_SIZE_MENU))));
		(_tmp20 == NULL) ? NULL : (_tmp20 = (g_object_unref (_tmp20), NULL));
		g_object_set (self->priv->textbox, "editable", TRUE, NULL);
		g_object_set ((GtkWidget*) self->priv->textbox, "sensitive", TRUE, NULL);
		g_object_set ((GtkWidget*) self->priv->textbox, "can-focus", TRUE, NULL);
		gtk_entry_set_text (self->priv->textbox, "Type here...");
		g_object_set ((GtkWidget*) self->priv->textbox, "is-focus", TRUE, NULL);
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->textbox);
		gtk_widget_show_all ((GtkWidget*) self);
		gtk_container_set_focus_child ((GtkContainer*) self, (GtkWidget*) self->priv->textbox);
		((GtkWidget*) self->priv->textbox)->state = (guchar) GTK_STATE_NORMAL;
		g_signal_connect ((GtkWidget*) self->priv->textbox, "state-changed", (GCallback) ___lambda0_gtk_widget_state_changed, self);
		g_signal_connect ((GtkEditable*) self->priv->textbox, "changed", (GCallback) ___lambda1_gtk_editable_changed, self);
		g_signal_connect (self->priv->textbox, "activate", (GCallback) ___lambda2_gtk_entry_activate, self);
		g_signal_connect ((GtkWidget*) self, "state-changed", (GCallback) ___lambda3_gtk_widget_state_changed, self);
	}
	return obj;
}


static void search_box_menu_item_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	SearchBoxMenuItem * self;
	gpointer boxed;
	self = SEARCH_BOX_MENU_ITEM (object);
	switch (property_id) {
		case SEARCH_BOX_MENU_ITEM_TEXT:
		g_value_set_string (value, search_box_menu_item_get_text (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void search_box_menu_item_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	SearchBoxMenuItem * self;
	self = SEARCH_BOX_MENU_ITEM (object);
	switch (property_id) {
		case SEARCH_BOX_MENU_ITEM_TEXT:
		search_box_menu_item_set_text (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void search_box_menu_item_class_init (SearchBoxMenuItemClass * klass) {
	search_box_menu_item_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (SearchBoxMenuItemPrivate));
	G_OBJECT_CLASS (klass)->get_property = search_box_menu_item_get_property;
	G_OBJECT_CLASS (klass)->set_property = search_box_menu_item_set_property;
	G_OBJECT_CLASS (klass)->constructor = search_box_menu_item_constructor;
	G_OBJECT_CLASS (klass)->finalize = search_box_menu_item_finalize;
	GTK_WIDGET_CLASS (klass)->parent_set = search_box_menu_item_real_parent_set;
	GTK_MENU_ITEM_CLASS (klass)->activate_item = search_box_menu_item_real_activate_item;
	GTK_MENU_ITEM_CLASS (klass)->activate = search_box_menu_item_real_activate;
	g_object_class_install_property (G_OBJECT_CLASS (klass), SEARCH_BOX_MENU_ITEM_TEXT, g_param_spec_string ("text", "text", "text", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void search_box_menu_item_instance_init (SearchBoxMenuItem * self) {
	self->priv = SEARCH_BOX_MENU_ITEM_GET_PRIVATE (self);
	self->priv->textbox = g_object_ref_sink ((GtkEntry*) gtk_entry_new ());
}


static void search_box_menu_item_finalize (GObject* obj) {
	SearchBoxMenuItem * self;
	self = SEARCH_BOX_MENU_ITEM (obj);
	(self->priv->textbox == NULL) ? NULL : (self->priv->textbox = (g_object_unref (self->priv->textbox), NULL));
	G_OBJECT_CLASS (search_box_menu_item_parent_class)->finalize (obj);
}


GType search_box_menu_item_get_type (void) {
	static GType search_box_menu_item_type_id = 0;
	if (search_box_menu_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (SearchBoxMenuItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) search_box_menu_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (SearchBoxMenuItem), 0, (GInstanceInitFunc) search_box_menu_item_instance_init, NULL };
		search_box_menu_item_type_id = g_type_register_static (GTK_TYPE_IMAGE_MENU_ITEM, "SearchBoxMenuItem", &g_define_type_info, 0);
	}
	return search_box_menu_item_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




