
#include "workspaceselector.h"
#include <gdk/gdkx.h>
#include <float.h>
#include <math.h>
#include <glib/gi18n-lib.h>
#include <stdlib.h>
#include <string.h>




struct _WorkspaceSelectorPrivate {
	WnckWindow* target;
	GList* workspaces;
	GtkCheckButton* follow;
	guint iid;
};

#define WORKSPACE_SELECTOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_WORKSPACE_SELECTOR, WorkspaceSelectorPrivate))
enum  {
	WORKSPACE_SELECTOR_DUMMY_PROPERTY,
	WORKSPACE_SELECTOR_NROWS,
	WORKSPACE_SELECTOR_NCOLS,
	WORKSPACE_SELECTOR_SCREEN_WIDTH,
	WORKSPACE_SELECTOR_SCREEN_HEIGHT,
	WORKSPACE_SELECTOR_WORKSPACE_WIDTH,
	WORKSPACE_SELECTOR_WORKSPACE_HEIGHT,
	WORKSPACE_SELECTOR_CURRENT_COLUMN,
	WORKSPACE_SELECTOR_CURRENT_ROW,
	WORKSPACE_SELECTOR_SELECTED_ITEM
};
static void _g_list_free_g_object_unref (GList* self);
#define WORKSPACE_SELECTOR_WIDTH 128
#define WORKSPACE_SELECTOR_ICON_SCALE_RATIO 6
#define WORKSPACE_SELECTOR_ICON_ALPHA 255
static void workspace_selector_real_dispose (GObject* base);
static gboolean workspace_selector_on_leave_notify (WorkspaceSelector* self, WorkspaceSelector* wss, const GdkEventCrossing* event);
static gboolean workspace_selector_on_button_press (WorkspaceSelector* self, WorkspaceSelector* wss, const GdkEventButton* event);
static gboolean _workspace_selector_on_viewport_moved_gsource_func (gpointer self);
static gboolean workspace_selector_on_button_release (WorkspaceSelector* self, WorkspaceSelector* wss, const GdkEventButton* event);
static gboolean workspace_selector_on_viewport_moved (WorkspaceSelector* self);
static void workspace_selector_do_menu (WorkspaceSelector* self);
static GdkPixbuf* workspace_selector_get_mini_screenshot (WorkspaceSelector* self, gint col, gint row);
static void workspace_selector_get_window_abs_geometry (WorkspaceSelector* self, WnckWindow* window, gint* x, gint* y, gint* w, gint* h);
static gboolean workspace_selector_window_visible_on_desktop (WorkspaceSelector* self, WnckWindow* window, gint col, gint row);
static gboolean _workspace_selector_on_leave_notify_gtk_widget_leave_notify_event (WorkspaceSelector* _sender, const GdkEventCrossing* event, gpointer self);
static gboolean _workspace_selector_on_button_press_gtk_widget_button_press_event (WorkspaceSelector* _sender, const GdkEventButton* event, gpointer self);
static gboolean _workspace_selector_on_button_release_gtk_widget_button_release_event (WorkspaceSelector* _sender, const GdkEventButton* event, gpointer self);
static GObject * workspace_selector_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer workspace_selector_parent_class = NULL;
static void workspace_selector_finalize (GObject* obj);
struct _WorkspaceItemPrivate {
	GtkImage* _image;
	gint _margin;
	gboolean _selected;
	gboolean _current;
};

#define WORKSPACE_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_WORKSPACE_ITEM, WorkspaceItemPrivate))
enum  {
	WORKSPACE_ITEM_DUMMY_PROPERTY,
	WORKSPACE_ITEM_IMAGE,
	WORKSPACE_ITEM_MARGIN,
	WORKSPACE_ITEM_SELECTED,
	WORKSPACE_ITEM_CURRENT
};
static void workspace_item_update (WorkspaceItem* self);
static gboolean workspace_item_on_leave_notify (WorkspaceItem* self, WorkspaceItem* wi, const GdkEventCrossing* event);
static gboolean workspace_item_on_enter_notify (WorkspaceItem* self, WorkspaceItem* wi, const GdkEventCrossing* event);
static gboolean _workspace_item_on_leave_notify_gtk_widget_leave_notify_event (WorkspaceItem* _sender, const GdkEventCrossing* event, gpointer self);
static gboolean _workspace_item_on_enter_notify_gtk_widget_enter_notify_event (WorkspaceItem* _sender, const GdkEventCrossing* event, gpointer self);
static GObject * workspace_item_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer workspace_item_parent_class = NULL;
static void workspace_item_finalize (GObject* obj);



static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


WorkspaceSelector* workspace_selector_construct (GType object_type, WnckWindow* window) {
	WorkspaceSelector * self;
	WnckWindow* _tmp1;
	WnckWindow* _tmp0;
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->target = (_tmp1 = (_tmp0 = window, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->target == NULL) ? NULL : (self->priv->target = (g_object_unref (self->priv->target), NULL)), _tmp1);
	workspace_selector_do_menu (self);
	return self;
}


WorkspaceSelector* workspace_selector_new (WnckWindow* window) {
	return workspace_selector_construct (TYPE_WORKSPACE_SELECTOR, window);
}


static void workspace_selector_real_dispose (GObject* base) {
	WorkspaceSelector * self;
	WnckWindow* _tmp0;
	self = (WorkspaceSelector*) base;
	_tmp0 = NULL;
	self->priv->target = (_tmp0 = NULL, (self->priv->target == NULL) ? NULL : (self->priv->target = (g_object_unref (self->priv->target), NULL)), _tmp0);
	G_OBJECT_CLASS (workspace_selector_parent_class)->dispose ((GObject*) GTK_WINDOW (self));
}


static gboolean workspace_selector_on_leave_notify (WorkspaceSelector* self, WorkspaceSelector* wss, const GdkEventCrossing* event) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (wss != NULL, FALSE);
	if (self->priv->iid != 0) {
		return FALSE;
	}
	if ((*event).state == 0) {
		gtk_object_destroy ((GtkObject*) self);
	}
	return FALSE;
}


static gboolean workspace_selector_on_button_press (WorkspaceSelector* self, WorkspaceSelector* wss, const GdkEventButton* event) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (wss != NULL, FALSE);
	return FALSE;
}


static gboolean _workspace_selector_on_viewport_moved_gsource_func (gpointer self) {
	return workspace_selector_on_viewport_moved (self);
}


static gboolean workspace_selector_on_button_release (WorkspaceSelector* self, WorkspaceSelector* wss, const GdkEventButton* event) {
	GdkWindow* _tmp1;
	GdkWindow* window;
	gint x;
	gint y;
	gboolean _tmp3;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (wss != NULL, FALSE);
	if (workspace_selector_get_selected_item (self) == NULL) {
		return FALSE;
	}
	/* move the window! */
	_tmp1 = NULL;
	window = (_tmp1 = GDK_WINDOW (gdk_xid_table_lookup ((guint32) wnck_window_get_xid (self->priv->target))), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	if (window == NULL) {
		gboolean _tmp2;
		return (_tmp2 = FALSE, (window == NULL) ? NULL : (window = (g_object_unref (window), NULL)), _tmp2);
	}
	x = 0;
	y = 0;
	gdk_window_get_root_origin (window, &x, &y);
	x = x + (workspace_selector_get_selected_item (self)->viewport_x - wnck_workspace_get_viewport_x (wnck_window_get_workspace (self->priv->target)));
	y = y + (workspace_selector_get_selected_item (self)->viewport_y - wnck_workspace_get_viewport_y (wnck_window_get_workspace (self->priv->target)));
	gdk_window_move (window, x, y);
	/* perhaps go with it */
	if (gtk_toggle_button_get_active ((GtkToggleButton*) self->priv->follow)) {
		wnck_screen_move_viewport (wnck_window_get_screen (self->priv->target), workspace_selector_get_selected_item (self)->viewport_x, workspace_selector_get_selected_item (self)->viewport_y);
		/* Give Compiz the time to complete viewport moving animation */
		self->priv->iid = g_timeout_add ((guint) 1000, _workspace_selector_on_viewport_moved_gsource_func, self);
	} else {
		gtk_object_destroy ((GtkObject*) self);
	}
	return (_tmp3 = FALSE, (window == NULL) ? NULL : (window = (g_object_unref (window), NULL)), _tmp3);
}


static gboolean workspace_selector_on_viewport_moved (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	wnck_window_activate (self->priv->target, gtk_get_current_event_time ());
	gtk_object_destroy ((GtkObject*) self);
	return FALSE;
}


static void workspace_selector_do_menu (WorkspaceSelector* self) {
	GtkVBox* rows;
	GtkHBox* row;
	GtkCheckButton* _tmp5;
	GtkMenuItem* _tmp6;
	g_return_if_fail (self != NULL);
	rows = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0));
	row = NULL;
	{
		gint nrow;
		nrow = 0;
		for (; nrow < workspace_selector_get_nrows (self); nrow++) {
			GtkHBox* _tmp0;
			_tmp0 = NULL;
			row = (_tmp0 = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (TRUE, 0)), (row == NULL) ? NULL : (row = (g_object_unref (row), NULL)), _tmp0);
			gtk_box_pack_start ((GtkBox*) rows, (GtkWidget*) row, TRUE, TRUE, (guint) 0);
			{
				gint ncol;
				ncol = 0;
				for (; ncol < workspace_selector_get_ncols (self); ncol++) {
					GdkPixbuf* _tmp1;
					WorkspaceItem* _tmp2;
					WorkspaceItem* wi;
					gboolean _tmp3;
					WorkspaceItem* _tmp4;
					_tmp1 = NULL;
					_tmp2 = NULL;
					wi = (_tmp2 = g_object_ref_sink (workspace_item_new_with_pixbuf (_tmp1 = workspace_selector_get_mini_screenshot (self, ncol, nrow))), (_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL)), _tmp2);
					gtk_box_pack_start ((GtkBox*) row, (GtkWidget*) wi, TRUE, TRUE, (guint) 0);
					wi->viewport_x = ncol * workspace_selector_get_screen_width (self);
					wi->viewport_y = nrow * workspace_selector_get_screen_height (self);
					_tmp3 = FALSE;
					if (workspace_selector_get_current_row (self) == nrow) {
						_tmp3 = workspace_selector_get_current_column (self) == ncol;
					} else {
						_tmp3 = FALSE;
					}
					workspace_item_set_current (wi, _tmp3);
					_tmp4 = NULL;
					self->priv->workspaces = g_list_append (self->priv->workspaces, (_tmp4 = wi, (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4)));
					(wi == NULL) ? NULL : (wi = (g_object_unref (wi), NULL));
				}
			}
		}
	}
	_tmp5 = NULL;
	self->priv->follow = (_tmp5 = g_object_ref_sink ((GtkCheckButton*) gtk_check_button_new_with_label (_ ("Follow the window"))), (self->priv->follow == NULL) ? NULL : (self->priv->follow = (g_object_unref (self->priv->follow), NULL)), _tmp5);
	gtk_toggle_button_set_active ((GtkToggleButton*) self->priv->follow, TRUE);
	gtk_box_pack_start ((GtkBox*) rows, (GtkWidget*) self->priv->follow, TRUE, TRUE, (guint) 0);
	_tmp6 = NULL;
	gtk_widget_set_style ((GtkWidget*) self->priv->follow, gtk_rc_get_style ((GtkWidget*) (_tmp6 = g_object_ref_sink ((GtkMenuItem*) gtk_menu_item_new ()))));
	(_tmp6 == NULL) ? NULL : (_tmp6 = (g_object_unref (_tmp6), NULL));
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) rows);
	gtk_widget_show_all ((GtkWidget*) self);
	(rows == NULL) ? NULL : (rows = (g_object_unref (rows), NULL));
	(row == NULL) ? NULL : (row = (g_object_unref (row), NULL));
}


static GdkPixbuf* workspace_selector_get_mini_screenshot (WorkspaceSelector* self, gint col, gint row) {
	GdkPixmap* p;
	GdkPixbuf* ss;
	GList* windows;
	GdkPixbuf* pb;
	double ratio;
	GdkPixbuf* _tmp3;
	g_return_val_if_fail (self != NULL, NULL);
	/* TODO:
	 * Render also windows in selected Desktop */
	p = (GdkPixmap*) wnck_screen_get_background_pixmap (wnck_window_get_screen (self->priv->target));
	/* create fullsize Pixbuf */
	ss = _wnck_gdk_pixbuf_get_from_pixmap (NULL, p, 0, 0, 0, 0, -1, -1);
	/* render visible windows */
	windows = wnck_screen_get_windows (wnck_screen_get_default ());
	/*string buf = "";*/
	{
		GList* window_collection;
		GList* window_it;
		window_collection = windows;
		for (window_it = window_collection; window_it != NULL; window_it = window_it->next) {
			WnckWindow* window;
			window = (WnckWindow*) window_it->data;
			{
				gboolean _tmp0;
				_tmp0 = FALSE;
				if (workspace_selector_window_visible_on_desktop (self, window, col, row)) {
					_tmp0 = !wnck_window_is_skip_pager (window);
				} else {
					_tmp0 = FALSE;
				}
				if (_tmp0) {
					GdkPixbuf* _tmp1;
					GdkPixbuf* mi;
					gint x;
					gint y;
					gint w;
					gint h;
					GdkWindow* _tmp2;
					GdkWindow* gwindow;
					GdkPixbuf* wi;
					_tmp1 = NULL;
					mi = (_tmp1 = wnck_window_get_icon (window), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
					x = 0;
					y = 0;
					w = 0;
					h = 0;
					workspace_selector_get_window_abs_geometry (self, window, &x, &y, &w, &h);
					x = x - (col * workspace_selector_get_screen_width (self));
					y = y - (row * workspace_selector_get_screen_height (self));
					_tmp2 = NULL;
					gwindow = (_tmp2 = GDK_WINDOW (gdk_xid_table_lookup ((guint32) wnck_window_get_xid (window))), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
					/* TODO: Draw windows thumbnails rather than grey rectangles
					 * The following function should help, however it only works
					 * for windows in selected desktop... further investigation is required
					
					Gdk.Pixbuf wi = gdk_pixbuf_get_from_drawable(null,
					  gwindow,
					  null,
					  0, 0,
					  x, y,
					  w, h);*/
					wi = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w, h);
					gdk_pixbuf_fill (wi, (guint32) 0x0000007f);
					gdk_pixbuf_composite (mi, wi, 0, 0, gdk_pixbuf_get_width (mi) * WORKSPACE_SELECTOR_ICON_SCALE_RATIO, gdk_pixbuf_get_height (mi) * WORKSPACE_SELECTOR_ICON_SCALE_RATIO, (double) 0, (double) 0, (double) WORKSPACE_SELECTOR_ICON_SCALE_RATIO, (double) WORKSPACE_SELECTOR_ICON_SCALE_RATIO, GDK_INTERP_NEAREST, WORKSPACE_SELECTOR_ICON_ALPHA);
					gdk_pixbuf_composite (wi, ss, x, y, w, h, (double) x, (double) y, (double) 1, (double) 1, GDK_INTERP_NEAREST, 255);
					(mi == NULL) ? NULL : (mi = (g_object_unref (mi), NULL));
					(gwindow == NULL) ? NULL : (gwindow = (g_object_unref (gwindow), NULL));
					(wi == NULL) ? NULL : (wi = (g_object_unref (wi), NULL));
				}
			}
		}
	}
	/*buf += window.get_application().get_name();
	buf += " " + x.to_string();
	buf += " " + y.to_string();
	buf += "\n";
	GnomeMenuHelper.message(buf);
	 create thumbnail Pixbuf */
	pb = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, WORKSPACE_SELECTOR_WIDTH, (gint) ((((double) WORKSPACE_SELECTOR_WIDTH) / ((double) workspace_selector_get_screen_width (self))) * workspace_selector_get_screen_height (self)));
	/* resize */
	ratio = ((double) gdk_pixbuf_get_width (pb)) / ((double) gdk_pixbuf_get_width (ss));
	gdk_pixbuf_scale (ss, pb, 0, 0, gdk_pixbuf_get_width (pb), gdk_pixbuf_get_height (pb), (double) 0, (double) 0, ratio, ratio, GDK_INTERP_BILINEAR);
	_tmp3 = NULL;
	return (_tmp3 = pb, (ss == NULL) ? NULL : (ss = (g_object_unref (ss), NULL)), _tmp3);
}


static void workspace_selector_get_window_abs_geometry (WorkspaceSelector* self, WnckWindow* window, gint* x, gint* y, gint* w, gint* h) {
	WnckWorkspace* _tmp0;
	WnckWorkspace* workspace;
	g_return_if_fail (self != NULL);
	g_return_if_fail (window != NULL);
	wnck_window_get_geometry (window, &(*x), &(*y), &(*w), &(*h));
	_tmp0 = NULL;
	workspace = (_tmp0 = wnck_window_get_workspace (window), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (workspace != NULL) {
		(*x) = (*x) + wnck_workspace_get_viewport_x (workspace);
		(*y) = (*y) + wnck_workspace_get_viewport_y (workspace);
	}
	(workspace == NULL) ? NULL : (workspace = (g_object_unref (workspace), NULL));
}


static gboolean workspace_selector_window_visible_on_desktop (WorkspaceSelector* self, WnckWindow* window, gint col, gint row) {
	gint x;
	gint y;
	gint w;
	gint h;
	gboolean _tmp1;
	gboolean _tmp2;
	gboolean _tmp3;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (window != NULL, FALSE);
	if (wnck_window_is_minimized (window)) {
		return FALSE;
	}
	x = 0;
	y = 0;
	w = 0;
	h = 0;
	workspace_selector_get_window_abs_geometry (self, window, &x, &y, &w, &h);
	_tmp1 = FALSE;
	_tmp2 = FALSE;
	_tmp3 = FALSE;
	if (x >= (col * workspace_selector_get_screen_width (self))) {
		_tmp3 = x < ((col + 1) * workspace_selector_get_screen_width (self));
	} else {
		_tmp3 = FALSE;
	}
	if (_tmp3) {
		_tmp2 = y >= (row * workspace_selector_get_screen_height (self));
	} else {
		_tmp2 = FALSE;
	}
	if (_tmp2) {
		_tmp1 = y < ((row + 1) * workspace_selector_get_screen_height (self));
	} else {
		_tmp1 = FALSE;
	}
	return _tmp1;
}


gint workspace_selector_get_nrows (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, 0);
	return workspace_selector_get_workspace_height (self) / workspace_selector_get_screen_height (self);
}


gint workspace_selector_get_ncols (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, 0);
	return workspace_selector_get_workspace_width (self) / workspace_selector_get_screen_width (self);
}


gint workspace_selector_get_screen_width (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, 0);
	return wnck_screen_get_width (wnck_window_get_screen (self->priv->target));
}


gint workspace_selector_get_screen_height (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, 0);
	return wnck_screen_get_height (wnck_window_get_screen (self->priv->target));
}


gint workspace_selector_get_workspace_width (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, 0);
	return wnck_workspace_get_width (wnck_window_get_workspace (self->priv->target));
}


gint workspace_selector_get_workspace_height (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, 0);
	return wnck_workspace_get_height (wnck_window_get_workspace (self->priv->target));
}


gint workspace_selector_get_current_column (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, 0);
	return wnck_workspace_get_viewport_x (wnck_window_get_workspace (self->priv->target)) / workspace_selector_get_screen_width (self);
}


gint workspace_selector_get_current_row (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, 0);
	return wnck_workspace_get_viewport_y (wnck_window_get_workspace (self->priv->target)) / workspace_selector_get_screen_height (self);
}


WorkspaceItem* workspace_selector_get_selected_item (WorkspaceSelector* self) {
	g_return_val_if_fail (self != NULL, NULL);
	{
		GList* wi_collection;
		GList* wi_it;
		wi_collection = self->priv->workspaces;
		for (wi_it = wi_collection; wi_it != NULL; wi_it = wi_it->next) {
			WorkspaceItem* wi;
			wi = (WorkspaceItem*) wi_it->data;
			{
				if (workspace_item_get_selected (wi)) {
					return wi;
				}
			}
		}
	}
	return NULL;
}


static gboolean _workspace_selector_on_leave_notify_gtk_widget_leave_notify_event (WorkspaceSelector* _sender, const GdkEventCrossing* event, gpointer self) {
	return workspace_selector_on_leave_notify (self, _sender, event);
}


static gboolean _workspace_selector_on_button_press_gtk_widget_button_press_event (WorkspaceSelector* _sender, const GdkEventButton* event, gpointer self) {
	return workspace_selector_on_button_press (self, _sender, event);
}


static gboolean _workspace_selector_on_button_release_gtk_widget_button_release_event (WorkspaceSelector* _sender, const GdkEventButton* event, gpointer self) {
	return workspace_selector_on_button_release (self, _sender, event);
}


static GObject * workspace_selector_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	WorkspaceSelectorClass * klass;
	GObjectClass * parent_class;
	WorkspaceSelector * self;
	klass = WORKSPACE_SELECTOR_CLASS (g_type_class_peek (TYPE_WORKSPACE_SELECTOR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = WORKSPACE_SELECTOR (obj);
	{
		GtkMenu* _tmp0;
		gtk_widget_set_name ((GtkWidget*) self, "WorkspaceSelector");
		gtk_window_set_type_hint ((GtkWindow*) self, GDK_WINDOW_TYPE_HINT_MENU);
		g_object_set ((GtkWindow*) self, "window-position", GTK_WIN_POS_CENTER_ALWAYS, NULL);
		gtk_window_set_resizable ((GtkWindow*) self, FALSE);
		g_object_set ((GtkWindow*) self, "allow-grow", FALSE, NULL);
		gtk_window_set_decorated ((GtkWindow*) self, FALSE);
		gtk_window_set_skip_pager_hint ((GtkWindow*) self, TRUE);
		gtk_window_set_skip_taskbar_hint ((GtkWindow*) self, TRUE);
		if (((GtkBin*) self)->child != NULL) {
			gtk_widget_show_all (((GtkBin*) self)->child);
		}
		g_object_set ((GtkWindow*) self, "default-width", 100, NULL);
		g_object_set ((GtkWindow*) self, "default-height", 100, NULL);
		g_signal_connect_object ((GtkWidget*) self, "leave-notify-event", (GCallback) _workspace_selector_on_leave_notify_gtk_widget_leave_notify_event, self, 0);
		g_signal_connect_object ((GtkWidget*) self, "button-press-event", (GCallback) _workspace_selector_on_button_press_gtk_widget_button_press_event, self, 0);
		g_signal_connect_object ((GtkWidget*) self, "button-release-event", (GCallback) _workspace_selector_on_button_release_gtk_widget_button_release_event, self, 0);
		_tmp0 = NULL;
		gtk_widget_set_style ((GtkWidget*) self, gtk_rc_get_style ((GtkWidget*) (_tmp0 = g_object_ref_sink ((GtkMenu*) gtk_menu_new ()))));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	}
	return obj;
}


static void workspace_selector_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	WorkspaceSelector * self;
	gpointer boxed;
	self = WORKSPACE_SELECTOR (object);
	switch (property_id) {
		case WORKSPACE_SELECTOR_NROWS:
		g_value_set_int (value, workspace_selector_get_nrows (self));
		break;
		case WORKSPACE_SELECTOR_NCOLS:
		g_value_set_int (value, workspace_selector_get_ncols (self));
		break;
		case WORKSPACE_SELECTOR_SCREEN_WIDTH:
		g_value_set_int (value, workspace_selector_get_screen_width (self));
		break;
		case WORKSPACE_SELECTOR_SCREEN_HEIGHT:
		g_value_set_int (value, workspace_selector_get_screen_height (self));
		break;
		case WORKSPACE_SELECTOR_WORKSPACE_WIDTH:
		g_value_set_int (value, workspace_selector_get_workspace_width (self));
		break;
		case WORKSPACE_SELECTOR_WORKSPACE_HEIGHT:
		g_value_set_int (value, workspace_selector_get_workspace_height (self));
		break;
		case WORKSPACE_SELECTOR_CURRENT_COLUMN:
		g_value_set_int (value, workspace_selector_get_current_column (self));
		break;
		case WORKSPACE_SELECTOR_CURRENT_ROW:
		g_value_set_int (value, workspace_selector_get_current_row (self));
		break;
		case WORKSPACE_SELECTOR_SELECTED_ITEM:
		g_value_set_object (value, workspace_selector_get_selected_item (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void workspace_selector_class_init (WorkspaceSelectorClass * klass) {
	workspace_selector_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (WorkspaceSelectorPrivate));
	G_OBJECT_CLASS (klass)->get_property = workspace_selector_get_property;
	G_OBJECT_CLASS (klass)->constructor = workspace_selector_constructor;
	G_OBJECT_CLASS (klass)->finalize = workspace_selector_finalize;
	G_OBJECT_CLASS (klass)->dispose = workspace_selector_real_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_NROWS, g_param_spec_int ("nrows", "nrows", "nrows", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_NCOLS, g_param_spec_int ("ncols", "ncols", "ncols", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_SCREEN_WIDTH, g_param_spec_int ("screen-width", "screen-width", "screen-width", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_SCREEN_HEIGHT, g_param_spec_int ("screen-height", "screen-height", "screen-height", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_WORKSPACE_WIDTH, g_param_spec_int ("workspace-width", "workspace-width", "workspace-width", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_WORKSPACE_HEIGHT, g_param_spec_int ("workspace-height", "workspace-height", "workspace-height", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_CURRENT_COLUMN, g_param_spec_int ("current-column", "current-column", "current-column", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_CURRENT_ROW, g_param_spec_int ("current-row", "current-row", "current-row", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_SELECTOR_SELECTED_ITEM, g_param_spec_object ("selected-item", "selected-item", "selected-item", TYPE_WORKSPACE_ITEM, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void workspace_selector_instance_init (WorkspaceSelector * self) {
	self->priv = WORKSPACE_SELECTOR_GET_PRIVATE (self);
	self->priv->iid = (guint) 0;
}


static void workspace_selector_finalize (GObject* obj) {
	WorkspaceSelector * self;
	self = WORKSPACE_SELECTOR (obj);
	(self->priv->target == NULL) ? NULL : (self->priv->target = (g_object_unref (self->priv->target), NULL));
	(self->priv->workspaces == NULL) ? NULL : (self->priv->workspaces = (_g_list_free_g_object_unref (self->priv->workspaces), NULL));
	(self->priv->follow == NULL) ? NULL : (self->priv->follow = (g_object_unref (self->priv->follow), NULL));
	G_OBJECT_CLASS (workspace_selector_parent_class)->finalize (obj);
}


GType workspace_selector_get_type (void) {
	static GType workspace_selector_type_id = 0;
	if (workspace_selector_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (WorkspaceSelectorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) workspace_selector_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (WorkspaceSelector), 0, (GInstanceInitFunc) workspace_selector_instance_init, NULL };
		workspace_selector_type_id = g_type_register_static (GTK_TYPE_WINDOW, "WorkspaceSelector", &g_define_type_info, 0);
	}
	return workspace_selector_type_id;
}


WorkspaceItem* workspace_item_construct (GType object_type) {
	WorkspaceItem * self;
	GtkImage* _tmp0;
	self = g_object_newv (object_type, 0, NULL);
	_tmp0 = NULL;
	self->priv->_image = (_tmp0 = NULL, (self->priv->_image == NULL) ? NULL : (self->priv->_image = (g_object_unref (self->priv->_image), NULL)), _tmp0);
	gtk_widget_show ((GtkWidget*) self);
	return self;
}


WorkspaceItem* workspace_item_new (void) {
	return workspace_item_construct (TYPE_WORKSPACE_ITEM);
}


WorkspaceItem* workspace_item_construct_with_pixbuf (GType object_type, GdkPixbuf* pixbuf) {
	WorkspaceItem * self;
	GtkImage* _tmp0;
	g_return_val_if_fail (pixbuf != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp0 = NULL;
	workspace_item_set_image (self, _tmp0 = g_object_ref_sink ((GtkImage*) gtk_image_new_from_pixbuf (pixbuf)));
	(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	gtk_widget_show_all ((GtkWidget*) self);
	return self;
}


WorkspaceItem* workspace_item_new_with_pixbuf (GdkPixbuf* pixbuf) {
	return workspace_item_construct_with_pixbuf (TYPE_WORKSPACE_ITEM, pixbuf);
}


static void workspace_item_update (WorkspaceItem* self) {
	GdkPixbuf* _tmp1;
	GdkPixbuf* _tmp0;
	GdkPixbuf* _tmp3;
	GdkPixbuf* _tmp2;
	g_return_if_fail (self != NULL);
	if (workspace_item_get_image (self) == NULL) {
		return;
	}
	_tmp1 = NULL;
	_tmp0 = NULL;
	g_object_set ((GtkWidget*) self, "width-request", gdk_pixbuf_get_width (_tmp1 = (g_object_get (workspace_item_get_image (self), "pixbuf", &_tmp0, NULL), _tmp0)) + (workspace_item_get_margin (self) * 2), NULL);
	(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
	_tmp3 = NULL;
	_tmp2 = NULL;
	g_object_set ((GtkWidget*) self, "height-request", gdk_pixbuf_get_height (_tmp3 = (g_object_get (workspace_item_get_image (self), "pixbuf", &_tmp2, NULL), _tmp2)) + (workspace_item_get_margin (self) * 2), NULL);
	(_tmp3 == NULL) ? NULL : (_tmp3 = (g_object_unref (_tmp3), NULL));
}


static gboolean workspace_item_on_leave_notify (WorkspaceItem* self, WorkspaceItem* wi, const GdkEventCrossing* event) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (wi != NULL, FALSE);
	if ((*event).state == 0) {
		workspace_item_set_selected (self, FALSE);
	}
	return FALSE;
}


static gboolean workspace_item_on_enter_notify (WorkspaceItem* self, WorkspaceItem* wi, const GdkEventCrossing* event) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (wi != NULL, FALSE);
	workspace_item_set_selected (self, TRUE);
	return FALSE;
}


GtkImage* workspace_item_get_image (WorkspaceItem* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_image;
}


void workspace_item_set_image (WorkspaceItem* self, GtkImage* value) {
	GtkImage* _tmp2;
	GtkImage* _tmp1;
	g_return_if_fail (self != NULL);
	if (self->priv->_image != NULL) {
		gtk_container_remove ((GtkContainer*) self, (GtkWidget*) self->priv->_image);
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_image = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_image == NULL) ? NULL : (self->priv->_image = (g_object_unref (self->priv->_image), NULL)), _tmp2);
	if (self->priv->_image != NULL) {
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->_image);
	}
	workspace_item_update (self);
	g_object_notify ((GObject *) self, "image");
}


gint workspace_item_get_margin (WorkspaceItem* self) {
	g_return_val_if_fail (self != NULL, 0);
	return self->priv->_margin;
}


void workspace_item_set_margin (WorkspaceItem* self, gint value) {
	g_return_if_fail (self != NULL);
	self->priv->_margin = value;
	workspace_item_update (self);
	g_object_notify ((GObject *) self, "margin");
}


gboolean workspace_item_get_selected (WorkspaceItem* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_selected;
}


void workspace_item_set_selected (WorkspaceItem* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_selected = value;
	if (self->priv->_selected) {
		gtk_widget_set_state ((GtkWidget*) self, GTK_STATE_SELECTED);
	} else {
		if (!workspace_item_get_current (self)) {
			gtk_widget_set_state ((GtkWidget*) self, GTK_STATE_NORMAL);
		}
	}
	g_object_notify ((GObject *) self, "selected");
}


gboolean workspace_item_get_current (WorkspaceItem* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_current;
}


void workspace_item_set_current (WorkspaceItem* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_current = value;
	if (self->priv->_current) {
		gtk_widget_set_state ((GtkWidget*) self, GTK_STATE_SELECTED);
	}
	g_object_notify ((GObject *) self, "current");
}


static gboolean _workspace_item_on_leave_notify_gtk_widget_leave_notify_event (WorkspaceItem* _sender, const GdkEventCrossing* event, gpointer self) {
	return workspace_item_on_leave_notify (self, _sender, event);
}


static gboolean _workspace_item_on_enter_notify_gtk_widget_enter_notify_event (WorkspaceItem* _sender, const GdkEventCrossing* event, gpointer self) {
	return workspace_item_on_enter_notify (self, _sender, event);
}


static GObject * workspace_item_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	WorkspaceItemClass * klass;
	GObjectClass * parent_class;
	WorkspaceItem * self;
	klass = WORKSPACE_ITEM_CLASS (g_type_class_peek (TYPE_WORKSPACE_ITEM));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = WORKSPACE_ITEM (obj);
	{
		GtkMenuItem* _tmp1;
		_tmp1 = NULL;
		gtk_widget_set_style ((GtkWidget*) self, gtk_rc_get_style ((GtkWidget*) (_tmp1 = g_object_ref_sink ((GtkMenuItem*) gtk_menu_item_new ()))));
		(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		g_signal_connect_object ((GtkWidget*) self, "leave-notify-event", (GCallback) _workspace_item_on_leave_notify_gtk_widget_leave_notify_event, self, 0);
		g_signal_connect_object ((GtkWidget*) self, "enter-notify-event", (GCallback) _workspace_item_on_enter_notify_gtk_widget_enter_notify_event, self, 0);
	}
	return obj;
}


static void workspace_item_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	WorkspaceItem * self;
	gpointer boxed;
	self = WORKSPACE_ITEM (object);
	switch (property_id) {
		case WORKSPACE_ITEM_IMAGE:
		g_value_set_object (value, workspace_item_get_image (self));
		break;
		case WORKSPACE_ITEM_MARGIN:
		g_value_set_int (value, workspace_item_get_margin (self));
		break;
		case WORKSPACE_ITEM_SELECTED:
		g_value_set_boolean (value, workspace_item_get_selected (self));
		break;
		case WORKSPACE_ITEM_CURRENT:
		g_value_set_boolean (value, workspace_item_get_current (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void workspace_item_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	WorkspaceItem * self;
	self = WORKSPACE_ITEM (object);
	switch (property_id) {
		case WORKSPACE_ITEM_IMAGE:
		workspace_item_set_image (self, g_value_get_object (value));
		break;
		case WORKSPACE_ITEM_MARGIN:
		workspace_item_set_margin (self, g_value_get_int (value));
		break;
		case WORKSPACE_ITEM_SELECTED:
		workspace_item_set_selected (self, g_value_get_boolean (value));
		break;
		case WORKSPACE_ITEM_CURRENT:
		workspace_item_set_current (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void workspace_item_class_init (WorkspaceItemClass * klass) {
	workspace_item_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (WorkspaceItemPrivate));
	G_OBJECT_CLASS (klass)->get_property = workspace_item_get_property;
	G_OBJECT_CLASS (klass)->set_property = workspace_item_set_property;
	G_OBJECT_CLASS (klass)->constructor = workspace_item_constructor;
	G_OBJECT_CLASS (klass)->finalize = workspace_item_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_ITEM_IMAGE, g_param_spec_object ("image", "image", "image", GTK_TYPE_IMAGE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_ITEM_MARGIN, g_param_spec_int ("margin", "margin", "margin", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_ITEM_SELECTED, g_param_spec_boolean ("selected", "selected", "selected", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), WORKSPACE_ITEM_CURRENT, g_param_spec_boolean ("current", "current", "current", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void workspace_item_instance_init (WorkspaceItem * self) {
	self->priv = WORKSPACE_ITEM_GET_PRIVATE (self);
	self->priv->_margin = 6;
	self->priv->_selected = FALSE;
	self->priv->_current = FALSE;
}


static void workspace_item_finalize (GObject* obj) {
	WorkspaceItem * self;
	self = WORKSPACE_ITEM (obj);
	(self->priv->_image == NULL) ? NULL : (self->priv->_image = (g_object_unref (self->priv->_image), NULL));
	G_OBJECT_CLASS (workspace_item_parent_class)->finalize (obj);
}


GType workspace_item_get_type (void) {
	static GType workspace_item_type_id = 0;
	if (workspace_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (WorkspaceItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) workspace_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (WorkspaceItem), 0, (GInstanceInitFunc) workspace_item_instance_init, NULL };
		workspace_item_type_id = g_type_register_static (GTK_TYPE_EVENT_BOX, "WorkspaceItem", &g_define_type_info, 0);
	}
	return workspace_item_type_id;
}




