
#include <gtk-modules/globalmenu/globalmenu-serializer.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gdk/gdkx.h>
#include <gdk/gdk.h>
#include <gtk-modules/globalmenu/globalmenu.h>




static glong string_get_length (const char* self);
static GlobalMenuGTKSerializer* global_menu_gtk_serializer_new (void);
static void global_menu_gtk_serializer_visit (GlobalMenuGTKSerializer* self, GtkWidget* widget);
static void global_menu_gtk_serializer_visit_container (GlobalMenuGTKSerializer* self, GtkContainer* container);
static void global_menu_gtk_serializer_visit_menubar (GlobalMenuGTKSerializer* self, GtkMenuBar* menubar);
static void global_menu_gtk_serializer_visit_menu (GlobalMenuGTKSerializer* self, GtkMenu* menu);
static void global_menu_gtk_serializer_visit_menuitem (GlobalMenuGTKSerializer* self, GtkMenuItem* menuitem);
static void global_menu_gtk_serializer_visit_label (GlobalMenuGTKSerializer* self, GtkLabel* label);
static void global_menu_gtk_serializer_visit_image (GlobalMenuGTKSerializer* self, GtkImage* image);
static char* global_menu_gtk_serializer_pixbuf_encode_b64 (GlobalMenuGTKSerializer* self, GdkPixbuf* pixbuf);
static void global_menu_gtk_serializer_append_icon_attribute (GlobalMenuGTKSerializer* self, GtkImage* image);
static void global_menu_gtk_serializer_indent (GlobalMenuGTKSerializer* self);
static void global_menu_gtk_serializer_linebreak (GlobalMenuGTKSerializer* self);
static void global_menu_gtk_serializer_instance_init (GlobalMenuGTKSerializer * self);
static int _vala_strcmp0 (const char * str1, const char * str2);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static GlobalMenuGTKSerializer* global_menu_gtk_serializer_new (void) {
	GlobalMenuGTKSerializer* self;
	self = g_slice_new0 (GlobalMenuGTKSerializer);
	global_menu_gtk_serializer_instance_init (self);
	return self;
}


char* global_menu_gtk_serializer_to_string (GtkMenuBar* menubar, gboolean pretty_print) {
	GlobalMenuGTKSerializer* s;
	GTimer* timer;
	GtkMenuBar* _tmp1;
	GtkMenuBar* _tmp0;
	GString* _tmp2;
	GString* _tmp3;
	const char* _tmp4;
	char* _tmp5;
	g_return_val_if_fail (menubar != NULL, NULL);
	s = global_menu_gtk_serializer_new ();
	timer = g_timer_new ();
	_tmp1 = NULL;
	_tmp0 = NULL;
	s->menubar = (_tmp1 = (_tmp0 = menubar, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (s->menubar == NULL) ? NULL : (s->menubar = (g_object_unref (s->menubar), NULL)), _tmp1);
	s->pretty_print = pretty_print;
	_tmp2 = NULL;
	s->sb = (_tmp2 = g_string_new (""), (s->sb == NULL) ? NULL : (s->sb = (g_string_free (s->sb, TRUE), NULL)), _tmp2);
	_tmp3 = NULL;
	s->label_sb = (_tmp3 = g_string_new (""), (s->label_sb == NULL) ? NULL : (s->label_sb = (g_string_free (s->label_sb, TRUE), NULL)), _tmp3);
	global_menu_gtk_serializer_visit (s, (GtkWidget*) menubar);
	g_debug ("globalmenu-serializer.vala:16: Serializer consumption = %lf", g_timer_elapsed (timer, NULL));
	_tmp4 = NULL;
	_tmp5 = NULL;
	return (_tmp5 = (_tmp4 = s->sb->str, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), (s == NULL) ? NULL : (s = (global_menu_gtk_serializer_free (s), NULL)), (timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL)), _tmp5);
}


static void global_menu_gtk_serializer_visit (GlobalMenuGTKSerializer* self, GtkWidget* widget) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	if (GTK_IS_MENU_BAR (widget)) {
		GtkWidget* _tmp0;
		_tmp0 = NULL;
		global_menu_gtk_serializer_visit_menubar (self, (_tmp0 = widget, GTK_IS_MENU_BAR (_tmp0) ? ((GtkMenuBar*) _tmp0) : NULL));
	} else {
		if (GTK_IS_MENU (widget)) {
			GtkWidget* _tmp1;
			_tmp1 = NULL;
			global_menu_gtk_serializer_visit_menu (self, (_tmp1 = widget, GTK_IS_MENU (_tmp1) ? ((GtkMenu*) _tmp1) : NULL));
		} else {
			if (GTK_IS_MENU_ITEM (widget)) {
				GtkWidget* _tmp2;
				_tmp2 = NULL;
				global_menu_gtk_serializer_visit_menuitem (self, (_tmp2 = widget, GTK_IS_MENU_ITEM (_tmp2) ? ((GtkMenuItem*) _tmp2) : NULL));
			} else {
				if (GTK_IS_LABEL (widget)) {
					GtkWidget* _tmp3;
					_tmp3 = NULL;
					global_menu_gtk_serializer_visit_label (self, (_tmp3 = widget, GTK_IS_LABEL (_tmp3) ? ((GtkLabel*) _tmp3) : NULL));
				} else {
					if (GTK_IS_IMAGE (widget)) {
						GtkWidget* _tmp4;
						_tmp4 = NULL;
						global_menu_gtk_serializer_visit_image (self, (_tmp4 = widget, GTK_IS_IMAGE (_tmp4) ? ((GtkImage*) _tmp4) : NULL));
					} else {
						if (GTK_IS_CONTAINER (widget)) {
							GtkWidget* _tmp5;
							_tmp5 = NULL;
							global_menu_gtk_serializer_visit_container (self, (_tmp5 = widget, GTK_IS_CONTAINER (_tmp5) ? ((GtkContainer*) _tmp5) : NULL));
						}
					}
				}
			}
		}
	}
}


static void global_menu_gtk_serializer_visit_container (GlobalMenuGTKSerializer* self, GtkContainer* container) {
	GList* children;
	g_return_if_fail (self != NULL);
	g_return_if_fail (container != NULL);
	children = gtk_container_get_children (container);
	g_debug ("globalmenu-serializer.vala:34: %p has %u children", container, g_list_length (children));
	{
		GList* child_collection;
		GList* child_it;
		child_collection = children;
		for (child_it = child_collection; child_it != NULL; child_it = child_it->next) {
			GtkWidget* child;
			child = (GtkWidget*) child_it->data;
			{
				global_menu_gtk_serializer_visit (self, child);
			}
		}
	}
	(children == NULL) ? NULL : (children = (g_list_free (children), NULL));
}


static void global_menu_gtk_serializer_visit_menubar (GlobalMenuGTKSerializer* self, GtkMenuBar* menubar) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (menubar != NULL);
	global_menu_gtk_serializer_indent (self);
	g_string_append (self->sb, "<menu>");
	global_menu_gtk_serializer_linebreak (self);
	self->level++;
	global_menu_gtk_serializer_visit_container (self, (GtkContainer*) menubar);
	self->level--;
	global_menu_gtk_serializer_indent (self);
	g_string_append (self->sb, "</menu>");
	global_menu_gtk_serializer_linebreak (self);
}


static void global_menu_gtk_serializer_visit_menu (GlobalMenuGTKSerializer* self, GtkMenu* menu) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (menu != NULL);
	global_menu_gtk_serializer_indent (self);
	g_string_append (self->sb, "<menu>");
	global_menu_gtk_serializer_linebreak (self);
	global_menu_gtk_serializer_visit_container (self, (GtkContainer*) menu);
	global_menu_gtk_serializer_indent (self);
	g_string_append (self->sb, "</menu>");
	global_menu_gtk_serializer_linebreak (self);
}


static void global_menu_gtk_serializer_visit_menuitem (GlobalMenuGTKSerializer* self, GtkMenuItem* menuitem) {
	char* _tmp0;
	gboolean _tmp2;
	gboolean _tmp8;
	gboolean _tmp9;
	g_return_if_fail (self != NULL);
	g_return_if_fail (menuitem != NULL);
	if (GTK_IS_TEAROFF_MENU_ITEM (menuitem)) {
		return;
	}
	global_menu_gtk_serializer_indent (self);
	g_string_append (self->sb, "<item");
	_tmp0 = NULL;
	g_string_append (self->sb, _tmp0 = g_markup_printf_escaped (" id=\"W%lu\"", (gulong) menuitem));
	_tmp0 = (g_free (_tmp0), NULL);
	if (global_menu_gtk_hybrid) {
		g_string_append (self->sb, " client-side=\"1\"");
	}
	g_string_erase (self->label_sb, (glong) 0, (glong) (-1));
	self->last_item_empty = TRUE;
	self->guessed_type = NULL;
	global_menu_gtk_serializer_visit_container (self, (GtkContainer*) menuitem);
	if (self->label_sb->len > 0) {
		char* _tmp1;
		_tmp1 = NULL;
		g_string_append (self->sb, _tmp1 = g_markup_printf_escaped (" label=\"%s\"", self->label_sb->str));
		_tmp1 = (g_free (_tmp1), NULL);
		self->last_item_empty = FALSE;
	}
	_tmp2 = FALSE;
	if (GTK_IS_SEPARATOR_MENU_ITEM (menuitem)) {
		_tmp2 = TRUE;
	} else {
		_tmp2 = gtk_bin_get_child ((GtkBin*) menuitem) == NULL;
	}
	if (_tmp2) {
		self->guessed_type = "s";
		self->last_item_empty = FALSE;
	}
	if (GTK_IS_IMAGE_MENU_ITEM (menuitem)) {
		GtkImage* _tmp5;
		GtkMenuItem* _tmp3;
		GtkWidget* _tmp4;
		GtkImage* image;
		_tmp5 = NULL;
		_tmp3 = NULL;
		_tmp4 = NULL;
		image = (_tmp5 = (_tmp4 = gtk_image_menu_item_get_image ((_tmp3 = menuitem, GTK_IS_IMAGE_MENU_ITEM (_tmp3) ? ((GtkImageMenuItem*) _tmp3) : NULL)), GTK_IS_IMAGE (_tmp4) ? ((GtkImage*) _tmp4) : NULL), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5));
		if (image != NULL) {
			self->guessed_type = "i";
			global_menu_gtk_serializer_append_icon_attribute (self, image);
			self->last_item_empty = FALSE;
		}
		(image == NULL) ? NULL : (image = (g_object_unref (image), NULL));
	}
	if (GTK_IS_CHECK_MENU_ITEM (menuitem)) {
		GtkCheckMenuItem* _tmp7;
		GtkMenuItem* _tmp6;
		GtkCheckMenuItem* checkmenuitem;
		_tmp7 = NULL;
		_tmp6 = NULL;
		checkmenuitem = (_tmp7 = (_tmp6 = menuitem, GTK_IS_CHECK_MENU_ITEM (_tmp6) ? ((GtkCheckMenuItem*) _tmp6) : NULL), (_tmp7 == NULL) ? NULL : g_object_ref (_tmp7));
		if (gtk_check_menu_item_get_draw_as_radio (checkmenuitem)) {
			self->guessed_type = "r";
		} else {
			self->guessed_type = "c";
		}
		if (!gtk_check_menu_item_get_inconsistent (checkmenuitem)) {
			if (gtk_check_menu_item_get_active (checkmenuitem)) {
				g_string_append (self->sb, " state=\"1\"");
			} else {
				g_string_append (self->sb, " state=\"0\"");
			}
		}
		self->last_item_empty = FALSE;
		(checkmenuitem == NULL) ? NULL : (checkmenuitem = (g_object_unref (checkmenuitem), NULL));
	}
	if ((g_object_get ((GtkWidget*) menuitem, "visible", &_tmp8, NULL), _tmp8) == FALSE) {
		g_string_append (self->sb, " visible=\"0\"");
		self->last_item_empty = FALSE;
	}
	if ((g_object_get ((GtkWidget*) menuitem, "sensitive", &_tmp9, NULL), _tmp9) == FALSE) {
		g_string_append (self->sb, " sensitive=\"0\"");
	}
	if (self->last_item_empty) {
		/* hide the empty items with
		 * no label, no image, or
		 * any other visible elements.
		 *
		 * eg, PidginMenuTray
		 * */
		g_string_append (self->sb, " visible=\"0\"");
	}
	if (self->guessed_type != NULL) {
		g_string_append_printf (self->sb, " type=\"%s\"", self->guessed_type);
	}
	if (gtk_menu_item_get_submenu (menuitem) == NULL) {
		g_string_append (self->sb, "/>");
		global_menu_gtk_serializer_linebreak (self);
	} else {
		g_string_append_c (self->sb, '>');
		global_menu_gtk_serializer_linebreak (self);
		self->level++;
		if (gtk_menu_item_get_submenu (menuitem) != NULL) {
			global_menu_gtk_serializer_visit_menu (self, gtk_menu_item_get_submenu (menuitem));
		}
		self->level--;
		global_menu_gtk_serializer_indent (self);
		g_string_append (self->sb, "</item>");
		global_menu_gtk_serializer_linebreak (self);
	}
}


static void global_menu_gtk_serializer_visit_label (GlobalMenuGTKSerializer* self, GtkLabel* label) {
	const char* _tmp0;
	char* label_text;
	g_return_if_fail (self != NULL);
	g_return_if_fail (label != NULL);
	_tmp0 = NULL;
	label_text = (_tmp0 = gtk_label_get_label (label), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	g_string_append (self->label_sb, label_text);
	g_debug ("globalmenu-serializer.vala:146: append text = %s", label_text);
	if (GTK_IS_ACCEL_LABEL (label)) {
		GtkLabel* _tmp1;
		const char* _tmp3;
		GtkLabel* _tmp2;
		char* accel_string;
		gboolean _tmp4;
		_tmp1 = NULL;
		gtk_accel_label_refetch ((_tmp1 = label, GTK_IS_ACCEL_LABEL (_tmp1) ? ((GtkAccelLabel*) _tmp1) : NULL));
		_tmp3 = NULL;
		_tmp2 = NULL;
		accel_string = (_tmp3 = (_tmp2 = label, GTK_IS_ACCEL_LABEL (_tmp2) ? ((GtkAccelLabel*) _tmp2) : NULL)->accel_string, (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
		g_strstrip (accel_string);
		_tmp4 = FALSE;
		if (string_get_length (accel_string) > 0) {
			_tmp4 = _vala_strcmp0 (accel_string, "-/-") != 0;
		} else {
			_tmp4 = FALSE;
		}
		if (_tmp4) {
			char* _tmp5;
			/*refer to gtkaccellabel.c:802*/
			_tmp5 = NULL;
			g_string_append (self->sb, _tmp5 = g_markup_printf_escaped (" accel=\"%s\"", accel_string));
			_tmp5 = (g_free (_tmp5), NULL);
		}
		accel_string = (g_free (accel_string), NULL);
	}
	label_text = (g_free (label_text), NULL);
}


static void global_menu_gtk_serializer_visit_image (GlobalMenuGTKSerializer* self, GtkImage* image) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (image != NULL);
	/*workaround a bug before Gtk 2.14*/
	if (GTK_IS_IMAGE_MENU_ITEM (gtk_widget_get_parent ((GtkWidget*) image))) {
		return;
	}
	/*Disable pure image menu item since
	  the purpose of introducing it was
	  to get Pidgin's menu icon tray working
	  but it turns out not working;
	  and causing issue 243
	sb.append(" type=\"icon\"");
	
	*
	 * adobe reader sucks. It doesn't use
	 * image menu item but use a hbox
	 * to contain the image
	 */
	self->guessed_type = "i";
	global_menu_gtk_serializer_append_icon_attribute (self, image);
}


static char* global_menu_gtk_serializer_pixbuf_encode_b64 (GlobalMenuGTKSerializer* self, GdkPixbuf* pixbuf) {
	GdkPixdata pixdata = {(guint32) 0};
	guchar* _tmp1;
	gint _tmp0;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (pixbuf != NULL, NULL);
	gdk_pixdata_from_pixbuf (&pixdata, pixbuf, TRUE);
	_tmp1 = NULL;
	return (_tmp1 = gdk_pixdata_serialize (&pixdata, &_tmp0), g_base64_encode (_tmp1, _tmp0));
}


static void global_menu_gtk_serializer_append_icon_attribute (GlobalMenuGTKSerializer* self, GtkImage* image) {
	char* _tmp1;
	char* _tmp0;
	gboolean _tmp2;
	g_return_if_fail (self != NULL);
	g_return_if_fail (image != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	if ((_tmp2 = (_tmp1 = (g_object_get (image, "file", &_tmp0, NULL), _tmp0)) != NULL, _tmp1 = (g_free (_tmp1), NULL), _tmp2)) {
		char* _tmp5;
		char* _tmp4;
		char* _tmp3;
		_tmp5 = NULL;
		_tmp4 = NULL;
		_tmp3 = NULL;
		g_string_append (self->sb, _tmp5 = g_markup_printf_escaped (" icon=\"file:%s\"", _tmp4 = (g_object_get (image, "file", &_tmp3, NULL), _tmp3)));
		_tmp5 = (g_free (_tmp5), NULL);
		_tmp4 = (g_free (_tmp4), NULL);
	} else {
		if (gtk_image_get_storage_type (image) == GTK_IMAGE_STOCK) {
			char* _tmp6;
			char* stock;
			/* STOCK is not shared between app and the applet */
			_tmp6 = NULL;
			stock = (g_object_get (image, "stock", &_tmp6, NULL), _tmp6);
			if (g_str_has_prefix (stock, "gtk")) {
				char* _tmp7;
				/*a GTK stock, shared between app and applet*/
				_tmp7 = NULL;
				g_string_append (self->sb, _tmp7 = g_markup_printf_escaped (" icon=\"%s\"", stock));
				_tmp7 = (g_free (_tmp7), NULL);
			} else {
				if (!global_menu_gtk_disable_pixbuf) {
					char* _tmp9;
					char* _tmp8;
					GdkPixbuf* _tmp10;
					GdkPixbuf* pixbuf;
					_tmp9 = NULL;
					_tmp8 = NULL;
					_tmp10 = NULL;
					pixbuf = (_tmp10 = gtk_widget_render_icon ((GtkWidget*) image, _tmp9 = (g_object_get (image, "stock", &_tmp8, NULL), _tmp8), GTK_ICON_SIZE_MENU, NULL), _tmp9 = (g_free (_tmp9), NULL), _tmp10);
					if (pixbuf != NULL) {
						char* _tmp12;
						char* _tmp11;
						_tmp12 = NULL;
						_tmp11 = NULL;
						g_string_append (self->sb, _tmp12 = g_markup_printf_escaped (" icon=\"pixbuf:%s\"", _tmp11 = global_menu_gtk_serializer_pixbuf_encode_b64 (self, pixbuf)));
						_tmp12 = (g_free (_tmp12), NULL);
						_tmp11 = (g_free (_tmp11), NULL);
					}
					(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
				}
			}
			stock = (g_free (stock), NULL);
		}
		if (gtk_image_get_storage_type (image) == GTK_IMAGE_ICON_NAME) {
			char* _tmp15;
			char* _tmp14;
			char* _tmp13;
			/* THEME is shared between applet and the application*/
			_tmp15 = NULL;
			_tmp14 = NULL;
			_tmp13 = NULL;
			g_string_append (self->sb, _tmp15 = g_markup_printf_escaped (" icon=\"theme:%s\"", _tmp14 = (g_object_get (image, "icon-name", &_tmp13, NULL), _tmp13)));
			_tmp15 = (g_free (_tmp15), NULL);
			_tmp14 = (g_free (_tmp14), NULL);
		}
		if (gtk_image_get_storage_type (image) == GTK_IMAGE_PIXBUF) {
			if (!global_menu_gtk_disable_pixbuf) {
				GdkPixbuf* _tmp17;
				GdkPixbuf* _tmp16;
				gboolean _tmp18;
				_tmp17 = NULL;
				_tmp16 = NULL;
				if ((_tmp18 = (_tmp17 = (g_object_get (image, "pixbuf", &_tmp16, NULL), _tmp16)) != NULL, (_tmp17 == NULL) ? NULL : (_tmp17 = (g_object_unref (_tmp17), NULL)), _tmp18)) {
					char* _tmp22;
					char* _tmp21;
					GdkPixbuf* _tmp20;
					GdkPixbuf* _tmp19;
					_tmp22 = NULL;
					_tmp21 = NULL;
					_tmp20 = NULL;
					_tmp19 = NULL;
					g_string_append (self->sb, _tmp22 = g_markup_printf_escaped (" icon=\"pixbuf:%s\"", _tmp21 = global_menu_gtk_serializer_pixbuf_encode_b64 (self, _tmp20 = (g_object_get (image, "pixbuf", &_tmp19, NULL), _tmp19))));
					_tmp22 = (g_free (_tmp22), NULL);
					_tmp21 = (g_free (_tmp21), NULL);
					(_tmp20 == NULL) ? NULL : (_tmp20 = (g_object_unref (_tmp20), NULL));
				}
			}
		}
		if (gtk_image_get_storage_type (image) == GTK_IMAGE_PIXMAP) {
			gulong pixmap_xid;
			gulong mask_xid;
			GdkPixmap* _tmp24;
			GdkPixmap* _tmp23;
			gboolean _tmp25;
			GdkPixmap* _tmp29;
			GdkPixmap* _tmp28;
			gboolean _tmp30;
			char* _tmp33;
			pixmap_xid = (gulong) 0;
			mask_xid = (gulong) 0;
			_tmp24 = NULL;
			_tmp23 = NULL;
			if ((_tmp25 = (_tmp24 = (g_object_get (image, "pixmap", &_tmp23, NULL), _tmp23)) != NULL, (_tmp24 == NULL) ? NULL : (_tmp24 = (g_object_unref (_tmp24), NULL)), _tmp25)) {
				GdkPixmap* _tmp27;
				GdkPixmap* _tmp26;
				_tmp27 = NULL;
				_tmp26 = NULL;
				pixmap_xid = GDK_DRAWABLE_XID ((GdkDrawable*) (_tmp27 = (g_object_get (image, "pixmap", &_tmp26, NULL), _tmp26)));
				(_tmp27 == NULL) ? NULL : (_tmp27 = (g_object_unref (_tmp27), NULL));
			}
			_tmp29 = NULL;
			_tmp28 = NULL;
			if ((_tmp30 = (_tmp29 = (g_object_get (image, "mask", &_tmp28, NULL), _tmp28)) != NULL, (_tmp29 == NULL) ? NULL : (_tmp29 = (g_object_unref (_tmp29), NULL)), _tmp30)) {
				GdkPixmap* _tmp32;
				GdkPixmap* _tmp31;
				_tmp32 = NULL;
				_tmp31 = NULL;
				mask_xid = GDK_DRAWABLE_XID ((GdkDrawable*) (_tmp32 = (g_object_get (image, "mask", &_tmp31, NULL), _tmp31)));
				(_tmp32 == NULL) ? NULL : (_tmp32 = (g_object_unref (_tmp32), NULL));
			}
			_tmp33 = NULL;
			g_string_append (self->sb, _tmp33 = g_markup_printf_escaped (" icon=\"pixmap:%lu,%lu\"", pixmap_xid, mask_xid));
			_tmp33 = (g_free (_tmp33), NULL);
		}
	}
}


static void global_menu_gtk_serializer_indent (GlobalMenuGTKSerializer* self) {
	g_return_if_fail (self != NULL);
	if (!self->pretty_print) {
		return;
	}
	if (!self->newline) {
		return;
	}
	{
		gint i;
		i = 0;
		for (; i < self->level; i++) {
			g_string_append_c (self->sb, ' ');
		}
	}
	self->newline = FALSE;
}


static void global_menu_gtk_serializer_linebreak (GlobalMenuGTKSerializer* self) {
	g_return_if_fail (self != NULL);
	if (!self->pretty_print) {
		return;
	}
	g_string_append_c (self->sb, '\n');
	self->newline = TRUE;
}


static void global_menu_gtk_serializer_instance_init (GlobalMenuGTKSerializer * self) {
}


void global_menu_gtk_serializer_free (GlobalMenuGTKSerializer* self) {
	(self->menubar == NULL) ? NULL : (self->menubar = (g_object_unref (self->menubar), NULL));
	(self->sb == NULL) ? NULL : (self->sb = (g_string_free (self->sb, TRUE), NULL));
	(self->label_sb == NULL) ? NULL : (self->label_sb = (g_string_free (self->label_sb, TRUE), NULL));
	g_slice_free (GlobalMenuGTKSerializer, self);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




