
#include <gtk-modules/globalmenu/module-main.h>
#include <glib/gi18n-lib.h>
#include <dyn-patch.h>
#include <configmake.h>
#include <gtk-modules/globalmenu/globalmenu.h>




static gboolean global_menu_gnome_verbose = FALSE;
static gboolean global_menu_gnome_disabled = FALSE;
static gboolean global_menu_gnome_initialized = FALSE;
static gboolean global_menu_gnome_disable_pixbuf = FALSE;
static gboolean global_menu_gnome_hybrid = FALSE;
static char* global_menu_gnome_log_file_name = NULL;
static FILE* global_menu_gnome_log_stream = NULL;
static GQuark global_menu_gnome_domain = 0U;
static guint global_menu_gnome_deferred_init_id = 0U;
static gboolean _global_menu_gnome_deferred_init_gsource_func (gpointer self);
static gboolean global_menu_gnome_deferred_init (void);
static void _global_menu_gnome_default_log_handler_glog_func (const char* log_domain, GLogLevelFlags log_levels, const char* message, gpointer self);
static gboolean global_menu_gnome_parse_args (void);
static void global_menu_gnome_prepare_log_file (void);
static void global_menu_gnome_default_log_handler (const char* domain, GLogLevelFlags level, const char* message);
static gboolean global_menu_gnome_is_quirky_app (void);
static void global_menu_gnome_instance_init (GlobalMenuGNOME * self);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const GOptionEntry GLOBAL_MENU_GNOME_options[] = {{"verbose", 'v', 0, G_OPTION_ARG_NONE, &global_menu_gnome_verbose, N_ ("Be verbose"), NULL}, {"disable-pixbuf", 'P', 0, G_OPTION_ARG_NONE, &global_menu_gnome_disable_pixbuf, N_ ("disable serializing pixbuf"), NULL}, {"disable", 'd', 0, G_OPTION_ARG_NONE, &global_menu_gnome_disabled, N_ ("Disable the Plugin"), NULL}, {"log-file", 'l', 0, G_OPTION_ARG_FILENAME, &global_menu_gnome_log_file_name, N_ ("File to save the log, default to ~/.gnomenu.log"), NULL}, {"hybrid", 'h', 0, G_OPTION_ARG_NONE, &global_menu_gnome_hybrid, N_ ("Enable hybrid mode"), NULL}, {NULL}};


static gboolean _global_menu_gnome_deferred_init_gsource_func (gpointer self) {
	return global_menu_gnome_deferred_init ();
}


void gtk_module_init (int* args_length1, char*** args) {
	if (!global_menu_gnome_disabled) {
		global_menu_gnome_deferred_init_id = g_idle_add_full (G_PRIORITY_HIGH, _global_menu_gnome_deferred_init_gsource_func, NULL, NULL);
	}
}


static gboolean global_menu_gnome_deferred_init (void) {
	if (!global_menu_gnome_initialized) {
		GlobalMenuGTKFlags flags;
		global_menu_gnome_initialized = TRUE;
		dyn_patch_init ();
		flags = 0;
		if (global_menu_gnome_disable_pixbuf) {
			flags = flags | GLOBAL_MENU_GTK_FLAGS_DISABLE_PIXBUF;
		}
		if (global_menu_gnome_hybrid) {
			flags = flags | GLOBAL_MENU_GTK_FLAGS_HYBRID;
		}
		global_menu_gtk_init (flags);
	}
	global_menu_gnome_deferred_init_id = (guint) 0;
	return FALSE;
}


static void _global_menu_gnome_default_log_handler_glog_func (const char* log_domain, GLogLevelFlags log_levels, const char* message, gpointer self) {
	global_menu_gnome_default_log_handler (log_domain, log_levels, message);
}


char* g_module_check_init (GModule* module) {
	g_return_val_if_fail (module != NULL, NULL);
	global_menu_gnome_domain = g_quark_from_string ("GlobalMenu");
	if (global_menu_gnome_is_quirky_app ()) {
		global_menu_gnome_disabled = TRUE;
	}
	global_menu_gnome_parse_args ();
	/* Do not write any log messages before we prepare the log file.*/
	global_menu_gnome_prepare_log_file ();
	g_log_set_handler (g_quark_to_string (global_menu_gnome_domain), G_LOG_LEVEL_DEBUG, _global_menu_gnome_default_log_handler_glog_func, NULL);
	g_debug ("module-main.vala:51: Global Menu Version: %s:%s", VERSION, SVNVERSION);
	if (!global_menu_gnome_disabled) {
		g_debug ("module-main.vala:53: Global Menu is enabled");
	} else {
		return g_strdup ("Global Menu is disabled");
	}
	return NULL;
}


void g_module_unload (GModule* module) {
	g_return_if_fail (module != NULL);
	if (!global_menu_gnome_disabled) {
		FILE* _tmp0;
		if (global_menu_gnome_deferred_init_id != 0) {
			g_source_remove (global_menu_gnome_deferred_init_id);
		}
		if (global_menu_gnome_initialized) {
			dyn_patch_uninit_vfuncs ();
			global_menu_gtk_uninit ();
			dyn_patch_uninit_final ();
		}
		g_debug ("module-main.vala:72: Global Menu plugin module is unloaded");
		/******
		 * this line is causing vala gives two criticals
		 * */
		g_log_set_handler (g_quark_to_string (global_menu_gnome_domain), G_LOG_LEVEL_MASK, (GLogFunc) g_log_default_handler, NULL);
		_tmp0 = NULL;
		global_menu_gnome_log_stream = (_tmp0 = NULL, (global_menu_gnome_log_stream == NULL) ? NULL : (global_menu_gnome_log_stream = (fclose (global_menu_gnome_log_stream), NULL)), _tmp0);
	}
}


static gboolean global_menu_gnome_parse_args (void) {
	GError * inner_error;
	char** _tmp0;
	gint args_size;
	gint args_length1;
	char** args;
	const char* _tmp1;
	char* env;
	gboolean rt;
	gboolean _tmp5;
	inner_error = NULL;
	_tmp0 = NULL;
	args = (_tmp0 = NULL, args_length1 = 0, args_size = args_length1, _tmp0);
	_tmp1 = NULL;
	env = (_tmp1 = g_getenv ("GLOBALMENU_GNOME_ARGS"), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
	rt = TRUE;
	if (env != NULL) {
		char* command_line;
		command_line = g_strconcat ("globalmenu-gnome ", env, NULL);
		{
			gboolean _tmp2;
			_tmp2 = g_shell_parse_argv (command_line, &args_length1, &args, &inner_error);
			if (inner_error != NULL) {
				goto __catch0_g_error;
				goto __finally0;
			}
			rt = _tmp2;
		}
		goto __finally0;
		__catch0_g_error:
		{
			GError * e;
			e = inner_error;
			inner_error = NULL;
			{
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally0:
		if (inner_error != NULL) {
			command_line = (g_free (command_line), NULL);
			args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
			env = (g_free (env), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
			return FALSE;
		}
		if (rt) {
			GOptionContext* context;
			context = g_option_context_new (_ ("- Global Menu plugin Module for GTK"));
			g_option_context_set_description (context, _ ("These parameters should be supplied in environment GLOBALMENU_GNOME_ARGS instead of the command line.\n\t\tNOTE: Environment GTK_MENUBAR_NO_MAC contains the applications to be ignored by the plugin.\n\t\t"));
			g_option_context_set_help_enabled (context, FALSE);
			g_option_context_set_ignore_unknown_options (context, TRUE);
			g_option_context_add_main_entries (context, GLOBAL_MENU_GNOME_options, GETTEXT_PACKAGE);
			{
				gboolean _tmp3;
				_tmp3 = g_option_context_parse (context, &args_length1, &args, &inner_error);
				if (inner_error != NULL) {
					goto __catch1_g_error;
					goto __finally1;
				}
				rt = _tmp3;
			}
			goto __finally1;
			__catch1_g_error:
			{
				GError * e;
				e = inner_error;
				inner_error = NULL;
				{
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
				}
			}
			__finally1:
			if (inner_error != NULL) {
				(context == NULL) ? NULL : (context = (g_option_context_free (context), NULL));
				command_line = (g_free (command_line), NULL);
				args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
				env = (g_free (env), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
				g_clear_error (&inner_error);
				return FALSE;
			}
			(context == NULL) ? NULL : (context = (g_option_context_free (context), NULL));
		}
		command_line = (g_free (command_line), NULL);
	}
	if (global_menu_gnome_log_file_name == NULL) {
		char* _tmp4;
		_tmp4 = NULL;
		global_menu_gnome_log_file_name = (_tmp4 = g_strconcat (g_get_home_dir (), "/.gnomenu.log", NULL), global_menu_gnome_log_file_name = (g_free (global_menu_gnome_log_file_name), NULL), _tmp4);
	}
	return (_tmp5 = rt, args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), env = (g_free (env), NULL), _tmp5);
}


static void global_menu_gnome_prepare_log_file (void) {
	FILE* _tmp0;
	if (!global_menu_gnome_verbose) {
		return;
	}
	_tmp0 = NULL;
	global_menu_gnome_log_stream = (_tmp0 = fopen (global_menu_gnome_log_file_name, "a+"), (global_menu_gnome_log_stream == NULL) ? NULL : (global_menu_gnome_log_stream = (fclose (global_menu_gnome_log_stream), NULL)), _tmp0);
}


static void global_menu_gnome_default_log_handler (const char* domain, GLogLevelFlags level, const char* message) {
	GTimeVal time = {(glong) 0};
	char* s;
	g_return_if_fail (message != NULL);
	if (!global_menu_gnome_verbose) {
		return;
	}
	g_get_current_time (&time);
	s = g_strdup_printf ("%.10ld | %20s | %10s | %s\n", time.tv_usec, g_get_prgname (), domain, message);
	fputs (s, global_menu_gnome_log_stream);
	fflush (global_menu_gnome_log_stream);
	s = (g_free (s), NULL);
}


static gboolean global_menu_gnome_is_quirky_app (void) {
	const char* _tmp0;
	char* disabled_application_names;
	const char* _tmp1;
	char* app_name;
	gboolean _tmp2;
	gboolean _tmp4;
	_tmp0 = NULL;
	disabled_application_names = (_tmp0 = g_getenv ("GTK_MENUBAR_NO_MAC"), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	_tmp1 = NULL;
	app_name = (_tmp1 = g_get_prgname (), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
	_tmp2 = FALSE;
	if (disabled_application_names != NULL) {
		_tmp2 = strstr (disabled_application_names, app_name) != NULL;
	} else {
		_tmp2 = FALSE;
	}
	/* Don't use switch case because vala will create
	 * static quarks which cause core dumps when 
	 * the module is unloaded 
	 Try to figure this out by filtering out 
	 * menubars in 'PanelApplet'
	 * and sub classes of 'PanelMenuBar'
	 * in globalmenu.vala
	if(app_name == "gnome-panel"
	|| app_name == "gdm-user-switch-applet")
	return true;
	*/
	if (_tmp2) {
		gboolean _tmp3;
		return (_tmp3 = TRUE, disabled_application_names = (g_free (disabled_application_names), NULL), app_name = (g_free (app_name), NULL), _tmp3);
	}
	return (_tmp4 = FALSE, disabled_application_names = (g_free (disabled_application_names), NULL), app_name = (g_free (app_name), NULL), _tmp4);
}


GlobalMenuGNOME* global_menu_gnome_new (void) {
	GlobalMenuGNOME* self;
	self = g_slice_new0 (GlobalMenuGNOME);
	global_menu_gnome_instance_init (self);
	return self;
}


static void global_menu_gnome_instance_init (GlobalMenuGNOME * self) {
}


void global_menu_gnome_free (GlobalMenuGNOME* self) {
	g_slice_free (GlobalMenuGNOME, self);
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}




