
#include <libgnomenu/globalmenu.h>
#include <stdlib.h>
#include <string.h>
#include <libgnomenu/window.h>
#include <libgnomenu/menuitem.h>
#include <libgnomenu/label.h>
#include <libgnomenu/parser.h>
#include <libgnomenu/interface-shell.h>
#include <libgnomenu/interface-item.h>




struct _GnomenuGlobalMenuPrivate {
	GnomenuWindow* current_window;
	GnomenuWindow* _root_gnomenu_window;
	GHashTable* keys;
};

#define GNOMENU_GLOBAL_MENU_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOMENU_TYPE_GLOBAL_MENU, GnomenuGlobalMenuPrivate))
enum  {
	GNOMENU_GLOBAL_MENU_DUMMY_PROPERTY
};
static void gnomenu_global_menu_grab_mnemonic_keys (GnomenuGlobalMenu* self);
static void gnomenu_global_menu_ungrab_mnemonic_keys (GnomenuGlobalMenu* self);
static void gnomenu_global_menu_regrab_menu_bar_key (GnomenuGlobalMenu* self);
static void _gnomenu_global_menu_regrab_menu_bar_key_g_object_notify (GtkSettings* _sender, GParamSpec* pspec, gpointer self);
static void gnomenu_global_menu_attach_to_screen (GnomenuGlobalMenu* self, GdkScreen* screen);
static void gnomenu_global_menu_detach_from_screen (GnomenuGlobalMenu* self, GdkScreen* screen);
static gboolean __lambda0 (GtkWidget* w, const GdkEventKey* event, GnomenuGlobalMenu* self);
static gboolean ___lambda0_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self);
static void __lambda1 (GtkWindow* window, GnomenuGlobalMenu* self);
static void ___lambda1_gtk_window_keys_changed (GtkWindow* _sender, gpointer self);
static void gnomenu_global_menu_real_hierarchy_changed (GtkWidget* base, GtkWidget* old_toplevel);
static void gnomenu_global_menu_real_screen_changed (GtkWidget* base, GdkScreen* previous_screen);
static void __lambda2 (GnomenuWindow* window, GnomenuGlobalMenu* self);
static void ___lambda2_gnomenu_window_menu_context_changed (GnomenuWindow* _sender, gpointer self);
static void gnomenu_global_menu_update (GnomenuGlobalMenu* self);
static void gnomenu_global_menu_ungrab_menu_bar_key (GnomenuGlobalMenu* self);
static void gnomenu_global_menu_grab_menu_bar_key (GnomenuGlobalMenu* self);
static void gnomenu_global_menu_get_accel_key (GnomenuGlobalMenu* self, guint* keyval, GdkModifierType* mods);
static void __lambda3 (GnomenuGlobalMenu* menubar, GnomenuMenuItem* item, GnomenuGlobalMenu* self);
static void ___lambda3_gnomenu_menu_bar_activate (GnomenuGlobalMenu* _sender, GnomenuMenuItem* item, gpointer self);
static GObject * gnomenu_global_menu_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gnomenu_global_menu_parent_class = NULL;
static void gnomenu_global_menu_finalize (GObject* obj);



static void gnomenu_global_menu_grab_mnemonic_keys (GnomenuGlobalMenu* self) {
	GdkModifierType mods;
	g_return_if_fail (self != NULL);
	mods = GDK_MOD1_MASK;
	{
		GList* widget_collection;
		GList* widget_it;
		widget_collection = gtk_container_get_children ((GtkContainer*) self);
		for (widget_it = widget_collection; widget_it != NULL; widget_it = widget_it->next) {
			GtkWidget* _tmp5;
			GtkWidget* widget;
			_tmp5 = NULL;
			widget = (_tmp5 = (GtkWidget*) widget_it->data, (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5));
			{
				GnomenuMenuItem* _tmp1;
				GtkWidget* _tmp0;
				GnomenuMenuItem* item;
				GnomenuMenuLabel* _tmp3;
				GtkWidget* _tmp2;
				GnomenuMenuLabel* label;
				guint keyval;
				GtkWidget* _tmp4;
				_tmp1 = NULL;
				_tmp0 = NULL;
				item = (_tmp1 = (_tmp0 = widget, GNOMENU_IS_MENU_ITEM (_tmp0) ? ((GnomenuMenuItem*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
				if (item == NULL) {
					(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
					(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
					continue;
				}
				_tmp3 = NULL;
				_tmp2 = NULL;
				label = (_tmp3 = (_tmp2 = gtk_bin_get_child ((GtkBin*) item), GNOMENU_IS_MENU_LABEL (_tmp2) ? ((GnomenuMenuLabel*) _tmp2) : NULL), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
				if (label == NULL) {
					(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
					(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
					(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
					continue;
				}
				keyval = gnomenu_menu_label_get_mnemonic_keyval (label);
				g_debug ("globalmenu.vala:23: grabbing key for %s:%u", gnomenu_menu_label_get_label (label), keyval);
				if (self->priv->current_window != NULL) {
					gnomenu_window_grab_key (self->priv->current_window, keyval, mods);
				}
				_tmp4 = NULL;
				g_hash_table_insert (self->priv->keys, GUINT_TO_POINTER (keyval), (_tmp4 = widget, (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4)));
				(widget == NULL) ? NULL : (widget = (g_object_unref (widget), NULL));
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
				(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
			}
		}
		(widget_collection == NULL) ? NULL : (widget_collection = (g_list_free (widget_collection), NULL));
	}
}


static void gnomenu_global_menu_ungrab_mnemonic_keys (GnomenuGlobalMenu* self) {
	GdkModifierType mods;
	g_return_if_fail (self != NULL);
	mods = GDK_MOD1_MASK;
	{
		GList* keyval_collection;
		GList* keyval_it;
		keyval_collection = g_hash_table_get_keys (self->priv->keys);
		for (keyval_it = keyval_collection; keyval_it != NULL; keyval_it = keyval_it->next) {
			guint keyval;
			keyval = GPOINTER_TO_UINT (keyval_it->data);
			{
				g_debug ("globalmenu.vala:33: ungrabbing %u", keyval);
				if (self->priv->current_window != NULL) {
					gnomenu_window_ungrab_key (self->priv->current_window, keyval, mods);
				}
			}
		}
		(keyval_collection == NULL) ? NULL : (keyval_collection = (g_list_free (keyval_collection), NULL));
	}
	g_hash_table_remove_all (self->priv->keys);
}


static void gnomenu_global_menu_regrab_menu_bar_key (GnomenuGlobalMenu* self) {
	g_return_if_fail (self != NULL);
	g_debug ("globalmenu.vala:41: regrab menu_bar key");
	gnomenu_global_menu_ungrab_menu_bar_key (self);
	gnomenu_global_menu_grab_menu_bar_key (self);
}


static void _gnomenu_global_menu_regrab_menu_bar_key_g_object_notify (GtkSettings* _sender, GParamSpec* pspec, gpointer self) {
	gnomenu_global_menu_regrab_menu_bar_key (self);
}


static void gnomenu_global_menu_attach_to_screen (GnomenuGlobalMenu* self, GdkScreen* screen) {
	GnomenuWindow* _tmp0;
	GtkSettings* _tmp1;
	GtkSettings* settings;
	g_return_if_fail (self != NULL);
	g_return_if_fail (screen != NULL);
	_tmp0 = NULL;
	self->priv->_root_gnomenu_window = (_tmp0 = gnomenu_window_new (gtk_widget_get_root_window ((GtkWidget*) self)), (self->priv->_root_gnomenu_window == NULL) ? NULL : (self->priv->_root_gnomenu_window = (g_object_unref (self->priv->_root_gnomenu_window), NULL)), _tmp0);
	gnomenu_window_set_key_widget (self->priv->_root_gnomenu_window, gtk_widget_get_toplevel ((GtkWidget*) self));
	gnomenu_global_menu_grab_menu_bar_key (self);
	gnomenu_global_menu_grab_mnemonic_keys (self);
	_tmp1 = NULL;
	settings = (_tmp1 = gtk_widget_get_settings ((GtkWidget*) self), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	g_signal_connect_object ((GObject*) settings, "notify::gtk-menu-bar-accel", (GCallback) _gnomenu_global_menu_regrab_menu_bar_key_g_object_notify, self, 0);
	(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
}


static void gnomenu_global_menu_detach_from_screen (GnomenuGlobalMenu* self, GdkScreen* screen) {
	GtkSettings* _tmp0;
	GtkSettings* settings;
	GQuark _tmp2;
	guint _tmp1;
	GnomenuWindow* _tmp3;
	g_return_if_fail (self != NULL);
	g_return_if_fail (screen != NULL);
	if (self->priv->_root_gnomenu_window != NULL) {
		gnomenu_window_set_key_widget (self->priv->_root_gnomenu_window, NULL);
		gnomenu_global_menu_ungrab_menu_bar_key (self);
		gnomenu_global_menu_ungrab_mnemonic_keys (self);
	}
	_tmp0 = NULL;
	settings = (_tmp0 = gtk_widget_get_settings ((GtkWidget*) self), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	g_signal_handlers_disconnect_matched ((GObject*) settings, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DETAIL | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("notify::gtk-menu-bar-accel", G_TYPE_OBJECT, &_tmp1, &_tmp2, FALSE), _tmp1), 0, NULL, (GCallback) _gnomenu_global_menu_regrab_menu_bar_key_g_object_notify, self);
	_tmp3 = NULL;
	self->priv->_root_gnomenu_window = (_tmp3 = NULL, (self->priv->_root_gnomenu_window == NULL) ? NULL : (self->priv->_root_gnomenu_window = (g_object_unref (self->priv->_root_gnomenu_window), NULL)), _tmp3);
	(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
}


static gboolean __lambda0 (GtkWidget* w, const GdkEventKey* event, GnomenuGlobalMenu* self) {
	g_return_val_if_fail (w != NULL, FALSE);
	g_debug ("globalmenu.vala:66: key %s", (*event).string);
	/*Gtk.Widget widget = keys.lookup(event.keyval);
	if(widget != null) {
	widget.mnemonic_activate(true);
	return true;
	}*/
	return FALSE;
}


static gboolean ___lambda0_gtk_widget_key_press_event (GtkWidget* _sender, const GdkEventKey* event, gpointer self) {
	return __lambda0 (_sender, event, self);
}


static void __lambda1 (GtkWindow* window, GnomenuGlobalMenu* self) {
	GType type;
	GtkWindowClass* window_class;
	g_return_if_fail (window != NULL);
	type = GTK_TYPE_WINDOW;
	window_class = (GtkWindowClass*) g_type_class_ref (type);
	window_class->keys_changed (window);
	g_debug ("globalmenu.vala:78: keys changed");
	(window_class == NULL) ? NULL : (window_class = (g_type_class_unref (window_class), NULL));
}


static void ___lambda1_gtk_window_keys_changed (GtkWindow* _sender, gpointer self) {
	__lambda1 (_sender, self);
}


static void gnomenu_global_menu_real_hierarchy_changed (GtkWidget* base, GtkWidget* old_toplevel) {
	GnomenuGlobalMenu * self;
	GtkWidget* _tmp0;
	self = (GnomenuGlobalMenu*) base;
	g_signal_connect (gtk_widget_get_toplevel ((GtkWidget*) self), "key-press-event", (GCallback) ___lambda0_gtk_widget_key_press_event, self);
	_tmp0 = NULL;
	g_signal_connect ((_tmp0 = gtk_widget_get_toplevel ((GtkWidget*) self), GTK_IS_WINDOW (_tmp0) ? ((GtkWindow*) _tmp0) : NULL), "keys-changed", (GCallback) ___lambda1_gtk_window_keys_changed, self);
}


static void gnomenu_global_menu_real_screen_changed (GtkWidget* base, GdkScreen* previous_screen) {
	GnomenuGlobalMenu * self;
	GdkScreen* _tmp0;
	GdkScreen* screen;
	self = (GnomenuGlobalMenu*) base;
	_tmp0 = NULL;
	screen = (_tmp0 = gtk_widget_get_screen ((GtkWidget*) self), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (previous_screen != screen) {
		if (previous_screen != NULL) {
			gnomenu_global_menu_detach_from_screen (self, previous_screen);
		}
		if (screen != NULL) {
			gnomenu_global_menu_attach_to_screen (self, screen);
		}
	}
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


static void __lambda2 (GnomenuWindow* window, GnomenuGlobalMenu* self) {
	g_return_if_fail (window != NULL);
	gnomenu_global_menu_update (self);
}


static void ___lambda2_gnomenu_window_menu_context_changed (GnomenuWindow* _sender, gpointer self) {
	__lambda2 (_sender, self);
}


void gnomenu_global_menu_switch_to (GnomenuGlobalMenu* self, gulong xid) {
	GnomenuWindow* _tmp0;
	g_return_if_fail (self != NULL);
	gnomenu_global_menu_ungrab_mnemonic_keys (self);
	_tmp0 = NULL;
	self->priv->current_window = (_tmp0 = gnomenu_window_foreign_new (xid), (self->priv->current_window == NULL) ? NULL : (self->priv->current_window = (g_object_unref (self->priv->current_window), NULL)), _tmp0);
	if (self->priv->current_window != NULL) {
		g_signal_connect (self->priv->current_window, "menu-context-changed", (GCallback) ___lambda2_gnomenu_window_menu_context_changed, self);
		gnomenu_global_menu_update (self);
		gnomenu_window_set_key_widget (self->priv->current_window, gtk_widget_get_toplevel ((GtkWidget*) self));
	}
}


static void gnomenu_global_menu_update (GnomenuGlobalMenu* self) {
	GError * inner_error;
	const char* context;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	gnomenu_global_menu_ungrab_mnemonic_keys (self);
	context = gnomenu_window_get_menu_context (self->priv->current_window);
	if (context != NULL) {
		{
			gnomenu_parser_parse ((GnomenuShell*) self, context, &inner_error);
			if (inner_error != NULL) {
				goto __catch2_g_error;
				goto __finally2;
			}
		}
		goto __finally2;
		__catch2_g_error:
		{
			GError * e;
			e = inner_error;
			inner_error = NULL;
			{
				g_warning ("globalmenu.vala:106: %s", e->message);
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally2:
		if (inner_error != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
			return;
		}
		gtk_widget_show ((GtkWidget*) self);
		gnomenu_global_menu_grab_mnemonic_keys (self);
		return;
	}
	gtk_widget_hide ((GtkWidget*) self);
}


static void gnomenu_global_menu_ungrab_menu_bar_key (GnomenuGlobalMenu* self) {
	gint keyval;
	GdkModifierType mods;
	g_return_if_fail (self != NULL);
	keyval = (gint) g_object_get_data ((GObject*) self->priv->_root_gnomenu_window, "menu-bar-keyval");
	mods = (GdkModifierType) g_object_get_data ((GObject*) self->priv->_root_gnomenu_window, "menu-bar-keymods");
	gnomenu_window_ungrab_key (self->priv->_root_gnomenu_window, (guint) keyval, mods);
	g_object_set_data ((GObject*) self->priv->_root_gnomenu_window, "menu-bar-keyval", NULL);
	g_object_set_data ((GObject*) self->priv->_root_gnomenu_window, "menu-bar-keymods", NULL);
}


static void gnomenu_global_menu_grab_menu_bar_key (GnomenuGlobalMenu* self) {
	gint keyval;
	GdkModifierType mods;
	g_return_if_fail (self != NULL);
	/*FIXME: listen to changes in GTK_SETTINGS.*/
	keyval = 0;
	mods = 0;
	gnomenu_global_menu_get_accel_key (self, &keyval, &mods);
	gnomenu_window_grab_key (self->priv->_root_gnomenu_window, (guint) keyval, mods);
	g_object_set_data ((GObject*) self->priv->_root_gnomenu_window, "menu-bar-keyval", (void*) keyval);
	g_object_set_data ((GObject*) self->priv->_root_gnomenu_window, "menu-bar-keymods", (void*) mods);
}


/**
 * return the accelerator key combination for invoking menu bars
 * in GTK Settings. It is usually F10.
 */
static void gnomenu_global_menu_get_accel_key (GnomenuGlobalMenu* self, guint* keyval, GdkModifierType* mods) {
	GtkSettings* _tmp0;
	GtkSettings* settings;
	char* accel;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	settings = (_tmp0 = gtk_widget_get_settings ((GtkWidget*) self), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	accel = NULL;
	g_object_get ((GObject*) settings, "gtk_menu_bar_accel", &accel, NULL, NULL);
	if (accel != NULL) {
		gtk_accelerator_parse (accel, &(*keyval), &(*mods));
	}
	(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
	accel = (g_free (accel), NULL);
}


GnomenuGlobalMenu* gnomenu_global_menu_construct (GType object_type) {
	GnomenuGlobalMenu * self;
	self = (GnomenuGlobalMenu*) gnomenu_menu_bar_construct (object_type);
	return self;
}


GnomenuGlobalMenu* gnomenu_global_menu_new (void) {
	return gnomenu_global_menu_construct (GNOMENU_TYPE_GLOBAL_MENU);
}


static void __lambda3 (GnomenuGlobalMenu* menubar, GnomenuMenuItem* item, GnomenuGlobalMenu* self) {
	g_return_if_fail (menubar != NULL);
	g_return_if_fail (item != NULL);
	if (self->priv->current_window != NULL) {
		char* _tmp0;
		_tmp0 = NULL;
		gnomenu_window_emit_menu_event (self->priv->current_window, _tmp0 = gnomenu_item_get_item_path ((GnomenuItem*) item));
		_tmp0 = (g_free (_tmp0), NULL);
	}
}


static void ___lambda3_gnomenu_menu_bar_activate (GnomenuGlobalMenu* _sender, GnomenuMenuItem* item, gpointer self) {
	__lambda3 (_sender, item, self);
}


static GObject * gnomenu_global_menu_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GnomenuGlobalMenuClass * klass;
	GObjectClass * parent_class;
	GnomenuGlobalMenu * self;
	klass = GNOMENU_GLOBAL_MENU_CLASS (g_type_class_peek (GNOMENU_TYPE_GLOBAL_MENU));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GNOMENU_GLOBAL_MENU (obj);
	{
		g_signal_connect ((GnomenuMenuBar*) self, "activate", (GCallback) ___lambda3_gnomenu_menu_bar_activate, self);
	}
	return obj;
}


static void gnomenu_global_menu_class_init (GnomenuGlobalMenuClass * klass) {
	gnomenu_global_menu_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GnomenuGlobalMenuPrivate));
	G_OBJECT_CLASS (klass)->constructor = gnomenu_global_menu_constructor;
	G_OBJECT_CLASS (klass)->finalize = gnomenu_global_menu_finalize;
	GTK_WIDGET_CLASS (klass)->hierarchy_changed = gnomenu_global_menu_real_hierarchy_changed;
	GTK_WIDGET_CLASS (klass)->screen_changed = gnomenu_global_menu_real_screen_changed;
}


static void gnomenu_global_menu_instance_init (GnomenuGlobalMenu * self) {
	self->priv = GNOMENU_GLOBAL_MENU_GET_PRIVATE (self);
	self->priv->keys = g_hash_table_new (g_direct_hash, g_direct_equal);
}


static void gnomenu_global_menu_finalize (GObject* obj) {
	GnomenuGlobalMenu * self;
	self = GNOMENU_GLOBAL_MENU (obj);
	(self->priv->current_window == NULL) ? NULL : (self->priv->current_window = (g_object_unref (self->priv->current_window), NULL));
	(self->priv->_root_gnomenu_window == NULL) ? NULL : (self->priv->_root_gnomenu_window = (g_object_unref (self->priv->_root_gnomenu_window), NULL));
	(self->priv->keys == NULL) ? NULL : (self->priv->keys = (g_hash_table_unref (self->priv->keys), NULL));
	G_OBJECT_CLASS (gnomenu_global_menu_parent_class)->finalize (obj);
}


GType gnomenu_global_menu_get_type (void) {
	static GType gnomenu_global_menu_type_id = 0;
	if (gnomenu_global_menu_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuGlobalMenuClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gnomenu_global_menu_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GnomenuGlobalMenu), 0, (GInstanceInitFunc) gnomenu_global_menu_instance_init, NULL };
		gnomenu_global_menu_type_id = g_type_register_static (GNOMENU_TYPE_MENU_BAR, "GnomenuGlobalMenu", &g_define_type_info, 0);
	}
	return gnomenu_global_menu_type_id;
}




