
#include <libgnomenu/menu.h>
#include <gdk/gdk.h>
#include <libgnomenu/menushellutils.h>
#include <libgnomenu/menuitem.h>




struct _GnomenuMenuPrivate {
	gboolean disposed;
	gboolean _use_rgba_colormap;
};

#define GNOMENU_MENU_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOMENU_TYPE_MENU, GnomenuMenuPrivate))
enum  {
	GNOMENU_MENU_DUMMY_PROPERTY,
	GNOMENU_MENU_USE_RGBA_COLORMAP,
	GNOMENU_MENU_OWNER,
	GNOMENU_MENU_LENGTH
};
static void gnomenu_menu_real_dispose (GObject* base);
static GnomenuItem* gnomenu_menu_real_get_item (GnomenuShell* base, gint position);
static GnomenuItem* gnomenu_menu_real_get_item_by_id (GnomenuShell* base, const char* id);
static gint gnomenu_menu_real_get_item_position (GnomenuShell* base, GnomenuItem* item);
static void __lambda1 (GtkSettings* settings, GnomenuMenu* self);
static void ___lambda1_g_object_notify (GtkSettings* _sender, GParamSpec* pspec, gpointer self);
static GObject * gnomenu_menu_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gnomenu_menu_parent_class = NULL;
static GnomenuShellIface* gnomenu_menu_gnomenu_shell_parent_iface = NULL;
static void gnomenu_menu_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



GnomenuMenu* gnomenu_menu_construct (GType object_type) {
	GnomenuMenu * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


GnomenuMenu* gnomenu_menu_new (void) {
	return gnomenu_menu_construct (GNOMENU_TYPE_MENU);
}


/*
 * We don't do dispose, but gtk won't reset these pointers
 * related to the colormaps and valgrind won't be happy.
 */
static void gnomenu_menu_real_dispose (GObject* base) {
	GnomenuMenu * self;
	self = (GnomenuMenu*) base;
	if (!self->priv->disposed) {
		self->priv->disposed = TRUE;
	}
	G_OBJECT_CLASS (gnomenu_menu_parent_class)->dispose ((GObject*) GTK_MENU (self));
}


static GnomenuItem* gnomenu_menu_real_get_item (GnomenuShell* base, gint position) {
	GnomenuMenu * self;
	GnomenuItem* _tmp1;
	GtkMenuItem* _tmp0;
	self = (GnomenuMenu*) base;
	_tmp1 = NULL;
	_tmp0 = NULL;
	return (_tmp1 = (_tmp0 = gtk_menu_shell_get_item (self, position), GNOMENU_IS_ITEM (_tmp0) ? ((GnomenuItem*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
}


static GnomenuItem* gnomenu_menu_real_get_item_by_id (GnomenuShell* base, const char* id) {
	GnomenuMenu * self;
	self = (GnomenuMenu*) base;
	g_return_val_if_fail (id != NULL, NULL);
	{
		GList* child_collection;
		GList* child_it;
		child_collection = gtk_container_get_children ((GtkContainer*) self);
		for (child_it = child_collection; child_it != NULL; child_it = child_it->next) {
			GtkWidget* child;
			child = (GtkWidget*) child_it->data;
			{
				GnomenuItem* _tmp1;
				GtkWidget* _tmp0;
				GnomenuItem* item;
				_tmp1 = NULL;
				_tmp0 = NULL;
				item = (_tmp1 = (_tmp0 = child, GNOMENU_IS_ITEM (_tmp0) ? ((GnomenuItem*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
				if (_vala_strcmp0 (gnomenu_item_get_item_id (item), id) == 0) {
					GnomenuItem* _tmp2;
					_tmp2 = NULL;
					return (_tmp2 = item, (child_collection == NULL) ? NULL : (child_collection = (g_list_free (child_collection), NULL)), _tmp2);
				}
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			}
		}
		(child_collection == NULL) ? NULL : (child_collection = (g_list_free (child_collection), NULL));
	}
	return NULL;
}


static gint gnomenu_menu_real_get_item_position (GnomenuShell* base, GnomenuItem* item) {
	GnomenuMenu * self;
	GnomenuItem* _tmp0;
	self = (GnomenuMenu*) base;
	g_return_val_if_fail (item != NULL, 0);
	_tmp0 = NULL;
	return gtk_menu_shell_get_item_position (self, (_tmp0 = item, GNOMENU_IS_MENU_ITEM (_tmp0) ? ((GnomenuMenuItem*) _tmp0) : NULL));
}


gboolean gnomenu_menu_get_use_rgba_colormap (GnomenuMenu* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_use_rgba_colormap;
}


void gnomenu_menu_set_use_rgba_colormap (GnomenuMenu* self, gboolean value) {
	GdkScreen* _tmp1;
	GdkScreen* screen;
	GdkColormap* _tmp2;
	GdkColormap* colormap;
	g_return_if_fail (self != NULL);
	if (self->priv->_use_rgba_colormap == value) {
		return;
	}
	self->priv->_use_rgba_colormap = value;
	_tmp1 = NULL;
	screen = (_tmp1 = gtk_widget_get_screen (((GtkMenu*) self)->toplevel), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	_tmp2 = NULL;
	colormap = (_tmp2 = gdk_screen_get_rgba_colormap (screen), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
	if (colormap != NULL) {
		gtk_widget_set_colormap (((GtkMenu*) self)->toplevel, colormap);
		gtk_widget_set_colormap ((GtkWidget*) self, colormap);
	}
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
	(colormap == NULL) ? NULL : (colormap = (g_object_unref (colormap), NULL));
	g_object_notify ((GObject *) self, "use-rgba-colormap");
}


static GnomenuItem* gnomenu_menu_real_get_owner (GnomenuShell* base) {
	GnomenuMenu* self;
	GtkWidget* _tmp0;
	self = (GnomenuMenu*) base;
	_tmp0 = NULL;
	return (_tmp0 = gtk_menu_get_attach_widget ((GtkMenu*) self), GNOMENU_IS_ITEM (_tmp0) ? ((GnomenuItem*) _tmp0) : NULL);
}


static gint gnomenu_menu_real_get_length (GnomenuShell* base) {
	GnomenuMenu* self;
	self = (GnomenuMenu*) base;
	return gtk_menu_shell_length_without_truncated (self);
}


static void gnomenu_menu_real_set_length (GnomenuShell* base, gint value) {
	GnomenuMenu* self;
	self = (GnomenuMenu*) base;
	gtk_menu_shell_truncate (self, value);
	g_object_notify ((GObject *) self, "length");
}


static void __lambda1 (GtkSettings* settings, GnomenuMenu* self) {
	gboolean val;
	g_return_if_fail (settings != NULL);
	val = FALSE;
	g_object_get ((GObject*) settings, "use-rgba-colormap", &val, NULL, NULL);
	gnomenu_menu_set_use_rgba_colormap (self, val);
}


static void ___lambda1_g_object_notify (GtkSettings* _sender, GParamSpec* pspec, gpointer self) {
	__lambda1 (_sender, self);
}


static GObject * gnomenu_menu_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GnomenuMenuClass * klass;
	GObjectClass * parent_class;
	GnomenuMenu * self;
	klass = GNOMENU_MENU_CLASS (g_type_class_peek (GNOMENU_TYPE_MENU));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GNOMENU_MENU (obj);
	{
		gboolean val;
		g_signal_connect ((GObject*) gtk_widget_get_settings ((GtkWidget*) self), "notify::use-rgba-colormap", (GCallback) ___lambda1_g_object_notify, self);
		val = FALSE;
		g_object_get ((GObject*) gtk_widget_get_settings ((GtkWidget*) self), "use-rgba-colormap", &val, NULL, NULL);
		gnomenu_menu_set_use_rgba_colormap (self, val);
	}
	return obj;
}


static void gnomenu_menu_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GnomenuMenu * self;
	gpointer boxed;
	self = GNOMENU_MENU (object);
	switch (property_id) {
		case GNOMENU_MENU_USE_RGBA_COLORMAP:
		g_value_set_boolean (value, gnomenu_menu_get_use_rgba_colormap (self));
		break;
		case GNOMENU_MENU_OWNER:
		g_value_set_object (value, gnomenu_shell_get_owner ((GnomenuShell*) self));
		break;
		case GNOMENU_MENU_LENGTH:
		g_value_set_int (value, gnomenu_shell_get_length ((GnomenuShell*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gnomenu_menu_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GnomenuMenu * self;
	self = GNOMENU_MENU (object);
	switch (property_id) {
		case GNOMENU_MENU_USE_RGBA_COLORMAP:
		gnomenu_menu_set_use_rgba_colormap (self, g_value_get_boolean (value));
		break;
		case GNOMENU_MENU_LENGTH:
		gnomenu_shell_set_length ((GnomenuShell*) self, g_value_get_int (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gnomenu_menu_class_init (GnomenuMenuClass * klass) {
	gnomenu_menu_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GnomenuMenuPrivate));
	G_OBJECT_CLASS (klass)->get_property = gnomenu_menu_get_property;
	G_OBJECT_CLASS (klass)->set_property = gnomenu_menu_set_property;
	G_OBJECT_CLASS (klass)->constructor = gnomenu_menu_constructor;
	G_OBJECT_CLASS (klass)->finalize = gnomenu_menu_finalize;
	G_OBJECT_CLASS (klass)->dispose = gnomenu_menu_real_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_USE_RGBA_COLORMAP, g_param_spec_boolean ("use-rgba-colormap", "use-rgba-colormap", "use-rgba-colormap", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_OWNER, "owner");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_LENGTH, "length");
	{
		GParamSpecBoolean* _tmp0;
		_tmp0 = NULL;
		gtk_settings_install_property ((GParamSpec*) (_tmp0 = g_param_spec_boolean ("use-rgba-colormap", "use RGBA colormap if possible", "Use RGBA colormap if possible", FALSE, G_PARAM_READABLE | G_PARAM_WRITABLE)));
		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_param_spec_unref (_tmp0), NULL));
	}
}


static void gnomenu_menu_gnomenu_shell_interface_init (GnomenuShellIface * iface) {
	gnomenu_menu_gnomenu_shell_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_item = gnomenu_menu_real_get_item;
	iface->get_item_by_id = gnomenu_menu_real_get_item_by_id;
	iface->get_item_position = gnomenu_menu_real_get_item_position;
	iface->get_owner = gnomenu_menu_real_get_owner;
	iface->get_length = gnomenu_menu_real_get_length;
	iface->set_length = gnomenu_menu_real_set_length;
}


static void gnomenu_menu_instance_init (GnomenuMenu * self) {
	self->priv = GNOMENU_MENU_GET_PRIVATE (self);
	self->priv->disposed = FALSE;
	self->priv->_use_rgba_colormap = FALSE;
}


static void gnomenu_menu_finalize (GObject* obj) {
	GnomenuMenu * self;
	self = GNOMENU_MENU (obj);
	G_OBJECT_CLASS (gnomenu_menu_parent_class)->finalize (obj);
}


GType gnomenu_menu_get_type (void) {
	static GType gnomenu_menu_type_id = 0;
	if (gnomenu_menu_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuMenuClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gnomenu_menu_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GnomenuMenu), 0, (GInstanceInitFunc) gnomenu_menu_instance_init, NULL };
		static const GInterfaceInfo gnomenu_shell_info = { (GInterfaceInitFunc) gnomenu_menu_gnomenu_shell_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gnomenu_menu_type_id = g_type_register_static (GTK_TYPE_MENU, "GnomenuMenu", &g_define_type_info, 0);
		g_type_add_interface_static (gnomenu_menu_type_id, GNOMENU_TYPE_SHELL, &gnomenu_shell_info);
	}
	return gnomenu_menu_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




