
#include <libgnomenu/menuitem.h>
#include <pango/pango.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <glib/gi18n-lib.h>
#include <libgnomenu/label.h>
#include <libgnomenu/menubar.h>




struct _GnomenuMenuItemPrivate {
	gint _max_width_chars;
	gboolean _has_sub_shell;
	gboolean _client_side_sub_shell;
	gboolean _item_visible;
	gboolean disposed;
	gboolean _truncated;
	char* _font;
	char* _label;
	char* _icon;
	char* _accel_text;
	char* _id;
	GnomenuGravity _gravity;
	GnomenuItemType _item_type;
	GnomenuItemState _item_state;
	gboolean _use_underline;
	GtkImage* _cached_image_widget;
	GtkArrow* _cached_arrow_widget;
	GnomenuMenuLabel* _cached_label_widget;
	GtkImage* _image_widget;
	GtkImage* _icon_widget;
};

#define GNOMENU_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOMENU_TYPE_MENU_ITEM, GnomenuMenuItemPrivate))
enum  {
	GNOMENU_MENU_ITEM_DUMMY_PROPERTY,
	GNOMENU_MENU_ITEM_MAX_WIDTH_CHARS,
	GNOMENU_MENU_ITEM_SHELL,
	GNOMENU_MENU_ITEM_SUB_SHELL,
	GNOMENU_MENU_ITEM_HAS_SUB_SHELL,
	GNOMENU_MENU_ITEM_CLIENT_SIDE_SUB_SHELL,
	GNOMENU_MENU_ITEM_ITEM_ID,
	GNOMENU_MENU_ITEM_ITEM_LABEL,
	GNOMENU_MENU_ITEM_ITEM_ICON,
	GNOMENU_MENU_ITEM_ITEM_ACCEL_TEXT,
	GNOMENU_MENU_ITEM_ITEM_FONT,
	GNOMENU_MENU_ITEM_ITEM_TYPE,
	GNOMENU_MENU_ITEM_ITEM_USE_UNDERLINE,
	GNOMENU_MENU_ITEM_ITEM_STATE,
	GNOMENU_MENU_ITEM_ITEM_VISIBLE,
	GNOMENU_MENU_ITEM_ITEM_SENSITIVE,
	GNOMENU_MENU_ITEM_GRAVITY,
	GNOMENU_MENU_ITEM_TRUNCATED,
	GNOMENU_MENU_ITEM_IMAGE
};
static gint gnomenu_menu_item_icon_width = 0;
static gint gnomenu_menu_item_icon_height = 0;
static void gnomenu_menu_item_real_dispose (GObject* base);
static void gnomenu_menu_item_maybe_connect_cached_submenu (GnomenuMenuItem* self);
static void gnomenu_menu_item_real_toggle_size_request (GtkMenuItem* base, void* requisition);
static gboolean gnomenu_menu_item_real_expose_event (GtkWidget* base, const GdkEventExpose* event);
static void gnomenu_menu_item_real_forall (GtkContainer* base, gboolean include_internals, GtkCallback callback, void* callback_target);
static void gnomenu_menu_item_real_activate (GtkMenuItem* base);
static void gnomenu_menu_item_update_show_image (GnomenuMenuItem* self);
static void gnomenu_menu_item_show_image_notify_r (GtkWidget* widget, GtkSettings* settings);
static void gnomenu_menu_item_show_image_notify (GtkSettings* settings);
static void _gnomenu_menu_item_show_image_notify_g_object_notify (GtkSettings* _sender, GParamSpec* pspec, gpointer self);
static void gnomenu_menu_item_real_screen_changed (GtkWidget* base, GdkScreen* previous_screen);
static void gnomenu_menu_item_real_size_request (GtkWidget* base, GtkRequisition* req);
static void gnomenu_menu_item_real_size_allocate (GtkWidget* base, const GdkRectangle* a);
static void gnomenu_menu_item_update_font (GnomenuMenuItem* self);
static void gnomenu_menu_item_update_label_text (GnomenuMenuItem* self);
static void gnomenu_menu_item_update_label_underline (GnomenuMenuItem* self);
static void gnomenu_menu_item_update_image (GnomenuMenuItem* self);
static void gnomenu_menu_item_real_parent_set (GtkWidget* base, GtkWidget* old_parent);
static void gnomenu_menu_item_create_labels (GnomenuMenuItem* self);
static GnomenuMenuLabel* gnomenu_menu_item_get_label_widget (GnomenuMenuItem* self);
static void gnomenu_menu_item_remove_child (GnomenuMenuItem* self);
static void gnomenu_menu_item_create_arrow (GnomenuMenuItem* self);
static void gnomenu_menu_item_update_arrow_type (GnomenuMenuItem* self);
static gboolean gnomenu_menu_item_get__show_image (GnomenuMenuItem* self);
static GtkPackDirection gnomenu_menu_item_get_pack_direction (GnomenuMenuItem* self);
static GObject * gnomenu_menu_item_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gnomenu_menu_item_parent_class = NULL;
static GnomenuItemIface* gnomenu_menu_item_gnomenu_item_parent_iface = NULL;
static void gnomenu_menu_item_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



GnomenuMenuItem* gnomenu_menu_item_construct (GType object_type) {
	GnomenuMenuItem * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


GnomenuMenuItem* gnomenu_menu_item_new (void) {
	return gnomenu_menu_item_construct (GNOMENU_TYPE_MENU_ITEM);
}


static void gnomenu_menu_item_real_dispose (GObject* base) {
	GnomenuMenuItem * self;
	self = (GnomenuMenuItem*) base;
	if (!self->priv->disposed) {
		self->priv->disposed = TRUE;
		if (self->priv->_image_widget != NULL) {
			GtkImage* _tmp0;
			gtk_widget_unparent ((GtkWidget*) self->priv->_image_widget);
			_tmp0 = NULL;
			self->priv->_image_widget = (_tmp0 = NULL, (self->priv->_image_widget == NULL) ? NULL : (self->priv->_image_widget = (g_object_unref (self->priv->_image_widget), NULL)), _tmp0);
		}
	}
	G_OBJECT_CLASS (gnomenu_menu_item_parent_class)->dispose ((GObject*) GTK_MENU_ITEM (self));
}


static void gnomenu_menu_item_maybe_connect_cached_submenu (GnomenuMenuItem* self) {
	gboolean _tmp0;
	g_return_if_fail (self != NULL);
	_tmp0 = FALSE;
	if (self->priv->_has_sub_shell) {
		_tmp0 = !self->priv->_client_side_sub_shell;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		g_debug ("menuitem.vala:81: maybe?");
		if (gtk_menu_item_get_submenu ((GtkMenuItem*) self) != GTK_MENU (self->_submenu_cache)) {
			gtk_menu_item_set_submenu ((GtkMenuItem*) self, (GtkMenu*) self->_submenu_cache);
		}
	} else {
		if (gtk_menu_item_get_submenu ((GtkMenuItem*) self) != NULL) {
			gtk_menu_popdown (gtk_menu_item_get_submenu ((GtkMenuItem*) self));
		}
		gtk_menu_item_set_submenu ((GtkMenuItem*) self, NULL);
	}
}


static void gnomenu_menu_item_real_toggle_size_request (GtkMenuItem* base, void* requisition) {
	GnomenuMenuItem * self;
	gint toggle_spacing;
	gint indicator_size;
	self = (GnomenuMenuItem*) base;
	toggle_spacing = 0;
	indicator_size = 0;
	gtk_widget_style_get ((GtkWidget*) self, "toggle-spacing", &toggle_spacing, "indicator-size", &indicator_size, NULL, NULL);
	switch (self->priv->_item_type) {
		case GNOMENU_ITEM_TYPE_CHECK:
		case GNOMENU_ITEM_TYPE_RADIO:
		{
			*((gint*) requisition) = indicator_size + toggle_spacing;
			break;
		}
		case GNOMENU_ITEM_TYPE_IMAGE:
		{
			gboolean _tmp0;
			if (!gnomenu_menu_item_get__show_image (self)) {
				*((gint*) requisition) = 0;
				break;
			}
			_tmp0 = FALSE;
			if (gnomenu_menu_item_get_image (self) != NULL) {
				_tmp0 = self->priv->_icon != NULL;
			} else {
				_tmp0 = FALSE;
			}
			if (_tmp0) {
				GtkRequisition req = {0};
				gboolean _tmp1;
				gboolean _tmp2;
				gtk_widget_size_request ((GtkWidget*) gnomenu_menu_item_get_image (self), &req);
				_tmp1 = FALSE;
				if (gnomenu_menu_item_get_pack_direction (self) == GTK_PACK_DIRECTION_LTR) {
					_tmp1 = TRUE;
				} else {
					_tmp1 = gnomenu_menu_item_get_pack_direction (self) == GTK_PACK_DIRECTION_RTL;
				}
				if (_tmp1) {
					*((gint*) requisition) = req.width + toggle_spacing;
				}
				_tmp2 = FALSE;
				if (gnomenu_menu_item_get_pack_direction (self) == GTK_PACK_DIRECTION_TTB) {
					_tmp2 = TRUE;
				} else {
					_tmp2 = gnomenu_menu_item_get_pack_direction (self) == GTK_PACK_DIRECTION_BTT;
				}
				if (_tmp2) {
					*((gint*) requisition) = req.height + toggle_spacing;
				}
			} else {
				/*no image*/
				*((gint*) requisition) = 0;
			}
			break;
		}
		default:
		{
			*((gint*) requisition) = 0;
			break;
		}
	}
}


/**
 *
 * The parent handler is first invoked.
 * Then the check box or the radio box
 * is drawn.
 *
 */
static gboolean gnomenu_menu_item_real_expose_event (GtkWidget* base, const GdkEventExpose* event) {
	GnomenuMenuItem * self;
	gint toggle_spacing;
	gint indicator_size;
	gint horizontal_padding;
	GtkShadowType shadow_type;
	gint x;
	gint y;
	gint xoffset;
	gint yoffset;
	gint spacing;
	self = (GnomenuMenuItem*) base;
	GTK_WIDGET_CLASS (gnomenu_menu_item_parent_class)->expose_event ((GtkWidget*) GTK_MENU_ITEM (self), &(*event));
	toggle_spacing = 0;
	indicator_size = 0;
	horizontal_padding = 0;
	gtk_widget_style_get ((GtkWidget*) self, "toggle-spacing", &toggle_spacing, "indicator-size", &indicator_size, "horizontal-padding", &horizontal_padding, NULL, NULL);
	shadow_type = gnomenu_menu_item_item_state_to_shadow_type (self->priv->_item_state);
	/*FIXME: alignment !*/
	x = 0;
	y = 0;
	xoffset = (((GtkMenuItem*) self)->toggle_size - indicator_size) / 2;
	yoffset = (((GtkWidget*) self)->allocation.height - indicator_size) / 2;
	spacing = toggle_spacing / 2;
	switch (gtk_widget_get_direction ((GtkWidget*) self)) {
		case GTK_TEXT_DIR_LTR:
		{
			x = (((GtkWidget*) self)->allocation.x + xoffset) + spacing;
			y = ((GtkWidget*) self)->allocation.y + yoffset;
			break;
		}
		case GTK_TEXT_DIR_RTL:
		{
			x = (((((GtkWidget*) self)->allocation.x + ((GtkWidget*) self)->allocation.width) - ((GtkMenuItem*) self)->toggle_size) - xoffset) - spacing;
			y = ((GtkWidget*) self)->allocation.y + yoffset;
			break;
		}
	}
	switch (self->priv->_item_type) {
		case GNOMENU_ITEM_TYPE_CHECK:
		{
			gtk_paint_check (gtk_widget_get_style ((GtkWidget*) self), ((GtkWidget*) self)->window, (GtkStateType) ((GtkWidget*) self)->state, shadow_type, &(*event).area, (GtkWidget*) self, "check", x, y, indicator_size, indicator_size);
			break;
		}
		case GNOMENU_ITEM_TYPE_RADIO:
		{
			gtk_paint_option (gtk_widget_get_style ((GtkWidget*) self), ((GtkWidget*) self)->window, (GtkStateType) ((GtkWidget*) self)->state, shadow_type, &(*event).area, (GtkWidget*) self, "option", x, y, indicator_size, indicator_size);
			break;
		}
	}
	return FALSE;
}


/**
 * Overriding Gtk.Container.forall.
 *
 * There is a noterious problem in Vala Gtk bindings, causing
 * the ccode function differs from GtkContainerClass.forall.
 *
 * Therefore this function's ccode is patched with patch.sh
 * to avoid the problems. the dummy variable include_internals
 * should be kept here.
 *
 */
static void gnomenu_menu_item_real_forall (GtkContainer* base, gboolean include_internals, GtkCallback callback, void* callback_target) {
	GnomenuMenuItem * self;
	self = (GnomenuMenuItem*) base;
	if (include_internals) {
		gboolean _tmp0;
		_tmp0 = FALSE;
		if (self->priv->_item_type == GNOMENU_ITEM_TYPE_IMAGE) {
			_tmp0 = self->priv->_image_widget != NULL;
		} else {
			_tmp0 = FALSE;
		}
		/*_image_widget still can be null
		 * if item type is IMAGE,
		 * -- forall can be called during the construction
		 *  of the widget by remove_child()*/
		if (_tmp0) {
			callback ((GtkWidget*) self->priv->_image_widget, callback_target);
		}
	}
	GTK_CONTAINER_CLASS (gnomenu_menu_item_parent_class)->forall ((GtkContainer*) GTK_MENU_ITEM (self), include_internals, callback, callback_target);
}


static void gnomenu_menu_item_real_activate (GtkMenuItem* base) {
	GnomenuMenuItem * self;
	GnomenuShell* _tmp0;
	self = (GnomenuMenuItem*) base;
	_tmp0 = NULL;
	gnomenu_menu_bar_emit_activate ((_tmp0 = gnomenu_item_get_toplevel_shell ((GnomenuItem*) self), GNOMENU_IS_MENU_BAR (_tmp0) ? ((GnomenuMenuBar*) _tmp0) : NULL), self);
}


static void gnomenu_menu_item_update_show_image (GnomenuMenuItem* self) {
	g_return_if_fail (self != NULL);
	if (self->priv->_image_widget != NULL) {
		g_object_set ((GtkWidget*) self->priv->_image_widget, "visible", gnomenu_menu_item_get__show_image (self), NULL);
	}
	gtk_widget_queue_resize ((GtkWidget*) self);
}


static void gnomenu_menu_item_show_image_notify_r (GtkWidget* widget, GtkSettings* settings) {
	g_return_if_fail (widget != NULL);
	g_return_if_fail (settings != NULL);
	if (GNOMENU_IS_MENU_ITEM (widget)) {
		GnomenuMenuItem* _tmp1;
		GtkWidget* _tmp0;
		GnomenuMenuItem* item;
		_tmp1 = NULL;
		_tmp0 = NULL;
		item = (_tmp1 = (_tmp0 = widget, GNOMENU_IS_MENU_ITEM (_tmp0) ? ((GnomenuMenuItem*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
		gnomenu_menu_item_update_show_image (item);
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
	} else {
		if (GTK_IS_CONTAINER (widget)) {
			GtkWidget* _tmp2;
			GList* children;
			_tmp2 = NULL;
			children = gtk_container_get_children ((_tmp2 = widget, GTK_IS_CONTAINER (_tmp2) ? ((GtkContainer*) _tmp2) : NULL));
			{
				GList* child_collection;
				GList* child_it;
				child_collection = children;
				for (child_it = child_collection; child_it != NULL; child_it = child_it->next) {
					GtkWidget* _tmp3;
					GtkWidget* child;
					_tmp3 = NULL;
					child = (_tmp3 = (GtkWidget*) child_it->data, (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
					{
						gnomenu_menu_item_show_image_notify_r (child, settings);
						(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
					}
				}
			}
			(children == NULL) ? NULL : (children = (g_list_free (children), NULL));
		}
	}
}


static void gnomenu_menu_item_show_image_notify (GtkSettings* settings) {
	GList* toplevels;
	g_return_if_fail (settings != NULL);
	toplevels = gtk_window_list_toplevels ();
	{
		GList* c_collection;
		GList* c_it;
		c_collection = toplevels;
		for (c_it = c_collection; c_it != NULL; c_it = c_it->next) {
			GtkContainer* _tmp0;
			GtkContainer* c;
			_tmp0 = NULL;
			c = (_tmp0 = (GtkContainer*) ((GtkWindow*) c_it->data), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
			{
				gnomenu_menu_item_show_image_notify_r ((GtkWidget*) c, settings);
				(c == NULL) ? NULL : (c = (g_object_unref (c), NULL));
			}
		}
	}
	(toplevels == NULL) ? NULL : (toplevels = (g_list_free (toplevels), NULL));
}


static void _gnomenu_menu_item_show_image_notify_g_object_notify (GtkSettings* _sender, GParamSpec* pspec, gpointer self) {
	gnomenu_menu_item_show_image_notify (_sender);
}


static void gnomenu_menu_item_real_screen_changed (GtkWidget* base, GdkScreen* previous_screen) {
	GnomenuMenuItem * self;
	GtkSettings* _tmp0;
	GtkSettings* settings;
	self = (GnomenuMenuItem*) base;
	if (!gtk_widget_has_screen ((GtkWidget*) self)) {
		return;
	}
	_tmp0 = NULL;
	settings = (_tmp0 = gtk_widget_get_settings ((GtkWidget*) self), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (g_object_get_data ((GObject*) settings, "gnomenu-menu-item-connection") == NULL) {
		g_signal_connect ((GObject*) settings, "notify::gtk-menu-images", (GCallback) _gnomenu_menu_item_show_image_notify_g_object_notify, NULL);
		/*set it to non-null value*/
		g_object_set_data ((GObject*) settings, "gnomenu-menu-item-connection", settings);
	}
	/*if there is no such connection yet, 
	 * this widget must have been the first
	 * item attached to the screen, therefore there is no
	 * need to do a show_image_notify to all widgets.
	 * 
	If the connection to settings is already established,
	 * updating this widget is sufficient*/
	gnomenu_menu_item_update_show_image (self);
	(settings == NULL) ? NULL : (settings = (g_object_unref (settings), NULL));
}


static void gnomenu_menu_item_real_size_request (GtkWidget* base, GtkRequisition* req) {
	GnomenuMenuItem * self;
	self = (GnomenuMenuItem*) base;
	if (self->priv->_item_type == GNOMENU_ITEM_TYPE_IMAGE) {
		GtkRequisition image_req = {0};
		gtk_widget_size_request ((GtkWidget*) self->priv->_image_widget, &image_req);
	}
	/*Then throw it away*/
	if (self->priv->_item_type == GNOMENU_ITEM_TYPE_ICON) {
		gint horizontal_padding;
		horizontal_padding = 0;
		gtk_widget_style_get ((GtkWidget*) self, "horizontal-padding", &horizontal_padding, NULL, NULL);
		gtk_widget_size_request ((GtkWidget*) self->priv->_icon_widget, &(*req));
		(*req).width = (*req).width + ((((gint) gtk_container_get_border_width ((GtkContainer*) self)) * 2) + (horizontal_padding * 2));
		(*req).height = (*req).height + (((gint) gtk_container_get_border_width ((GtkContainer*) self)) * 2);
	} else {
		GTK_WIDGET_CLASS (gnomenu_menu_item_parent_class)->size_request ((GtkWidget*) GTK_MENU_ITEM (self), &(*req));
	}
}


static void gnomenu_menu_item_real_size_allocate (GtkWidget* base, const GdkRectangle* a) {
	GnomenuMenuItem * self;
	GdkRectangle ca = {0, 0, 0, 0};
	self = (GnomenuMenuItem*) base;
	if (self->priv->_item_type == GNOMENU_ITEM_TYPE_ICON) {
		gint horizontal_padding;
		horizontal_padding = 0;
		gtk_widget_style_get ((GtkWidget*) self, "horizontal-padding", &horizontal_padding, NULL, NULL);
		ca.x = ((*a).x + ((gint) gtk_container_get_border_width ((GtkContainer*) self))) + horizontal_padding;
		ca.y = (*a).y + ((gint) gtk_container_get_border_width ((GtkContainer*) self));
		ca.width = ((*a).width - (((gint) gtk_container_get_border_width ((GtkContainer*) self)) * 2)) - horizontal_padding;
		ca.height = (*a).height - (((gint) gtk_container_get_border_width ((GtkContainer*) self)) * 2);
		gtk_widget_size_allocate ((GtkWidget*) self->priv->_icon_widget, &ca);
		if ((GTK_WIDGET_FLAGS ((GtkWidget*) self) & GTK_REALIZED) != 0) {
			gdk_window_move_resize (((GtkMenuItem*) self)->event_window, ((GtkWidget*) self)->allocation.x, ((GtkWidget*) self)->allocation.y, ((GtkWidget*) self)->allocation.width, ((GtkWidget*) self)->allocation.height);
		}
		((GtkWidget*) self)->allocation = (GtkAllocation) (*a);
	} else {
		GTK_WIDGET_CLASS (gnomenu_menu_item_parent_class)->size_allocate ((GtkWidget*) GTK_MENU_ITEM (self), &(*a));
	}
	if (self->priv->_item_type == GNOMENU_ITEM_TYPE_IMAGE) {
		gint toggle_spacing;
		GtkRequisition icon_req = {0};
		/*FIXME: alignment !*/
		toggle_spacing = 0;
		gtk_widget_get_child_requisition ((GtkWidget*) self->priv->_image_widget, &icon_req);
		gtk_widget_style_get ((GtkWidget*) self, "toggle-spacing", &toggle_spacing, NULL, NULL);
		ca.width = icon_req.width;
		ca.height = icon_req.height;
		switch (gnomenu_menu_item_get_pack_direction (self)) {
			case GTK_PACK_DIRECTION_LTR:
			case GTK_PACK_DIRECTION_RTL:
			{
				gint xoffset;
				gint yoffset;
				gboolean _tmp0;
				xoffset = ((((GtkMenuItem*) self)->toggle_size - icon_req.width) + toggle_spacing) / 2;
				yoffset = ((*a).height - icon_req.height) / 2;
				ca.y = (*a).y + yoffset;
				_tmp0 = FALSE;
				if (gtk_widget_get_direction ((GtkWidget*) self) == GTK_TEXT_DIR_LTR) {
					_tmp0 = gnomenu_menu_item_get_pack_direction (self) == GTK_PACK_DIRECTION_LTR;
				} else {
					_tmp0 = FALSE;
				}
				if (_tmp0) {
					ca.x = (*a).x + xoffset;
				} else {
					ca.x = (((*a).x + (*a).width) - ca.width) - xoffset;
				}
				break;
			}
			case GTK_PACK_DIRECTION_TTB:
			case GTK_PACK_DIRECTION_BTT:
			{
				gint yoffset;
				gint xoffset;
				gboolean _tmp1;
				yoffset = ((((GtkMenuItem*) self)->toggle_size - icon_req.height) + toggle_spacing) / 2;
				xoffset = ((*a).width - icon_req.width) / 2;
				ca.x = (*a).x + xoffset;
				_tmp1 = FALSE;
				if (gtk_widget_get_direction ((GtkWidget*) self) == GTK_TEXT_DIR_LTR) {
					_tmp1 = gnomenu_menu_item_get_pack_direction (self) == GTK_PACK_DIRECTION_TTB;
				} else {
					_tmp1 = FALSE;
				}
				if (_tmp1) {
					ca.y = (*a).y + yoffset;
				} else {
					ca.y = (((*a).y + (*a).height) - ca.height) - yoffset;
				}
				break;
			}
		}
		gtk_widget_size_allocate ((GtkWidget*) self->priv->_image_widget, &ca);
	}
}


static void gnomenu_menu_item_update_font (GnomenuMenuItem* self) {
	PangoFontDescription* desc;
	GtkWidget* bin_child;
	g_return_if_fail (self != NULL);
	desc = NULL;
	if (self->priv->_font != NULL) {
		PangoFontDescription* _tmp0;
		_tmp0 = NULL;
		desc = (_tmp0 = pango_font_description_from_string (self->priv->_font), (desc == NULL) ? NULL : (desc = (pango_font_description_free (desc), NULL)), _tmp0);
	} else {
		PangoFontDescription* _tmp1;
		_tmp1 = NULL;
		desc = (_tmp1 = NULL, (desc == NULL) ? NULL : (desc = (pango_font_description_free (desc), NULL)), _tmp1);
	}
	bin_child = gtk_bin_get_child ((GtkBin*) self);
	gtk_widget_modify_font (bin_child, desc);
	(desc == NULL) ? NULL : (desc = (pango_font_description_free (desc), NULL));
}


static void gnomenu_menu_item_update_label_text (GnomenuMenuItem* self) {
	char* text;
	char* _tmp1;
	const char* _tmp0;
	GnomenuMenuLabel* _tmp3;
	GnomenuMenuLabel* label;
	g_return_if_fail (self != NULL);
	if (!gnomenu_item_type_has_label (self->priv->_item_type)) {
		return;
	}
	text = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	text = (_tmp1 = (_tmp0 = self->priv->_label, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), text = (g_free (text), NULL), _tmp1);
	if (text == NULL) {
		char* _tmp2;
		_tmp2 = NULL;
		text = (_tmp2 = gnomenu_item_get_item_path ((GnomenuItem*) self), text = (g_free (text), NULL), _tmp2);
	}
	_tmp3 = NULL;
	label = (_tmp3 = gnomenu_menu_item_get_label_widget (self), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
	;
	g_assert (label != NULL);
	gnomenu_menu_label_set_label (label, text);
	gnomenu_menu_label_set_accel (label, self->priv->_accel_text);
	gnomenu_menu_label_set_max_width_chars (label, self->priv->_max_width_chars);
	text = (g_free (text), NULL);
	(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
}


static void gnomenu_menu_item_update_label_underline (GnomenuMenuItem* self) {
	GnomenuMenuLabel* _tmp0;
	GnomenuMenuLabel* label;
	g_return_if_fail (self != NULL);
	if (!gnomenu_item_type_has_label (self->priv->_item_type)) {
		return;
	}
	_tmp0 = NULL;
	label = (_tmp0 = gnomenu_menu_item_get_label_widget (self), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	;
	g_assert (label != NULL);
	gnomenu_menu_label_set_use_underline (label, self->priv->_use_underline);
	(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
}


static void gnomenu_menu_item_update_image (GnomenuMenuItem* self) {
	GError * inner_error;
	gboolean _tmp0;
	gboolean _tmp1;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	_tmp0 = FALSE;
	if (self->priv->_item_type != GNOMENU_ITEM_TYPE_IMAGE) {
		_tmp0 = self->priv->_item_type != GNOMENU_ITEM_TYPE_ICON;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		return;
	}
	_tmp1 = FALSE;
	if (self->priv->_icon != NULL) {
		_tmp1 = g_str_has_prefix (self->priv->_icon, "theme:");
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		const char* icon_name;
		icon_name = g_utf8_offset_to_pointer (self->priv->_icon, (glong) 6);
		gtk_image_set_from_icon_name (gnomenu_menu_item_get_image (self), icon_name, GTK_ICON_SIZE_MENU);
	} else {
		gboolean _tmp2;
		_tmp2 = FALSE;
		if (self->priv->_icon != NULL) {
			_tmp2 = g_str_has_prefix (self->priv->_icon, "file:");
		} else {
			_tmp2 = FALSE;
		}
		if (_tmp2) {
			const char* filename;
			filename = g_utf8_offset_to_pointer (self->priv->_icon, (glong) 5);
			gtk_image_set_from_file (gnomenu_menu_item_get_image (self), filename);
		} else {
			gboolean _tmp3;
			_tmp3 = FALSE;
			if (self->priv->_icon != NULL) {
				_tmp3 = g_str_has_prefix (self->priv->_icon, "pixbuf:");
			} else {
				_tmp3 = FALSE;
			}
			if (_tmp3) {
				const char* b64data;
				gint len;
				guchar* _tmp4;
				gint data_size;
				gint data_length1;
				guchar* data;
				GdkPixdata pixdata = {(guint32) 0};
				GdkPixbuf* pixbuf;
				b64data = g_utf8_offset_to_pointer (self->priv->_icon, (glong) 7);
				len = 0;
				_tmp4 = NULL;
				data = (_tmp4 = g_base64_decode (b64data, &len), data_length1 = -1, data_size = data_length1, _tmp4);
				gdk_pixdata_deserialize (&pixdata, data_length1, data, &inner_error);
				if (inner_error != NULL) {
					data = (g_free (data), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
					g_clear_error (&inner_error);
					return;
				}
				pixbuf = gdk_pixbuf_from_pixdata (&pixdata, TRUE, &inner_error);
				if (inner_error != NULL) {
					data = (g_free (data), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
					g_clear_error (&inner_error);
					return;
				}
				gtk_image_set_from_pixbuf (gnomenu_menu_item_get_image (self), pixbuf);
				data = (g_free (data), NULL);
				(pixbuf == NULL) ? NULL : (pixbuf = (g_object_unref (pixbuf), NULL));
			} else {
				gboolean _tmp5;
				_tmp5 = FALSE;
				if (self->priv->_icon != NULL) {
					_tmp5 = g_str_has_prefix (self->priv->_icon, "custom:");
				} else {
					_tmp5 = FALSE;
				}
				if (_tmp5) {
				} else {
					/*Do nothing*/
					gtk_image_set_from_stock (gnomenu_menu_item_get_image (self), self->priv->_icon, GTK_ICON_SIZE_MENU);
				}
			}
		}
	}
}


static void gnomenu_menu_item_real_parent_set (GtkWidget* base, GtkWidget* old_parent) {
	GnomenuMenuItem * self;
	self = (GnomenuMenuItem*) base;
	if (gtk_widget_get_parent ((GtkWidget*) self) != NULL) {
		gnomenu_menu_item_update_label_text (self);
	}
}


static void gnomenu_menu_item_create_labels (GnomenuMenuItem* self) {
	g_return_if_fail (self != NULL);
	g_assert (gnomenu_item_type_has_label (self->priv->_item_type));
	g_object_set ((GtkWidget*) self->priv->_cached_label_widget, "visible", TRUE, NULL);
	g_object_set ((GtkWidget*) self->priv->_cached_label_widget, "sensitive", TRUE, NULL);
	gnomenu_menu_label_set_gravity (self->priv->_cached_label_widget, gnomenu_menu_item_get_gravity (self));
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->_cached_label_widget);
	gnomenu_menu_item_update_font (self);
	gnomenu_menu_item_update_label_underline (self);
}


static GnomenuMenuLabel* gnomenu_menu_item_get_label_widget (GnomenuMenuItem* self) {
	GtkWidget* _tmp0;
	GnomenuMenuLabel* label;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0 = NULL;
	label = (_tmp0 = gtk_bin_get_child ((GtkBin*) self), GNOMENU_IS_MENU_LABEL (_tmp0) ? ((GnomenuMenuLabel*) _tmp0) : NULL);
	return label;
}


static void gnomenu_menu_item_remove_child (GnomenuMenuItem* self) {
	GtkWidget* _tmp0;
	GtkWidget* child;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	child = (_tmp0 = gtk_bin_get_child ((GtkBin*) self), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	if (child != NULL) {
		gtk_container_remove ((GtkContainer*) self, child);
	}
	(child == NULL) ? NULL : (child = (g_object_unref (child), NULL));
}


static void gnomenu_menu_item_create_arrow (GnomenuMenuItem* self) {
	g_return_if_fail (self != NULL);
	g_assert (self->priv->_item_type == GNOMENU_ITEM_TYPE_ARROW);
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->_cached_arrow_widget);
	g_object_set ((GtkWidget*) self->priv->_cached_arrow_widget, "visible", TRUE, NULL);
	g_object_set ((GtkWidget*) self->priv->_cached_arrow_widget, "sensitive", TRUE, NULL);
	gnomenu_menu_item_update_arrow_type (self);
}


static void gnomenu_menu_item_update_arrow_type (GnomenuMenuItem* self) {
	GtkWidget* _tmp0;
	g_return_if_fail (self != NULL);
	if (self->priv->_item_type != GNOMENU_ITEM_TYPE_ARROW) {
		return;
	}
	_tmp0 = NULL;
	gtk_arrow_set ((_tmp0 = gtk_bin_get_child ((GtkBin*) self), GTK_IS_ARROW (_tmp0) ? ((GtkArrow*) _tmp0) : NULL), gnomenu_gravity_to_arrow_type (self->priv->_gravity), GTK_SHADOW_NONE);
}


GtkShadowType gnomenu_menu_item_item_state_to_shadow_type (GnomenuItemState state) {
	switch (state) {
		case GNOMENU_ITEM_STATE_TOGGLED:
		{
			return GTK_SHADOW_IN;
		}
		case GNOMENU_ITEM_STATE_UNTOGGLED:
		{
			return GTK_SHADOW_OUT;
		}
		default:
		{
			return GTK_SHADOW_ETCHED_IN;
		}
	}
}


gint gnomenu_menu_item_get_max_width_chars (GnomenuMenuItem* self) {
	g_return_val_if_fail (self != NULL, 0);
	return self->priv->_max_width_chars;
}


void gnomenu_menu_item_set_max_width_chars (GnomenuMenuItem* self, gint value) {
	g_return_if_fail (self != NULL);
	self->priv->_max_width_chars = value;
	gnomenu_menu_item_update_label_text (self);
	g_object_notify ((GObject *) self, "max-width-chars");
}


static GnomenuShell* gnomenu_menu_item_real_get_shell (GnomenuItem* base) {
	GnomenuMenuItem* self;
	GtkContainer* _tmp0;
	self = (GnomenuMenuItem*) base;
	_tmp0 = NULL;
	return (_tmp0 = gtk_widget_get_parent ((GtkWidget*) self), GNOMENU_IS_SHELL (_tmp0) ? ((GnomenuShell*) _tmp0) : NULL);
}


static GnomenuShell* gnomenu_menu_item_real_get_sub_shell (GnomenuItem* base) {
	GnomenuMenuItem* self;
	GnomenuMenu* _tmp0;
	self = (GnomenuMenuItem*) base;
	_tmp0 = NULL;
	return (_tmp0 = self->_submenu_cache, GNOMENU_IS_SHELL (_tmp0) ? ((GnomenuShell*) _tmp0) : NULL);
}


static gboolean gnomenu_menu_item_real_get_has_sub_shell (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_has_sub_shell;
}


static void gnomenu_menu_item_real_set_has_sub_shell (GnomenuItem* base, gboolean value) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	self->priv->_has_sub_shell = value;
	gnomenu_menu_item_maybe_connect_cached_submenu (self);
	g_object_notify ((GObject *) self, "has-sub-shell");
}


static gboolean gnomenu_menu_item_real_get_client_side_sub_shell (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_client_side_sub_shell;
}


static void gnomenu_menu_item_real_set_client_side_sub_shell (GnomenuItem* base, gboolean value) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	self->priv->_client_side_sub_shell = value;
	gnomenu_menu_item_maybe_connect_cached_submenu (self);
	g_object_notify ((GObject *) self, "client-side-sub-shell");
}


static const char* gnomenu_menu_item_real_get_item_id (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_id;
}


static void gnomenu_menu_item_real_set_item_id (GnomenuItem* base, const char* value) {
	GnomenuMenuItem* self;
	char* _tmp2;
	const char* _tmp1;
	self = (GnomenuMenuItem*) base;
	if (_vala_strcmp0 (self->priv->_id, value) == 0) {
		return;
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_id = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_id = (g_free (self->priv->_id), NULL), _tmp2);
	if (self->priv->_label == NULL) {
		gnomenu_menu_item_update_label_text (self);
	}
	g_object_notify ((GObject *) self, "item-id");
}


static const char* gnomenu_menu_item_real_get_item_label (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_label;
}


static void gnomenu_menu_item_real_set_item_label (GnomenuItem* base, const char* value) {
	GnomenuMenuItem* self;
	char* _tmp2;
	const char* _tmp1;
	self = (GnomenuMenuItem*) base;
	if (_vala_strcmp0 (self->priv->_label, value) == 0) {
		return;
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_label = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_label = (g_free (self->priv->_label), NULL), _tmp2);
	gnomenu_menu_item_update_label_text (self);
	g_object_notify ((GObject *) self, "item-label");
}


static const char* gnomenu_menu_item_real_get_item_icon (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_icon;
}


static void gnomenu_menu_item_real_set_item_icon (GnomenuItem* base, const char* value) {
	GnomenuMenuItem* self;
	char* _tmp2;
	const char* _tmp1;
	self = (GnomenuMenuItem*) base;
	if (_vala_strcmp0 (self->priv->_icon, value) == 0) {
		return;
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_icon = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_icon = (g_free (self->priv->_icon), NULL), _tmp2);
	gnomenu_menu_item_update_image (self);
	g_object_notify ((GObject *) self, "item-icon");
}


static const char* gnomenu_menu_item_real_get_item_accel_text (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_accel_text;
}


static void gnomenu_menu_item_real_set_item_accel_text (GnomenuItem* base, const char* value) {
	GnomenuMenuItem* self;
	char* _tmp2;
	const char* _tmp1;
	self = (GnomenuMenuItem*) base;
	if (_vala_strcmp0 (self->priv->_accel_text, value) == 0) {
		return;
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_accel_text = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_accel_text = (g_free (self->priv->_accel_text), NULL), _tmp2);
	gnomenu_menu_item_update_label_text (self);
	g_object_notify ((GObject *) self, "item-accel-text");
}


static const char* gnomenu_menu_item_real_get_item_font (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_font;
}


static void gnomenu_menu_item_real_set_item_font (GnomenuItem* base, const char* value) {
	GnomenuMenuItem* self;
	char* _tmp2;
	const char* _tmp1;
	self = (GnomenuMenuItem*) base;
	if (_vala_strcmp0 (self->priv->_font, value) == 0) {
		return;
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_font = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_font = (g_free (self->priv->_font), NULL), _tmp2);
	gnomenu_menu_item_update_font (self);
	g_object_notify ((GObject *) self, "item-font");
}


static GnomenuItemType gnomenu_menu_item_real_get_item_type (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_item_type;
}


static void gnomenu_menu_item_real_set_item_type (GnomenuItem* base, GnomenuItemType value) {
	GnomenuMenuItem* self;
	GnomenuItemType old_type;
	self = (GnomenuMenuItem*) base;
	if (value == self->priv->_item_type) {
		return;
	}
	old_type = self->priv->_item_type;
	self->priv->_item_type = value;
	switch (self->priv->_item_type) {
		case GNOMENU_ITEM_TYPE_SEPARATOR:
		{
			gnomenu_menu_item_remove_child (self);
			break;
		}
		case GNOMENU_ITEM_TYPE_NORMAL:
		case GNOMENU_ITEM_TYPE_IMAGE:
		case GNOMENU_ITEM_TYPE_CHECK:
		case GNOMENU_ITEM_TYPE_RADIO:
		{
			gboolean _tmp1;
			gboolean _tmp2;
			gboolean _tmp3;
			_tmp1 = FALSE;
			_tmp2 = FALSE;
			_tmp3 = FALSE;
			if (old_type != GNOMENU_ITEM_TYPE_NORMAL) {
				_tmp3 = old_type != GNOMENU_ITEM_TYPE_IMAGE;
			} else {
				_tmp3 = FALSE;
			}
			if (_tmp3) {
				_tmp2 = old_type != GNOMENU_ITEM_TYPE_RADIO;
			} else {
				_tmp2 = FALSE;
			}
			if (_tmp2) {
				_tmp1 = old_type != GNOMENU_ITEM_TYPE_CHECK;
			} else {
				_tmp1 = FALSE;
			}
			if (_tmp1) {
				gnomenu_menu_item_remove_child (self);
				gnomenu_menu_item_create_labels (self);
				gnomenu_menu_item_update_label_text (self);
			}
			break;
		}
		case GNOMENU_ITEM_TYPE_ARROW:
		{
			if (old_type != GNOMENU_ITEM_TYPE_ARROW) {
				gnomenu_menu_item_remove_child (self);
				gnomenu_menu_item_create_arrow (self);
				gnomenu_menu_item_update_arrow_type (self);
			}
			break;
		}
		case GNOMENU_ITEM_TYPE_ICON:
		{
			if (old_type != GNOMENU_ITEM_TYPE_ICON) {
				GtkImage* _tmp4;
				gnomenu_menu_item_remove_child (self);
				_tmp4 = NULL;
				self->priv->_icon_widget = (_tmp4 = g_object_ref_sink ((GtkImage*) gtk_image_new ()), (self->priv->_icon_widget == NULL) ? NULL : (self->priv->_icon_widget = (g_object_unref (self->priv->_icon_widget), NULL)), _tmp4);
				g_object_set ((GtkWidget*) self->priv->_icon_widget, "visible", TRUE, NULL);
				gtk_container_add ((GtkContainer*) self, (GtkWidget*) self->priv->_icon_widget);
				gnomenu_menu_item_update_image (self);
			}
			break;
		}
	}
	if (self->priv->_item_type == GNOMENU_ITEM_TYPE_IMAGE) {
		GtkImage* _tmp6;
		GtkImage* _tmp5;
		_tmp6 = NULL;
		_tmp5 = NULL;
		self->priv->_image_widget = (_tmp6 = (_tmp5 = self->priv->_cached_image_widget, (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (self->priv->_image_widget == NULL) ? NULL : (self->priv->_image_widget = (g_object_unref (self->priv->_image_widget), NULL)), _tmp6);
		g_object_set ((GtkWidget*) self->priv->_image_widget, "sensitive", FALSE, NULL);
		g_object_set ((GtkWidget*) self->priv->_image_widget, "sensitive", TRUE, NULL);
		gtk_widget_set_parent ((GtkWidget*) self->priv->_image_widget, (GtkWidget*) self);
		g_object_set ((GtkWidget*) self->priv->_image_widget, "visible", gnomenu_menu_item_get__show_image (self), NULL);
		gnomenu_menu_item_update_image (self);
	} else {
		if (self->priv->_image_widget != NULL) {
			GtkImage* _tmp7;
			gtk_widget_unparent ((GtkWidget*) self->priv->_image_widget);
			_tmp7 = NULL;
			self->priv->_image_widget = (_tmp7 = NULL, (self->priv->_image_widget == NULL) ? NULL : (self->priv->_image_widget = (g_object_unref (self->priv->_image_widget), NULL)), _tmp7);
		}
	}
	gtk_widget_queue_resize ((GtkWidget*) self);
	g_object_notify ((GObject *) self, "item-type");
}


static gboolean gnomenu_menu_item_real_get_item_use_underline (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_use_underline;
}


static void gnomenu_menu_item_real_set_item_use_underline (GnomenuItem* base, gboolean value) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	if (self->priv->_use_underline == value) {
		return;
	}
	self->priv->_use_underline = value;
	gnomenu_menu_item_update_label_underline (self);
	g_object_notify ((GObject *) self, "item-use-underline");
}


static GnomenuItemState gnomenu_menu_item_real_get_item_state (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_item_state;
}


static void gnomenu_menu_item_real_set_item_state (GnomenuItem* base, GnomenuItemState value) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	if (value == self->priv->_item_state) {
		return;
	}
	self->priv->_item_state = value;
	gtk_widget_queue_draw ((GtkWidget*) self);
	g_object_notify ((GObject *) self, "item-state");
}


static gboolean gnomenu_menu_item_real_get_item_visible (GnomenuItem* base) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	return self->priv->_item_visible;
}


static void gnomenu_menu_item_real_set_item_visible (GnomenuItem* base, gboolean value) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	self->priv->_item_visible = value;
	g_object_set ((GtkWidget*) self, "visible", value, NULL);
	g_object_notify ((GObject *) self, "item-visible");
}


static gboolean gnomenu_menu_item_real_get_item_sensitive (GnomenuItem* base) {
	GnomenuMenuItem* self;
	gboolean _tmp0;
	self = (GnomenuMenuItem*) base;
	return (g_object_get ((GtkWidget*) self, "sensitive", &_tmp0, NULL), _tmp0);
}


static void gnomenu_menu_item_real_set_item_sensitive (GnomenuItem* base, gboolean value) {
	GnomenuMenuItem* self;
	self = (GnomenuMenuItem*) base;
	g_object_set ((GtkWidget*) self, "sensitive", value, NULL);
	g_object_notify ((GObject *) self, "item-sensitive");
}


GnomenuGravity gnomenu_menu_item_get_gravity (GnomenuMenuItem* self) {
	g_return_val_if_fail (self != NULL, 0);
	return self->priv->_gravity;
}


void gnomenu_menu_item_set_gravity (GnomenuMenuItem* self, GnomenuGravity value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_gravity == value) {
		return;
	}
	self->priv->_gravity = value;
	if (gnomenu_item_type_has_label (self->priv->_item_type)) {
		gnomenu_menu_label_set_gravity (gnomenu_menu_item_get_label_widget (self), value);
	}
	gnomenu_menu_item_update_arrow_type (self);
	g_object_notify ((GObject *) self, "gravity");
}


gboolean gnomenu_menu_item_get_truncated (GnomenuMenuItem* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_truncated;
}


void gnomenu_menu_item_set_truncated (GnomenuMenuItem* self, gboolean value) {
	gboolean _tmp1;
	g_return_if_fail (self != NULL);
	self->priv->_truncated = value;
	_tmp1 = FALSE;
	if (self->priv->_item_visible) {
		_tmp1 = !gnomenu_menu_item_get_truncated (self);
	} else {
		_tmp1 = FALSE;
	}
	g_object_set ((GtkWidget*) self, "visible", _tmp1, NULL);
	g_object_notify ((GObject *) self, "truncated");
}


static gboolean gnomenu_menu_item_get__show_image (GnomenuMenuItem* self) {
	gboolean rt;
	g_return_val_if_fail (self != NULL, FALSE);
	rt = FALSE;
	g_object_get ((GObject*) gtk_widget_get_settings ((GtkWidget*) self), "gtk-menu-images", &rt, NULL, NULL);
	return rt;
}


GtkImage* gnomenu_menu_item_get_image (GnomenuMenuItem* self) {
	g_return_val_if_fail (self != NULL, NULL);
	if (self->priv->_item_type == GNOMENU_ITEM_TYPE_IMAGE) {
		return self->priv->_image_widget;
	}
	if (self->priv->_item_type == GNOMENU_ITEM_TYPE_ICON) {
		return self->priv->_icon_widget;
	}
	return NULL;
}


static GtkPackDirection gnomenu_menu_item_get_pack_direction (GnomenuMenuItem* self) {
	g_return_val_if_fail (self != NULL, 0);
	if (GTK_IS_MENU_BAR (gtk_widget_get_parent ((GtkWidget*) self))) {
		GtkContainer* _tmp0;
		_tmp0 = NULL;
		return gtk_menu_bar_get_child_pack_direction ((_tmp0 = gtk_widget_get_parent ((GtkWidget*) self), GTK_IS_MENU_BAR (_tmp0) ? ((GtkMenuBar*) _tmp0) : NULL));
	}
	return GTK_PACK_DIRECTION_LTR;
}


static GObject * gnomenu_menu_item_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GnomenuMenuItemClass * klass;
	GObjectClass * parent_class;
	GnomenuMenuItem * self;
	klass = GNOMENU_MENU_ITEM_CLASS (g_type_class_peek (GNOMENU_TYPE_MENU_ITEM));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GNOMENU_MENU_ITEM (obj);
	{
		GnomenuMenuLabel* _tmp0;
		GtkImage* _tmp1;
		GtkArrow* _tmp2;
		self->priv->disposed = FALSE;
		self->priv->_item_type = GNOMENU_ITEM_TYPE_NORMAL;
		_tmp0 = NULL;
		self->priv->_cached_label_widget = (_tmp0 = g_object_ref_sink (gnomenu_menu_label_new ()), (self->priv->_cached_label_widget == NULL) ? NULL : (self->priv->_cached_label_widget = (g_object_unref (self->priv->_cached_label_widget), NULL)), _tmp0);
		_tmp1 = NULL;
		self->priv->_cached_image_widget = (_tmp1 = g_object_ref_sink ((GtkImage*) gtk_image_new ()), (self->priv->_cached_image_widget == NULL) ? NULL : (self->priv->_cached_image_widget = (g_object_unref (self->priv->_cached_image_widget), NULL)), _tmp1);
		_tmp2 = NULL;
		self->priv->_cached_arrow_widget = (_tmp2 = g_object_ref_sink ((GtkArrow*) gtk_arrow_new (gnomenu_gravity_to_arrow_type (self->priv->_gravity), GTK_SHADOW_NONE)), (self->priv->_cached_arrow_widget == NULL) ? NULL : (self->priv->_cached_arrow_widget = (g_object_unref (self->priv->_cached_arrow_widget), NULL)), _tmp2);
		self->priv->_use_underline = TRUE;
		gnomenu_menu_item_create_labels (self);
	}
	return obj;
}


static void gnomenu_menu_item_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GnomenuMenuItem * self;
	gpointer boxed;
	self = GNOMENU_MENU_ITEM (object);
	switch (property_id) {
		case GNOMENU_MENU_ITEM_MAX_WIDTH_CHARS:
		g_value_set_int (value, gnomenu_menu_item_get_max_width_chars (self));
		break;
		case GNOMENU_MENU_ITEM_SHELL:
		g_value_set_object (value, gnomenu_item_get_shell ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_SUB_SHELL:
		g_value_set_object (value, gnomenu_item_get_sub_shell ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_HAS_SUB_SHELL:
		g_value_set_boolean (value, gnomenu_item_get_has_sub_shell ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_CLIENT_SIDE_SUB_SHELL:
		g_value_set_boolean (value, gnomenu_item_get_client_side_sub_shell ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_ID:
		g_value_set_string (value, gnomenu_item_get_item_id ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_LABEL:
		g_value_set_string (value, gnomenu_item_get_item_label ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_ICON:
		g_value_set_string (value, gnomenu_item_get_item_icon ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_ACCEL_TEXT:
		g_value_set_string (value, gnomenu_item_get_item_accel_text ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_FONT:
		g_value_set_string (value, gnomenu_item_get_item_font ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_TYPE:
		g_value_set_enum (value, gnomenu_item_get_item_type ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_USE_UNDERLINE:
		g_value_set_boolean (value, gnomenu_item_get_item_use_underline ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_STATE:
		g_value_set_enum (value, gnomenu_item_get_item_state ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_VISIBLE:
		g_value_set_boolean (value, gnomenu_item_get_item_visible ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_ITEM_SENSITIVE:
		g_value_set_boolean (value, gnomenu_item_get_item_sensitive ((GnomenuItem*) self));
		break;
		case GNOMENU_MENU_ITEM_GRAVITY:
		g_value_set_enum (value, gnomenu_menu_item_get_gravity (self));
		break;
		case GNOMENU_MENU_ITEM_TRUNCATED:
		g_value_set_boolean (value, gnomenu_menu_item_get_truncated (self));
		break;
		case GNOMENU_MENU_ITEM_IMAGE:
		g_value_set_object (value, gnomenu_menu_item_get_image (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gnomenu_menu_item_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GnomenuMenuItem * self;
	self = GNOMENU_MENU_ITEM (object);
	switch (property_id) {
		case GNOMENU_MENU_ITEM_MAX_WIDTH_CHARS:
		gnomenu_menu_item_set_max_width_chars (self, g_value_get_int (value));
		break;
		case GNOMENU_MENU_ITEM_HAS_SUB_SHELL:
		gnomenu_item_set_has_sub_shell ((GnomenuItem*) self, g_value_get_boolean (value));
		break;
		case GNOMENU_MENU_ITEM_CLIENT_SIDE_SUB_SHELL:
		gnomenu_item_set_client_side_sub_shell ((GnomenuItem*) self, g_value_get_boolean (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_ID:
		gnomenu_item_set_item_id ((GnomenuItem*) self, g_value_get_string (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_LABEL:
		gnomenu_item_set_item_label ((GnomenuItem*) self, g_value_get_string (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_ICON:
		gnomenu_item_set_item_icon ((GnomenuItem*) self, g_value_get_string (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_ACCEL_TEXT:
		gnomenu_item_set_item_accel_text ((GnomenuItem*) self, g_value_get_string (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_FONT:
		gnomenu_item_set_item_font ((GnomenuItem*) self, g_value_get_string (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_TYPE:
		gnomenu_item_set_item_type ((GnomenuItem*) self, g_value_get_enum (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_USE_UNDERLINE:
		gnomenu_item_set_item_use_underline ((GnomenuItem*) self, g_value_get_boolean (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_STATE:
		gnomenu_item_set_item_state ((GnomenuItem*) self, g_value_get_enum (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_VISIBLE:
		gnomenu_item_set_item_visible ((GnomenuItem*) self, g_value_get_boolean (value));
		break;
		case GNOMENU_MENU_ITEM_ITEM_SENSITIVE:
		gnomenu_item_set_item_sensitive ((GnomenuItem*) self, g_value_get_boolean (value));
		break;
		case GNOMENU_MENU_ITEM_GRAVITY:
		gnomenu_menu_item_set_gravity (self, g_value_get_enum (value));
		break;
		case GNOMENU_MENU_ITEM_TRUNCATED:
		gnomenu_menu_item_set_truncated (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gnomenu_menu_item_class_init (GnomenuMenuItemClass * klass) {
	gnomenu_menu_item_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GnomenuMenuItemPrivate));
	G_OBJECT_CLASS (klass)->get_property = gnomenu_menu_item_get_property;
	G_OBJECT_CLASS (klass)->set_property = gnomenu_menu_item_set_property;
	G_OBJECT_CLASS (klass)->constructor = gnomenu_menu_item_constructor;
	G_OBJECT_CLASS (klass)->finalize = gnomenu_menu_item_finalize;
	G_OBJECT_CLASS (klass)->dispose = gnomenu_menu_item_real_dispose;
	GTK_MENU_ITEM_CLASS (klass)->toggle_size_request = gnomenu_menu_item_real_toggle_size_request;
	GTK_WIDGET_CLASS (klass)->expose_event = gnomenu_menu_item_real_expose_event;
	GTK_CONTAINER_CLASS (klass)->forall = gnomenu_menu_item_real_forall;
	GTK_MENU_ITEM_CLASS (klass)->activate = gnomenu_menu_item_real_activate;
	GTK_WIDGET_CLASS (klass)->screen_changed = gnomenu_menu_item_real_screen_changed;
	GTK_WIDGET_CLASS (klass)->size_request = gnomenu_menu_item_real_size_request;
	GTK_WIDGET_CLASS (klass)->size_allocate = gnomenu_menu_item_real_size_allocate;
	GTK_WIDGET_CLASS (klass)->parent_set = gnomenu_menu_item_real_parent_set;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_MAX_WIDTH_CHARS, g_param_spec_int ("max-width-chars", "max-width-chars", "max-width-chars", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_SHELL, "shell");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_SUB_SHELL, "sub-shell");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_HAS_SUB_SHELL, "has-sub-shell");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_CLIENT_SIDE_SUB_SHELL, "client-side-sub-shell");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_ID, "item-id");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_LABEL, "item-label");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_ICON, "item-icon");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_ACCEL_TEXT, "item-accel-text");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_FONT, "item-font");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_TYPE, "item-type");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_USE_UNDERLINE, "item-use-underline");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_STATE, "item-state");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_VISIBLE, "item-visible");
	g_object_class_override_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_ITEM_SENSITIVE, "item-sensitive");
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_GRAVITY, g_param_spec_enum ("gravity", "gravity", "gravity", GNOMENU_TYPE_GRAVITY, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_TRUNCATED, g_param_spec_boolean ("truncated", "truncated", "truncated", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_MENU_ITEM_IMAGE, g_param_spec_object ("image", "image", "image", GTK_TYPE_IMAGE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	{
		GParamSpecInt* _tmp3;
		GtkImageMenuItem* item;
		_tmp3 = NULL;
		gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (klass), (GParamSpec*) (_tmp3 = g_param_spec_int ("indicator-size", _ ("Indicator Size"), _ ("Size of check or radio indicator"), 0, G_MAXINT, 13, G_PARAM_READABLE)));
		(_tmp3 == NULL) ? NULL : (_tmp3 = (g_param_spec_unref (_tmp3), NULL));
		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &gnomenu_menu_item_icon_width, &gnomenu_menu_item_icon_height);
		/*Load gtk-menu-images setting*/
		item = g_object_ref_sink ((GtkImageMenuItem*) gtk_image_menu_item_new ());
		(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
	}
}


static void gnomenu_menu_item_gnomenu_item_interface_init (GnomenuItemIface * iface) {
	gnomenu_menu_item_gnomenu_item_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_shell = gnomenu_menu_item_real_get_shell;
	iface->get_sub_shell = gnomenu_menu_item_real_get_sub_shell;
	iface->get_has_sub_shell = gnomenu_menu_item_real_get_has_sub_shell;
	iface->set_has_sub_shell = gnomenu_menu_item_real_set_has_sub_shell;
	iface->get_client_side_sub_shell = gnomenu_menu_item_real_get_client_side_sub_shell;
	iface->set_client_side_sub_shell = gnomenu_menu_item_real_set_client_side_sub_shell;
	iface->get_item_id = gnomenu_menu_item_real_get_item_id;
	iface->set_item_id = gnomenu_menu_item_real_set_item_id;
	iface->get_item_label = gnomenu_menu_item_real_get_item_label;
	iface->set_item_label = gnomenu_menu_item_real_set_item_label;
	iface->get_item_icon = gnomenu_menu_item_real_get_item_icon;
	iface->set_item_icon = gnomenu_menu_item_real_set_item_icon;
	iface->get_item_accel_text = gnomenu_menu_item_real_get_item_accel_text;
	iface->set_item_accel_text = gnomenu_menu_item_real_set_item_accel_text;
	iface->get_item_font = gnomenu_menu_item_real_get_item_font;
	iface->set_item_font = gnomenu_menu_item_real_set_item_font;
	iface->get_item_type = gnomenu_menu_item_real_get_item_type;
	iface->set_item_type = gnomenu_menu_item_real_set_item_type;
	iface->get_item_use_underline = gnomenu_menu_item_real_get_item_use_underline;
	iface->set_item_use_underline = gnomenu_menu_item_real_set_item_use_underline;
	iface->get_item_state = gnomenu_menu_item_real_get_item_state;
	iface->set_item_state = gnomenu_menu_item_real_set_item_state;
	iface->get_item_visible = gnomenu_menu_item_real_get_item_visible;
	iface->set_item_visible = gnomenu_menu_item_real_set_item_visible;
	iface->get_item_sensitive = gnomenu_menu_item_real_get_item_sensitive;
	iface->set_item_sensitive = gnomenu_menu_item_real_set_item_sensitive;
}


static void gnomenu_menu_item_instance_init (GnomenuMenuItem * self) {
	self->priv = GNOMENU_MENU_ITEM_GET_PRIVATE (self);
	self->priv->_max_width_chars = 30;
	self->_submenu_cache = g_object_ref_sink (gnomenu_menu_new ());
	self->priv->_has_sub_shell = FALSE;
	self->priv->_client_side_sub_shell = FALSE;
	self->priv->_use_underline = FALSE;
}


static void gnomenu_menu_item_finalize (GObject* obj) {
	GnomenuMenuItem * self;
	self = GNOMENU_MENU_ITEM (obj);
	(self->_submenu_cache == NULL) ? NULL : (self->_submenu_cache = (g_object_unref (self->_submenu_cache), NULL));
	self->priv->_font = (g_free (self->priv->_font), NULL);
	self->priv->_label = (g_free (self->priv->_label), NULL);
	self->priv->_icon = (g_free (self->priv->_icon), NULL);
	self->priv->_accel_text = (g_free (self->priv->_accel_text), NULL);
	self->priv->_id = (g_free (self->priv->_id), NULL);
	(self->priv->_cached_image_widget == NULL) ? NULL : (self->priv->_cached_image_widget = (g_object_unref (self->priv->_cached_image_widget), NULL));
	(self->priv->_cached_arrow_widget == NULL) ? NULL : (self->priv->_cached_arrow_widget = (g_object_unref (self->priv->_cached_arrow_widget), NULL));
	(self->priv->_cached_label_widget == NULL) ? NULL : (self->priv->_cached_label_widget = (g_object_unref (self->priv->_cached_label_widget), NULL));
	(self->priv->_image_widget == NULL) ? NULL : (self->priv->_image_widget = (g_object_unref (self->priv->_image_widget), NULL));
	(self->priv->_icon_widget == NULL) ? NULL : (self->priv->_icon_widget = (g_object_unref (self->priv->_icon_widget), NULL));
	G_OBJECT_CLASS (gnomenu_menu_item_parent_class)->finalize (obj);
}


GType gnomenu_menu_item_get_type (void) {
	static GType gnomenu_menu_item_type_id = 0;
	if (gnomenu_menu_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuMenuItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gnomenu_menu_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GnomenuMenuItem), 0, (GInstanceInitFunc) gnomenu_menu_item_instance_init, NULL };
		static const GInterfaceInfo gnomenu_item_info = { (GInterfaceInitFunc) gnomenu_menu_item_gnomenu_item_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gnomenu_menu_item_type_id = g_type_register_static (GTK_TYPE_MENU_ITEM, "GnomenuMenuItem", &g_define_type_info, 0);
		g_type_add_interface_static (gnomenu_menu_item_type_id, GNOMENU_TYPE_ITEM, &gnomenu_item_info);
	}
	return gnomenu_menu_item_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




