
#include <libgnomenu/serializer.h>
#include <libgnomenu/interface-shell.h>
#include <libgnomenu/interface-item.h>
#include <gobject/gvaluecollector.h>




struct _GnomenuSerializerPrivate {
	GString* sb;
	gint level;
	gboolean _newline;
	gboolean pretty_print;
};

#define GNOMENU_SERIALIZER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOMENU_TYPE_SERIALIZER, GnomenuSerializerPrivate))
enum  {
	GNOMENU_SERIALIZER_DUMMY_PROPERTY
};
static GnomenuSerializer* gnomenu_serializer_construct (GType object_type);
static GnomenuSerializer* gnomenu_serializer_new (void);
static void gnomenu_serializer_visit (GnomenuSerializer* self, GObject* node);
static void gnomenu_serializer_visit_shell (GnomenuSerializer* self, GnomenuShell* shell);
static void gnomenu_serializer_visit_item (GnomenuSerializer* self, GnomenuItem* item);
static void gnomenu_serializer_visit_item_attributes (GnomenuSerializer* self, GnomenuItem* item);
static void gnomenu_serializer_indent (GnomenuSerializer* self);
static void gnomenu_serializer_newline (GnomenuSerializer* self);
static gpointer gnomenu_serializer_parent_class = NULL;
static void gnomenu_serializer_finalize (GnomenuSerializer* obj);



char* gnomenu_serializer_to_string (GObject* obj, gboolean pretty_print) {
	GnomenuSerializer* s;
	const char* _tmp0;
	char* _tmp1;
	g_return_val_if_fail (obj != NULL, NULL);
	s = gnomenu_serializer_new ();
	s->priv->pretty_print = pretty_print;
	gnomenu_serializer_visit (s, obj);
	_tmp0 = NULL;
	_tmp1 = NULL;
	return (_tmp1 = (_tmp0 = s->priv->sb->str, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), (s == NULL) ? NULL : (s = (gnomenu_serializer_unref (s), NULL)), _tmp1);
}


static GnomenuSerializer* gnomenu_serializer_construct (GType object_type) {
	GnomenuSerializer* self;
	GString* _tmp0;
	self = (GnomenuSerializer*) g_type_create_instance (object_type);
	_tmp0 = NULL;
	self->priv->sb = (_tmp0 = g_string_new (""), (self->priv->sb == NULL) ? NULL : (self->priv->sb = (g_string_free (self->priv->sb, TRUE), NULL)), _tmp0);
	return self;
}


static GnomenuSerializer* gnomenu_serializer_new (void) {
	return gnomenu_serializer_construct (GNOMENU_TYPE_SERIALIZER);
}


static void gnomenu_serializer_visit (GnomenuSerializer* self, GObject* node) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	if (GNOMENU_IS_SHELL (node)) {
		GObject* _tmp0;
		_tmp0 = NULL;
		gnomenu_serializer_visit_shell (self, (_tmp0 = node, GNOMENU_IS_SHELL (_tmp0) ? ((GnomenuShell*) _tmp0) : NULL));
	}
	if (GNOMENU_IS_ITEM (node)) {
		GObject* _tmp1;
		_tmp1 = NULL;
		gnomenu_serializer_visit_item (self, (_tmp1 = node, GNOMENU_IS_ITEM (_tmp1) ? ((GnomenuItem*) _tmp1) : NULL));
	}
}


static void gnomenu_serializer_visit_shell (GnomenuSerializer* self, GnomenuShell* shell) {
	gint i;
	g_return_if_fail (self != NULL);
	g_return_if_fail (shell != NULL);
	i = 0;
	if (gnomenu_shell_get_length (shell) > 0) {
		gnomenu_serializer_indent (self);
		g_string_append_printf (self->priv->sb, "<menu>");
		gnomenu_serializer_newline (self);
		self->priv->level++;
		for (i = 0; i < gnomenu_shell_get_length (shell); i++) {
			GnomenuItem* _tmp0;
			_tmp0 = NULL;
			gnomenu_serializer_visit (self, (GObject*) (_tmp0 = gnomenu_shell_get_item (shell, i)));
			(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
		}
		self->priv->level--;
		gnomenu_serializer_indent (self);
		g_string_append_printf (self->priv->sb, "</menu>");
		gnomenu_serializer_newline (self);
	} else {
		gnomenu_serializer_indent (self);
		g_string_append_printf (self->priv->sb, "<menu/>");
		gnomenu_serializer_newline (self);
	}
}


static void gnomenu_serializer_visit_item (GnomenuSerializer* self, GnomenuItem* item) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (item != NULL);
	if (gnomenu_item_get_has_sub_shell (item)) {
		gnomenu_serializer_indent (self);
		g_string_append_printf (self->priv->sb, "<item");
		gnomenu_serializer_visit_item_attributes (self, item);
		g_string_append_c (self->priv->sb, '>');
		gnomenu_serializer_newline (self);
		self->priv->level++;
		gnomenu_serializer_visit_shell (self, gnomenu_item_get_sub_shell (item));
		self->priv->level--;
		gnomenu_serializer_indent (self);
		g_string_append_printf (self->priv->sb, "</item>");
		gnomenu_serializer_newline (self);
	} else {
		gnomenu_serializer_indent (self);
		g_string_append_printf (self->priv->sb, "<item");
		gnomenu_serializer_visit_item_attributes (self, item);
		g_string_append (self->priv->sb, "/>");
		gnomenu_serializer_newline (self);
	}
}


static void gnomenu_serializer_visit_item_attributes (GnomenuSerializer* self, GnomenuItem* item) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (item != NULL);
	if (gnomenu_item_get_item_label (item) != NULL) {
		char* _tmp0;
		_tmp0 = NULL;
		g_string_append (self->priv->sb, _tmp0 = g_markup_printf_escaped (" label=\"%s\"", gnomenu_item_get_item_label (item)));
		_tmp0 = (g_free (_tmp0), NULL);
	}
	if (gnomenu_item_get_item_type (item) != GNOMENU_ITEM_TYPE_NORMAL) {
		char* _tmp1;
		_tmp1 = NULL;
		g_string_append (self->priv->sb, _tmp1 = g_markup_printf_escaped (" type=\"%s\"", gnomenu_item_type_to_string (gnomenu_item_get_item_type (item))));
		_tmp1 = (g_free (_tmp1), NULL);
	}
	if (gnomenu_item_get_item_state (item) != GNOMENU_ITEM_STATE_TRISTATE) {
		char* _tmp2;
		_tmp2 = NULL;
		g_string_append (self->priv->sb, _tmp2 = g_markup_printf_escaped (" state=\"%s\"", gnomenu_item_state_to_string (gnomenu_item_get_item_state (item))));
		_tmp2 = (g_free (_tmp2), NULL);
	}
	if (gnomenu_item_get_item_font (item) != NULL) {
		char* _tmp3;
		_tmp3 = NULL;
		g_string_append (self->priv->sb, _tmp3 = g_markup_printf_escaped (" font=\"%s\"", gnomenu_item_get_item_font (item)));
		_tmp3 = (g_free (_tmp3), NULL);
	}
	if (gnomenu_item_get_item_id (item) != NULL) {
		char* _tmp4;
		_tmp4 = NULL;
		g_string_append (self->priv->sb, _tmp4 = g_markup_printf_escaped (" id=\"%s\"", gnomenu_item_get_item_id (item)));
		_tmp4 = (g_free (_tmp4), NULL);
	}
	if (gnomenu_item_get_client_side_sub_shell (item)) {
		char* _tmp5;
		_tmp5 = NULL;
		g_string_append (self->priv->sb, _tmp5 = g_markup_printf_escaped (" client-side=\"1\""));
		_tmp5 = (g_free (_tmp5), NULL);
	}
	if (gnomenu_item_get_item_accel_text (item) != NULL) {
		char* _tmp6;
		_tmp6 = NULL;
		g_string_append (self->priv->sb, _tmp6 = g_markup_printf_escaped (" accel=\"%s\"", gnomenu_item_get_item_accel_text (item)));
		_tmp6 = (g_free (_tmp6), NULL);
	}
	if (gnomenu_item_get_item_visible (item) == FALSE) {
		g_string_append (self->priv->sb, " visible=\"false\"");
	}
	if (gnomenu_item_get_item_sensitive (item) == FALSE) {
		g_string_append (self->priv->sb, " sensitive=\"false\"");
	}
	if (gnomenu_item_get_item_use_underline (item) == FALSE) {
		g_string_append (self->priv->sb, " underline=\"false\"");
	}
	if (gnomenu_item_get_item_icon (item) != NULL) {
		char* _tmp7;
		_tmp7 = NULL;
		g_string_append (self->priv->sb, _tmp7 = g_markup_printf_escaped (" icon=\"%s\"", gnomenu_item_get_item_icon (item)));
		_tmp7 = (g_free (_tmp7), NULL);
	}
}


static void gnomenu_serializer_indent (GnomenuSerializer* self) {
	g_return_if_fail (self != NULL);
	if (!self->priv->pretty_print) {
		return;
	}
	if (self->priv->_newline) {
		{
			gint i;
			i = 0;
			for (; i < self->priv->level; i++) {
				g_string_append_c (self->priv->sb, ' ');
			}
		}
		self->priv->_newline = FALSE;
	}
}


static void gnomenu_serializer_newline (GnomenuSerializer* self) {
	g_return_if_fail (self != NULL);
	if (!self->priv->pretty_print) {
		return;
	}
	g_string_append_c (self->priv->sb, '\n');
	self->priv->_newline = TRUE;
}


static void gnomenu_value_serializer_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void gnomenu_value_serializer_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		gnomenu_serializer_unref (value->data[0].v_pointer);
	}
}


static void gnomenu_value_serializer_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = gnomenu_serializer_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer gnomenu_value_serializer_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* gnomenu_value_serializer_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		GnomenuSerializer* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = gnomenu_serializer_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* gnomenu_value_serializer_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	GnomenuSerializer** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = gnomenu_serializer_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* gnomenu_param_spec_serializer (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	GnomenuParamSpecSerializer* spec;
	g_return_val_if_fail (g_type_is_a (object_type, GNOMENU_TYPE_SERIALIZER), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer gnomenu_value_get_serializer (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, GNOMENU_TYPE_SERIALIZER), NULL);
	return value->data[0].v_pointer;
}


void gnomenu_value_set_serializer (GValue* value, gpointer v_object) {
	GnomenuSerializer* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, GNOMENU_TYPE_SERIALIZER));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, GNOMENU_TYPE_SERIALIZER));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		gnomenu_serializer_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		gnomenu_serializer_unref (old);
	}
}


static void gnomenu_serializer_class_init (GnomenuSerializerClass * klass) {
	gnomenu_serializer_parent_class = g_type_class_peek_parent (klass);
	GNOMENU_SERIALIZER_CLASS (klass)->finalize = gnomenu_serializer_finalize;
	g_type_class_add_private (klass, sizeof (GnomenuSerializerPrivate));
}


static void gnomenu_serializer_instance_init (GnomenuSerializer * self) {
	self->priv = GNOMENU_SERIALIZER_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void gnomenu_serializer_finalize (GnomenuSerializer* obj) {
	GnomenuSerializer * self;
	self = GNOMENU_SERIALIZER (obj);
	(self->priv->sb == NULL) ? NULL : (self->priv->sb = (g_string_free (self->priv->sb, TRUE), NULL));
}


GType gnomenu_serializer_get_type (void) {
	static GType gnomenu_serializer_type_id = 0;
	if (gnomenu_serializer_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { gnomenu_value_serializer_init, gnomenu_value_serializer_free_value, gnomenu_value_serializer_copy_value, gnomenu_value_serializer_peek_pointer, "p", gnomenu_value_serializer_collect_value, "p", gnomenu_value_serializer_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuSerializerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gnomenu_serializer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GnomenuSerializer), 0, (GInstanceInitFunc) gnomenu_serializer_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		gnomenu_serializer_type_id = g_type_register_fundamental (g_type_fundamental_next (), "GnomenuSerializer", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return gnomenu_serializer_type_id;
}


gpointer gnomenu_serializer_ref (gpointer instance) {
	GnomenuSerializer* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void gnomenu_serializer_unref (gpointer instance) {
	GnomenuSerializer* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		GNOMENU_SERIALIZER_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




