
#include <libgnomenu/window.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <gdk/gdkx.h>
#include <libgnomenu/gnomenu.h>
#include <libgnomenu/keygrab.h>




struct _GnomenuWindowPrivate {
	GdkWindow* _window;
	GtkWidget* key_widget;
	gboolean disposed;
	char* _menu_context;
};

#define GNOMENU_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOMENU_TYPE_WINDOW, GnomenuWindowPrivate))
enum  {
	GNOMENU_WINDOW_DUMMY_PROPERTY,
	GNOMENU_WINDOW_WINDOW,
	GNOMENU_WINDOW_MENU_CONTEXT
};
static void gnomenu_window_real_dispose (GObject* base);
static GdkFilterReturn gnomenu_window_event_filter (GnomenuWindow* self, GdkXEvent* xevent, const GdkEvent* gdk_ev);
static GdkFilterReturn _gnomenu_window_event_filter_gdk_filter_func (GdkXEvent* xevent, const GdkEvent* event, gpointer self);
static void __lambda0 (GnomenuWindow* t, const char* prop, GnomenuWindow* self);
static void ___lambda0_gnomenu_window_property_notify_event (GnomenuWindow* _sender, const char* name, gpointer self);
static GObject * gnomenu_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gnomenu_window_parent_class = NULL;
static void gnomenu_window_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



GnomenuWindow* gnomenu_window_construct (GType object_type, GdkWindow* window) {
	GParameter * __params;
	GParameter * __params_it;
	GnomenuWindow * self;
	g_return_val_if_fail (window != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "window";
	g_value_init (&__params_it->value, GDK_TYPE_WINDOW);
	g_value_set_object (&__params_it->value, window);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


GnomenuWindow* gnomenu_window_new (GdkWindow* window) {
	return gnomenu_window_construct (GNOMENU_TYPE_WINDOW, window);
}


GnomenuWindow* gnomenu_window_foreign_new (gulong xid) {
	GdkWindow* gdk_window;
	GdkWindow* _tmp1;
	GdkWindow* _tmp0;
	GnomenuWindow* _tmp4;
	gdk_window = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	gdk_window = (_tmp1 = (_tmp0 = gdk_window_lookup ((GdkNativeWindow) xid), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (gdk_window == NULL) ? NULL : (gdk_window = (g_object_unref (gdk_window), NULL)), _tmp1);
	if (gdk_window == NULL) {
		GdkWindow* _tmp3;
		GdkWindow* _tmp2;
		_tmp3 = NULL;
		_tmp2 = NULL;
		gdk_window = (_tmp3 = (_tmp2 = gdk_window_foreign_new ((GdkNativeWindow) xid), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (gdk_window == NULL) ? NULL : (gdk_window = (g_object_unref (gdk_window), NULL)), _tmp3);
	}
	_tmp4 = NULL;
	return (_tmp4 = gnomenu_window_new (gdk_window), (gdk_window == NULL) ? NULL : (gdk_window = (g_object_unref (gdk_window), NULL)), _tmp4);
}


/*must be a toplevel */
void gnomenu_window_set_key_widget (GnomenuWindow* self, GtkWidget* widget) {
	GtkWidget* _tmp1;
	GtkWidget* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->key_widget = (_tmp1 = (_tmp0 = widget, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->key_widget == NULL) ? NULL : (self->priv->key_widget = (g_object_unref (self->priv->key_widget), NULL)), _tmp1);
}


static void gnomenu_window_real_dispose (GObject* base) {
	GnomenuWindow * self;
	self = (GnomenuWindow*) base;
	if (!self->priv->disposed) {
		self->priv->disposed = TRUE;
		gnomenu_window_set_window (self, NULL);
	}
}


static GdkFilterReturn gnomenu_window_event_filter (GnomenuWindow* self, GdkXEvent* xevent, const GdkEvent* gdk_ev) {
	XAnyEvent* event;
	GdkEvent ev;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (xevent != NULL, 0);
	if (self->priv->disposed) {
		g_critical ("window.vala:68: event_filter invoked on a disposed window");
		return GDK_FILTER_CONTINUE;
	}
	event = (XAnyEvent*) xevent;
	ev = (*gdk_ev);
	/* copy the gdk_event*/
	switch (event->type) {
		case PropertyNotify:
		{
			char* _tmp1;
			ev.type = GDK_PROPERTY_NOTIFY;
			ev.property.atom = gdk_x11_xatom_to_atom (((XPropertyEvent*) event)->atom);
			ev.property.time = ((XPropertyEvent*) event)->time;
			ev.property.state = (guint) ((XPropertyEvent*) event)->state;
			_tmp1 = NULL;
			g_signal_emit_by_name (self, "property-notify-event", _tmp1 = gdk_atom_name (ev.property.atom));
			_tmp1 = (g_free (_tmp1), NULL);
			break;
		}
		case KeyPress:
		{
			gboolean _tmp2;
			_tmp2 = FALSE;
			if (self->priv->key_widget != NULL) {
				_tmp2 = self->priv->key_widget->window != NULL;
			} else {
				_tmp2 = FALSE;
			}
			if (_tmp2) {
				GdkWindow* _tmp3;
				GdkWindow* gwin;
				Window xwin;
				Display* xd;
				/* Send a Fake key press event to the key widget
				 * if it is realized. */
				_tmp3 = NULL;
				gwin = (_tmp3 = self->priv->key_widget->window, (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
				xwin = GDK_WINDOW_XWINDOW (gwin);
				xd = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
				event->window = xwin;
				XSendEvent (xd, xwin, FALSE, (glong) 0, event);
				(gwin == NULL) ? NULL : (gwin = (g_object_unref (gwin), NULL));
			}
			break;
		}
	}
	return GDK_FILTER_CONTINUE;
}


char* gnomenu_window_get (GnomenuWindow* self, const char* property_name) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (property_name != NULL, NULL);
	return gnomenu_window_get_by_atom (self, gdk_atom_intern (property_name, FALSE));
}


void gnomenu_window_set (GnomenuWindow* self, const char* property_name, const char* value) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (property_name != NULL);
	gnomenu_window_set_by_atom (self, gdk_atom_intern (property_name, FALSE), value);
}


char* gnomenu_window_get_by_atom (GnomenuWindow* self, GdkAtom atom) {
	char* context;
	GdkAtom actual_type = {0};
	GdkAtom type;
	gint actual_format;
	gint actual_length;
	g_return_val_if_fail (self != NULL, NULL);
	context = NULL;
	type = gdk_atom_intern ("STRING", FALSE);
	actual_format = 0;
	actual_length = 0;
	gdk_property_get (gnomenu_window_get_window (self), atom, type, (gulong) 0, (gulong) G_MAXLONG, FALSE, &actual_type, &actual_format, &actual_length, &context);
	return context;
}


void gnomenu_window_set_by_atom (GnomenuWindow* self, GdkAtom atom, const char* value) {
	g_return_if_fail (self != NULL);
	if (value != NULL) {
		GdkAtom type;
		type = gdk_atom_intern ("STRING", FALSE);
		gdk_property_change (gnomenu_window_get_window (self), atom, type, 8, GDK_PROP_MODE_REPLACE, value, ((gint) strlen (value)) + 1);
	} else {
		gdk_property_delete (gnomenu_window_get_window (self), atom);
	}
}


/**
 * emitted when a menu item is activated
 */
void gnomenu_window_emit_menu_event (GnomenuWindow* self, const char* path) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (path != NULL);
	gnomenu_window_set (self, GNOMENU_NET_GLOBALMENU_MENU_EVENT, path);
}


/**
 * globally grab a key to this window.
 *
 * return false if failed.
 */
gboolean gnomenu_window_grab_key (GnomenuWindow* self, guint keyval, GdkModifierType state) {
	g_return_val_if_fail (self != NULL, FALSE);
	return gnomenu_grab_key (gnomenu_window_get_window (self), keyval, state);
}


/**
 * globally ungrab a grabbed a key to this window
 * return false if failed.
 */
gboolean gnomenu_window_ungrab_key (GnomenuWindow* self, guint keyval, GdkModifierType state) {
	g_return_val_if_fail (self != NULL, FALSE);
	return gnomenu_ungrab_key (gnomenu_window_get_window (self), keyval, state);
}


GdkWindow* gnomenu_window_get_window (GnomenuWindow* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_window;
}


static GdkFilterReturn _gnomenu_window_event_filter_gdk_filter_func (GdkXEvent* xevent, const GdkEvent* event, gpointer self) {
	return gnomenu_window_event_filter (self, xevent, event);
}


void gnomenu_window_set_window (GnomenuWindow* self, GdkWindow* value) {
	GdkWindow* _tmp2;
	GdkWindow* _tmp1;
	g_return_if_fail (self != NULL);
	if (self->priv->_window != NULL) {
		gdk_window_remove_filter (self->priv->_window, _gnomenu_window_event_filter_gdk_filter_func, self);
	}
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_window = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL)), _tmp2);
	if (self->priv->_window != NULL) {
		gdk_window_add_filter (self->priv->_window, _gnomenu_window_event_filter_gdk_filter_func, self);
	}
	g_object_notify ((GObject *) self, "window");
}


const char* gnomenu_window_get_menu_context (GnomenuWindow* self) {
	char* _tmp0;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0 = NULL;
	self->priv->_menu_context = (_tmp0 = gnomenu_window_get (self, GNOMENU_NET_GLOBALMENU_MENU_CONTEXT), self->priv->_menu_context = (g_free (self->priv->_menu_context), NULL), _tmp0);
	return self->priv->_menu_context;
}


void gnomenu_window_set_menu_context (GnomenuWindow* self, const char* value) {
	g_return_if_fail (self != NULL);
	gnomenu_window_set (self, GNOMENU_NET_GLOBALMENU_MENU_CONTEXT, value);
	g_object_notify ((GObject *) self, "menu-context");
}


static void __lambda0 (GnomenuWindow* t, const char* prop, GnomenuWindow* self) {
	g_return_if_fail (t != NULL);
	g_return_if_fail (prop != NULL);
	if (_vala_strcmp0 (prop, GNOMENU_NET_GLOBALMENU_MENU_CONTEXT) == 0) {
		g_signal_emit_by_name (self, "menu-context-changed");
	}
	if (_vala_strcmp0 (prop, GNOMENU_NET_GLOBALMENU_MENU_EVENT) == 0) {
		char* _tmp0;
		_tmp0 = NULL;
		g_signal_emit_by_name (self, "menu-event", _tmp0 = gnomenu_window_get (self, GNOMENU_NET_GLOBALMENU_MENU_EVENT));
		_tmp0 = (g_free (_tmp0), NULL);
	}
}


static void ___lambda0_gnomenu_window_property_notify_event (GnomenuWindow* _sender, const char* name, gpointer self) {
	__lambda0 (_sender, name, self);
}


static GObject * gnomenu_window_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GnomenuWindowClass * klass;
	GObjectClass * parent_class;
	GnomenuWindow * self;
	klass = GNOMENU_WINDOW_CLASS (g_type_class_peek (GNOMENU_TYPE_WINDOW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GNOMENU_WINDOW (obj);
	{
		g_signal_connect (self, "property-notify-event", (GCallback) ___lambda0_gnomenu_window_property_notify_event, self);
	}
	return obj;
}


static void gnomenu_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GnomenuWindow * self;
	gpointer boxed;
	self = GNOMENU_WINDOW (object);
	switch (property_id) {
		case GNOMENU_WINDOW_WINDOW:
		g_value_set_object (value, gnomenu_window_get_window (self));
		break;
		case GNOMENU_WINDOW_MENU_CONTEXT:
		g_value_set_string (value, gnomenu_window_get_menu_context (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gnomenu_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GnomenuWindow * self;
	self = GNOMENU_WINDOW (object);
	switch (property_id) {
		case GNOMENU_WINDOW_WINDOW:
		gnomenu_window_set_window (self, g_value_get_object (value));
		break;
		case GNOMENU_WINDOW_MENU_CONTEXT:
		gnomenu_window_set_menu_context (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gnomenu_window_class_init (GnomenuWindowClass * klass) {
	gnomenu_window_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GnomenuWindowPrivate));
	G_OBJECT_CLASS (klass)->get_property = gnomenu_window_get_property;
	G_OBJECT_CLASS (klass)->set_property = gnomenu_window_set_property;
	G_OBJECT_CLASS (klass)->constructor = gnomenu_window_constructor;
	G_OBJECT_CLASS (klass)->finalize = gnomenu_window_finalize;
	G_OBJECT_CLASS (klass)->dispose = gnomenu_window_real_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_WINDOW_WINDOW, g_param_spec_object ("window", "window", "window", GDK_TYPE_WINDOW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_WINDOW_MENU_CONTEXT, g_param_spec_string ("menu-context", "menu-context", "menu-context", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_signal_new ("property_notify_event", GNOMENU_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
	g_signal_new ("menu_context_changed", GNOMENU_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("menu_event", GNOMENU_TYPE_WINDOW, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
}


static void gnomenu_window_instance_init (GnomenuWindow * self) {
	self->priv = GNOMENU_WINDOW_GET_PRIVATE (self);
	self->priv->disposed = FALSE;
}


static void gnomenu_window_finalize (GObject* obj) {
	GnomenuWindow * self;
	self = GNOMENU_WINDOW (obj);
	(self->priv->_window == NULL) ? NULL : (self->priv->_window = (g_object_unref (self->priv->_window), NULL));
	(self->priv->key_widget == NULL) ? NULL : (self->priv->key_widget = (g_object_unref (self->priv->key_widget), NULL));
	self->priv->_menu_context = (g_free (self->priv->_menu_context), NULL);
	G_OBJECT_CLASS (gnomenu_window_parent_class)->finalize (obj);
}


GType gnomenu_window_get_type (void) {
	static GType gnomenu_window_type_id = 0;
	if (gnomenu_window_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gnomenu_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GnomenuWindow), 0, (GInstanceInitFunc) gnomenu_window_instance_init, NULL };
		gnomenu_window_type_id = g_type_register_static (G_TYPE_OBJECT, "GnomenuWindow", &g_define_type_info, 0);
	}
	return gnomenu_window_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




